Package: ggseg
Title: Plotting Tool for Brain Atlases
Version: 2.2.1
Authors@R: c(
    person("Athanasia Mo", "Mowinckel", , "a.m.mowinckel@psykologi.uio.no", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-5756-0223")),
    person("Didac", "Vidal-Piñeiro", , "d.v.pineiro@psykologi.uio.no", role = "aut",
           comment = c(ORCID = "0000-0001-9997-9156")),
    person("Ramiro", "Magno", , "ramiro.magno@gmail.com", role = "ctb",
           comment = c(ORCID = "0000-0001-5226-3441")),
    person("Center for Lifespan Changes in Brain and Cognition, University of Oslo, Norway",
           role = "cph")
  )
Description: Provides a 'ggplot2' geom and position for visualizing brain
    region data on cortical, subcortical, and white matter tract atlases.
    Brain atlas geometries are stored as polygon data, enabling
    seamless integration with the 'ggplot2' ecosystem including faceting,
    custom scales, and themes.  Mowinckel & Vidal-Piñeiro (2020)
    <doi:10.1177/2515245920928009>.
License: MIT + file LICENSE
URL: https://ggsegverse.github.io/ggseg/,
        https://github.com/ggsegverse/ggseg
BugReports: https://github.com/ggsegverse/ggseg/issues
Depends: R (>= 3.3)
Imports: cli, dplyr (>= 1.0.0), ggplot2 (>= 3.3), ggseg.formats (>=
        0.0.3), grid, lifecycle, rlang, tidyr (>= 1.0.0), utils
Suggests: covr, devtools, here, knitr, rmarkdown, sf, spelling,
        testthat (>= 3.0.0), vdiffr, withr
VignetteBuilder: knitr
Config/Needs/website: ggsegverse/ggseg.docs
Config/testthat/edition: 3
Config/testthat/parallel: true
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-07-02 18:07:44 UTC; athanasm
Author: Athanasia Mo Mowinckel [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-5756-0223>),
  Didac Vidal-Piñeiro [aut] (ORCID:
    <https://orcid.org/0000-0001-9997-9156>),
  Ramiro Magno [ctb] (ORCID: <https://orcid.org/0000-0001-5226-3441>),
  Center for Lifespan Changes in Brain and Cognition, University of Oslo,
    Norway [cph]
Maintainer: Athanasia Mo Mowinckel <a.m.mowinckel@psykologi.uio.no>
Repository: CRAN
Date/Publication: 2026-07-03 02:50:06 UTC
Built: R 4.6.0; ; 2026-07-03 04:17:00 UTC; unix

# Generated by roxygen2: do not edit by hand

export(GeomBrain)
export(annotate_brain)
export(aseg)
export(brain_join)
export(dk)
export(geom_brain)
export(ggseg)
export(position_brain)
export(reposition_brain)
export(scale_brain)
export(scale_brain2)
export(scale_brain_manual)
export(scale_color_brain)
export(scale_color_brain2)
export(scale_color_brain_manual)
export(scale_colour_brain)
export(scale_colour_brain2)
export(scale_colour_brain_manual)
export(scale_continous_brain)
export(scale_fill_brain)
export(scale_fill_brain2)
export(scale_fill_brain_manual)
export(scale_labs_brain)
export(scale_x_brain)
export(scale_y_brain)
export(theme_brain)
export(theme_brain2)
export(theme_custombrain)
export(theme_darkbrain)
export(tracula)
importFrom(dplyr,as_tibble)
importFrom(dplyr,full_join)
importFrom(dplyr,group_by)
importFrom(dplyr,is.grouped_df)
importFrom(dplyr,mutate)
importFrom(dplyr,row_number)
importFrom(dplyr,summarise)
importFrom(dplyr,ungroup)
importFrom(ggplot2,.pt)
importFrom(ggplot2,Geom)
importFrom(ggplot2,GeomPolygon)
importFrom(ggplot2,Position)
importFrom(ggplot2,aes)
importFrom(ggplot2,alpha)
importFrom(ggplot2,coord_sf)
importFrom(ggplot2,draw_key_polygon)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,element_text)
importFrom(ggplot2,ggproto)
importFrom(ggplot2,ggproto_parent)
importFrom(ggplot2,labs)
importFrom(ggplot2,layer)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_colour_manual)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,theme)
importFrom(ggseg.formats,aseg)
importFrom(ggseg.formats,atlas_palette)
importFrom(ggseg.formats,dk)
importFrom(ggseg.formats,tracula)
importFrom(sf,st_as_sf)
importFrom(sf,st_combine)
importFrom(sf,st_coordinates)
importFrom(tidyr,nest)
importFrom(tidyr,unnest)
importFrom(utils,capture.output)
importFrom(utils,getFromNamespace)

# ggseg <img src="man/figures/logo.png" align="right" alt="" width="138.5" />

<!-- badges: start -->

[![CRAN
status](https://www.r-pkg.org/badges/version/ggseg.png)](https://CRAN.R-project.org/package=ggseg)
[![R-CMD-check](https://github.com/ggsegverse/ggseg/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/ggsegverse/ggseg/actions/workflows/R-CMD-check.yaml)
[![code-quality](https://github.com/ggsegverse/ggseg/actions/workflows/code-quality.yaml/badge.svg)](https://github.com/ggsegverse/ggseg/actions/workflows/code-quality.yaml)
[![Coverage
Status](https://raw.githubusercontent.com/ggsegverse/ggseg/coverage/badges/coverage.svg)](https://github.com/ggsegverse/ggseg/actions/workflows/test-coverage.yaml)
[![downloads](https://cranlogs.r-pkg.org/badges/last-month/ggseg?color=blue)](https://r-pkg.org/pkg/ggseg)
[![Lifecycle:
stable](https://img.shields.io/badge/lifecycle-stable-brightgreen.svg)](https://lifecycle.r-lib.org/articles/stages.html)
[![pkgcheck](https://github.com/ggsegverse/ggseg/workflows/pkgcheck/badge.svg)](https://github.com/ggsegverse/ggseg/actions?query=workflow%3Apkgcheck)
<!-- badges: end -->

Neuroimaging analyses produce region-level results – cortical thickness,
p-values, network assignments – that need to end up on a brain figure.
ggseg stores brain atlas geometries as simple features and plots them as
ggplot2 layers, so you get publication-ready brain figures with the same
code you’d use for any other ggplot.

Mowinckel & Vidal-Piñeiro (2020). [*Visualization of Brain Statistics
With R Packages ggseg and
ggseg3d.*](https://doi.org/10.1177/2515245920928009) Advances in Methods
and Practices in Psychological Science.

## Installation

Install from CRAN:

``` r
install.packages("ggseg")
```

Or get the development version from the [ggsegverse
r-universe](https://ggsegverse.r-universe.dev):

``` r
options(repos = c(
  ggsegverse = "https://ggsegverse.r-universe.dev",
  CRAN = "https://cloud.r-project.org"
))
install.packages("ggseg")
```

## Quick start

``` r
library(ggseg)
library(ggplot2)
```

### Built-in atlases

ggseg ships with three atlases: `dk` (Desikan-Killiany cortical
parcellation), `aseg` (automatic subcortical segmentation), and
`tracula` (white matter tracts). `plot()` gives you a quick overview:

``` r
plot(dk())
plot(aseg())
```

<div id="fig-atlases-1">

<img src="man/figures/README-fig-atlases-1.png" style="width:100.0%" />

Figure 1: Overview of the dk and aseg built-in brain atlases.

</div>

<div id="fig-atlases-2">

<img src="man/figures/README-fig-atlases-2.png" style="width:100.0%" />

Figure 2: Overview of the dk and aseg built-in brain atlases.

</div>

### Plotting your own data

Pass a data frame to `ggplot()` with a column that matches the atlas
(typically `region` or `label`). `geom_brain()` handles the join:

``` r
library(dplyr)

some_data <- tibble(
  region = rep(
    c(
      "transverse temporal",
      "insula",
      "precentral",
      "superior parietal"
    ),
    2
  ),
  p = sample(seq(0, .5, .001), 8),
  groups = c(rep("g1", 4), rep("g2", 4))
)

ggplot(some_data) +
  geom_brain(
    atlas = dk(),
    position = position_brain(hemi ~ view),
    aes(fill = p)
  ) +
  facet_wrap(~groups) +
  scale_fill_viridis_c(option = "cividis", direction = -1) +
  theme_void()
```

<div id="fig-external-data">

<img src="man/figures/README-fig-external-data-1.png"
style="width:100.0%" />

Figure 3: Brain plot coloured by external data, faceted by group.

</div>

## More atlases

Many additional atlases are available through the [ggsegverse
r-universe](https://ggsegverse.r-universe.dev):

``` r
install.packages("ggsegYeo2011", repos = "https://ggsegverse.r-universe.dev")
```

## Learn more

The [package website](https://ggsegverse.github.io/ggseg/) has vignettes
covering external data, view positioning, the `geom_sf()` workflow, and
reading FreeSurfer stats files.

## Funding

This tool is partly funded by:

**EU Horizon 2020 Grant:** Healthy minds 0-100 years: Optimising the use
of European brain imaging cohorts (Lifebrain). Grant agreement number:
732592.

library(dplyr, quietly = TRUE, warn.conflicts = FALSE)
library(tidyr, quietly = TRUE, warn.conflicts = FALSE)
library(ggplot2, quietly = TRUE, warn.conflicts = FALSE)
library(vdiffr, quietly = TRUE, warn.conflicts = FALSE)
library(ggseg.formats, quietly = TRUE, warn.conflicts = FALSE)
set.seed(1234)

# A cortical atlas with 3D vertices but no 2D geometry, built through the public
# ggseg.formats constructors. Used to exercise the "no 2D geometry" error paths
# without reaching into atlas internals.
atlas_without_2d_geometry <- function() {
  vertices <- data.frame(label = "lh_frontal")
  vertices$vertices <- list(1:3)
  ggseg_atlas(
    atlas = "empty",
    type = "cortical",
    core = data.frame(hemi = "left", region = "frontal", label = "lh_frontal"),
    data = ggseg_data_cortical(vertices = vertices)
  )
}

describe("geom_brain_sf (deprecated sf path)", {
  it("works with basic atlas", {
    skip_if_not_installed("sf")
    withr::local_options(lifecycle_verbosity = "quiet")
    p <- ggplot() + geom_brain_sf(atlas = dk())
    expect_s3_class(p, "gg")
  })

  it("warns when deprecated side argument is used", {
    skip_if_not_installed("sf")
    withr::local_options(lifecycle_verbosity = "quiet")
    expect_warning(
      ggplot() + geom_brain_sf(atlas = dk(), side = "lateral"),
      "side.*deprecated"
    )
  })

  it("uses side value for view when view is NULL", {
    skip_if_not_installed("sf")
    withr::local_options(lifecycle_verbosity = "quiet")
    expect_warning(
      p <- ggplot() + geom_brain_sf(atlas = dk(), side = "lateral"),
      "side.*deprecated"
    )
    expect_s3_class(p, "gg")
  })

  it("filters by hemisphere", {
    skip_if_not_installed("sf")
    withr::local_options(lifecycle_verbosity = "quiet")
    p <- ggplot() + geom_brain_sf(atlas = dk(), hemi = "left")
    expect_s3_class(p, "gg")
    built <- ggplot_build(p)
    expect_true(all(built$plot$layers[[1]]$geom_params$hemi == "left"))
  })

  it("filters by view", {
    skip_if_not_installed("sf")
    withr::local_options(lifecycle_verbosity = "quiet")
    p <- ggplot() + geom_brain_sf(atlas = dk(), view = "lateral")
    expect_s3_class(p, "gg")
  })

  it("works with position_brain", {
    skip_if_not_installed("sf")
    withr::local_options(lifecycle_verbosity = "quiet")
    p <- ggplot() +
      geom_brain_sf(atlas = dk(), position = position_brain_sf(hemi ~ view))
    expect_s3_class(p, "gg")
  })

  it("works with aesthetic mapping", {
    skip_if_not_installed("sf")
    withr::local_options(lifecycle_verbosity = "quiet")
    p <- ggplot() + geom_brain_sf(atlas = dk(), mapping = aes(fill = region))
    expect_s3_class(p, "gg")
  })

  it("works with user data", {
    skip_if_not_installed("sf")
    withr::local_options(lifecycle_verbosity = "quiet")
    some_data <- tibble(
      region = c("transverse temporal", "insula"),
      p = c(0.1, 0.5)
    )
    p <- ggplot(some_data) +
      geom_brain_sf(atlas = dk(), mapping = aes(fill = p))
    expect_s3_class(p, "gg")
    expect_message(
      built <- ggplot_build(p),
      "Merging"
    )
    expect_gt(nrow(built$data[[1]]), 0)
  })

  it("works with show.legend FALSE", {
    skip_if_not_installed("sf")
    withr::local_options(lifecycle_verbosity = "quiet")
    p <- ggplot() + geom_brain_sf(atlas = dk(), show.legend = FALSE)
    expect_s3_class(p, "gg")
  })

  it("works with inherit.aes FALSE", {
    skip_if_not_installed("sf")
    withr::local_options(lifecycle_verbosity = "quiet")
    some_data <- tibble(region = "insula", p = 0.3)
    p <- ggplot(some_data, aes(fill = p)) +
      geom_brain_sf(atlas = dk(), inherit.aes = FALSE)
    expect_message(
      built <- ggplot_build(p),
      "Merging"
    )
    expect_s3_class(p, "gg")
  })

  it("passes additional arguments to geom_sf", {
    skip_if_not_installed("sf")
    withr::local_options(lifecycle_verbosity = "quiet")
    p <- ggplot() +
      geom_brain_sf(atlas = dk(), colour = "black", size = 0.5)
    expect_s3_class(p, "gg")
  })

  it("works with aseg atlas", {
    skip_if_not_installed("sf")
    withr::local_options(lifecycle_verbosity = "quiet")
    p <- ggplot() + geom_brain_sf(atlas = aseg())
    expect_s3_class(p, "gg")
  })
})

describe("geom_brain faceting", {
  some_data <- tibble(
    region = rep(
      c("transverse temporal", "insula", "precentral", "superior parietal"),
      2
    ),
    p = seq(0.1, 0.8, by = 0.1),
    group = c(rep("A", 4), rep("B", 4))
  )

  it("facet_wrap works without group_by", {
    skip_if_not_installed("sf")
    withr::local_options(lifecycle_verbosity = "quiet")
    p <- ggplot(some_data) +
      geom_brain_sf(atlas = dk(), mapping = aes(fill = p)) +
      facet_wrap(~group)
    expect_message(
      built <- ggplot_build(p),
      "Merging"
    )
    panels <- unique(built$data[[1]]$PANEL)
    expect_length(panels, 2)
  })

  it("each facet panel has complete atlas rows", {
    skip_if_not_installed("sf")
    withr::local_options(lifecycle_verbosity = "quiet")
    p <- ggplot(some_data) +
      geom_brain_sf(atlas = dk(), mapping = aes(fill = p)) +
      facet_wrap(~group)
    expect_message(
      built <- ggplot_build(p),
      "Merging"
    )
    atlas_rows <- nrow(as.data.frame(dk()))
    rows_per_panel <- tapply(
      built$data[[1]]$PANEL,
      built$data[[1]]$PANEL,
      length
    )
    expect_true(all(rows_per_panel == atlas_rows))
  })

  it("explicit group_by still works with facet_wrap", {
    skip_if_not_installed("sf")
    withr::local_options(lifecycle_verbosity = "quiet")
    p <- some_data |>
      group_by(group) |>
      ggplot() +
      geom_brain_sf(atlas = dk(), mapping = aes(fill = p)) +
      facet_wrap(~group)
    expect_message(
      built <- ggplot_build(p),
      "Merging"
    )
    panels <- unique(built$data[[1]]$PANEL)
    expect_length(panels, 2)
  })

  it("facet_grid works without group_by", {
    skip_if_not_installed("sf")
    withr::local_options(lifecycle_verbosity = "quiet")
    p <- ggplot(some_data) +
      geom_brain_sf(atlas = dk(), mapping = aes(fill = p)) +
      facet_grid(rows = vars(group))
    expect_message(
      built <- ggplot_build(p),
      "Merging"
    )
    panels <- unique(built$data[[1]]$PANEL)
    expect_length(panels, 2)
  })

  it("ignores atlas columns in facet vars", {
    skip_if_not_installed("sf")
    withr::local_options(lifecycle_verbosity = "quiet")
    data_with_hemi <- tibble(
      region = c("transverse temporal", "insula"),
      p = c(0.1, 0.5),
      hemi = c("left", "right")
    )
    p <- ggplot(data_with_hemi) +
      geom_brain_sf(atlas = dk(), mapping = aes(fill = p)) +
      facet_wrap(~hemi)
    expect_message(
      built <- ggplot_build(p),
      "Merging"
    )
    expect_s3_class(p, "gg")
  })
})

describe("LayerBrain", {
  it("errors when no atlas is supplied", {
    skip_if_not_installed("sf")
    withr::local_options(lifecycle_verbosity = "quiet")
    expect_error(
      ggplot_build(
        ggplot() +
          layer_brain(
            geom = GeomBrain,
            stat = "sf",
            position = position_brain_sf(),
            params = list(na.rm = FALSE, atlas = NULL)
          ) +
          coord_sf()
      ),
      "No atlas supplied"
    )
  })

  it("errors when atlas has no data", {
    skip_if_not_installed("sf")
    withr::local_options(lifecycle_verbosity = "quiet")
    empty_atlas <- atlas_without_2d_geometry()
    expect_error(
      ggplot_build(ggplot() + geom_brain_sf(atlas = empty_atlas)),
      "no data|no 2D geometry"
    )
  })

  it("errors on invalid hemisphere", {
    skip_if_not_installed("sf")
    withr::local_options(lifecycle_verbosity = "quiet")
    expect_error(
      ggplot_build(ggplot() + geom_brain_sf(atlas = dk(), hemi = "top")),
      "Invalid hemisphere"
    )
  })

  it("errors on invalid view", {
    skip_if_not_installed("sf")
    withr::local_options(lifecycle_verbosity = "quiet")
    expect_error(
      ggplot_build(ggplot() + geom_brain_sf(atlas = dk(), view = "top")),
      "Invalid view"
    )
  })

  it("warns on unmatched region data", {
    skip_if_not_installed("sf")
    withr::local_options(lifecycle_verbosity = "quiet")
    bad_data <- tibble(
      region = "not a real region",
      p = 0.5
    )
    expect_warning(
      expect_warning(
        expect_message(
          ggplot_build(
            ggplot(bad_data) +
              geom_brain_sf(atlas = dk(), mapping = aes(fill = p))
          ),
          "Merging atlas and data"
        ),
        "not merged properly"
      ),
      "not merged"
    )
  })

  it("renders atlas without user data (waiver)", {
    skip_if_not_installed("sf")
    withr::local_options(lifecycle_verbosity = "quiet")
    p <- ggplot() + geom_brain_sf(atlas = dk())
    built <- ggplot_build(p)
    atlas_rows <- nrow(as.data.frame(dk()))
    expect_identical(nrow(built$data[[1]]), atlas_rows)
  })

  it("auto-maps geometry, hemi, view, type, fill, label", {
    skip_if_not_installed("sf")
    withr::local_options(lifecycle_verbosity = "quiet")
    p <- ggplot() + geom_brain_sf(atlas = dk())
    built <- ggplot_build(p)
    expect_true("fill" %in% names(built$data[[1]]))
  })
})

describe("GeomBrain", {
  it("exists as ggproto object", {
    skip_if_not_installed("sf")
    withr::local_options(lifecycle_verbosity = "quiet")
    expect_s3_class(GeomBrain, "Geom")
  })

  it("has default aesthetics", {
    skip_if_not_installed("sf")
    withr::local_options(lifecycle_verbosity = "quiet")
    expect_true("default_aes" %in% names(GeomBrain))
    defaults <- GeomBrain$default_aes
    expect_true("linetype" %in% names(defaults))
    expect_true("stroke" %in% names(defaults))
  })

  it("has draw_panel method", {
    skip_if_not_installed("sf")
    withr::local_options(lifecycle_verbosity = "quiet")
    expect_true("draw_panel" %in% names(GeomBrain))
  })

  it("has draw_key method", {
    skip_if_not_installed("sf")
    withr::local_options(lifecycle_verbosity = "quiet")
    expect_true("draw_key" %in% names(GeomBrain))
  })
})

describe("brain_grob", {
  it("creates grob from transformed coord data", {
    skip_if_not_installed("sf")
    withr::local_options(lifecycle_verbosity = "quiet")
    p <- ggplot() + geom_brain_sf(atlas = dk())
    built <- ggplot_build(p)
    gt <- ggplot_gtable(built)
    expect_s3_class(gt, "gtable")
  })
})

describe("position_brain visual", {
  it("dk default horizontal", {
    expect_doppelganger(
      "dk default horizontal",
      ggplot() + geom_brain(atlas = dk(), show.legend = FALSE)
    )
  })

  it("dk hemi ~ view", {
    expect_doppelganger(
      "dk hemi ~ view",
      ggplot() +
        geom_brain(
          atlas = dk(),
          position = position_brain(hemi ~ view),
          show.legend = FALSE
        )
    )
  })

  it("dk view ~ hemi", {
    expect_doppelganger(
      "dk view ~ hemi",
      ggplot() +
        geom_brain(
          atlas = dk(),
          position = position_brain(view ~ hemi),
          show.legend = FALSE
        )
    )
  })

  it("dk hemi + view ~ .", {
    expect_doppelganger(
      "dk rows hemi+view",
      ggplot() +
        geom_brain(
          atlas = dk(),
          position = position_brain(hemi + view ~ .),
          show.legend = FALSE
        )
    )
  })

  it("dk . ~ hemi + view", {
    expect_doppelganger(
      "dk cols hemi+view",
      ggplot() +
        geom_brain(
          atlas = dk(),
          position = position_brain(. ~ hemi + view),
          show.legend = FALSE
        )
    )
  })

  it("dk vertical", {
    expect_doppelganger(
      "dk vertical",
      ggplot() +
        geom_brain(
          atlas = dk(),
          position = position_brain("vertical"),
          show.legend = FALSE
        )
    )
  })

  it("dk character view order", {
    expect_doppelganger(
      "dk custom view order",
      ggplot() +
        geom_brain(
          atlas = dk(),
          position = position_brain(c(
            "right lateral",
            "right medial",
            "left lateral",
            "left medial"
          )),
          show.legend = FALSE
        )
    )
  })

  it("aseg default horizontal", {
    expect_doppelganger(
      "aseg default horizontal",
      ggplot() + geom_brain(atlas = aseg(), show.legend = FALSE)
    )
  })

  it("aseg vertical", {
    expect_doppelganger(
      "aseg vertical",
      ggplot() +
        geom_brain(
          atlas = aseg(),
          position = position_brain("vertical"),
          show.legend = FALSE
        )
    )
  })

  it("aseg nrow 2", {
    expect_doppelganger(
      "aseg nrow 2",
      ggplot() +
        geom_brain(
          atlas = aseg(),
          position = position_brain(nrow = 2),
          show.legend = FALSE
        )
    )
  })

  it("aseg ncol 3", {
    expect_doppelganger(
      "aseg ncol 3",
      ggplot() +
        geom_brain(
          atlas = aseg(),
          position = position_brain(ncol = 3),
          show.legend = FALSE
        )
    )
  })

  it("aseg type ~ .", {
    expect_doppelganger(
      "aseg type rows",
      ggplot() +
        geom_brain(
          atlas = aseg(),
          position = position_brain(type ~ .),
          show.legend = FALSE
        )
    )
  })

  it("tracula default horizontal", {
    expect_doppelganger(
      "tracula default horizontal",
      ggplot() + geom_brain(atlas = tracula(), show.legend = FALSE)
    )
  })

  it("tracula vertical", {
    expect_doppelganger(
      "tracula vertical",
      ggplot() +
        geom_brain(
          atlas = tracula(),
          position = position_brain("vertical"),
          show.legend = FALSE
        )
    )
  })

  it("tracula nrow 2", {
    expect_doppelganger(
      "tracula nrow 2",
      ggplot() +
        geom_brain(
          atlas = tracula(),
          position = position_brain(nrow = 2),
          show.legend = FALSE
        )
    )
  })

  it("tracula ncol 3", {
    expect_doppelganger(
      "tracula ncol 3",
      ggplot() +
        geom_brain(
          atlas = tracula(),
          position = position_brain(ncol = 3),
          show.legend = FALSE
        )
    )
  })

  it("tracula type ~ .", {
    expect_doppelganger(
      "tracula type rows",
      ggplot() +
        geom_brain(
          atlas = tracula(),
          position = position_brain(type ~ .),
          show.legend = FALSE
        )
    )
  })
})

#' Ensure the sf package is installed
#'
#' Internal guard used by the deprecated sf code path. Since the
#' sf-optional milestone moved sf to Suggests, callers without sf
#' installed should get a clear error pointing to the polygon default
#' (`geom_brain()`, `position_brain()`, `annotate_brain()`).
#'
#' @param what Character describing the calling function or operation,
#'   used in the error message.
#' @return Invisible `TRUE` if sf is installed; aborts otherwise.
#' @keywords internal
#' @noRd
require_sf <- function(what) {
  if (!has_sf()) {
    cli::cli_abort(c(
      "{what} requires the {.pkg sf} package, which is not installed.",
      "i" = paste0(
        "Install with {.run install.packages(\"sf\")}, or use the ",
        "polygon default ({.fn geom_brain}, {.fn position_brain}, ",
        "{.fn annotate_brain})."
      )
    ))
  }
  invisible(TRUE)
}


#' Test whether sf is available without raising
#'
#' @return Logical.
#' @keywords internal
#' @noRd
has_sf <- function() {
  requireNamespace("sf", quietly = TRUE)
}

describe("require_sf", {
  it("aborts with install guidance when sf is unavailable", {
    local_mocked_bindings(has_sf = function() FALSE)
    expect_error(require_sf("foo()"), "requires the.+sf.+package")
    expect_error(require_sf("foo()"), "install.packages")
  })

  it("returns invisibly when sf is available", {
    local_mocked_bindings(has_sf = function() TRUE)
    expect_invisible(require_sf("foo()"))
    expect_true(require_sf("foo()"))
  })
})

describe("sf-dependent entry points fail gracefully without sf", {
  it("reposition_brain() aborts via require_sf when sf is unavailable", {
    local_mocked_bindings(has_sf = function() FALSE)
    expect_error(
      reposition_brain(dk(), hemi ~ view),
      "reposition_brain.+requires the.+sf.+package"
    )
  })

  it("brain_join() with an sf-geometry data.frame aborts via require_sf", {
    local_mocked_bindings(has_sf = function() FALSE)
    # A plain data.frame carrying a `geometry` column reaches the st_as_sf
    # branch directly (as.data.frame() is a no-op), so the guard there fires.
    df <- data.frame(atlas = "x", region = "insula", geometry = NA)
    expect_error(
      suppressMessages(brain_join(data.frame(region = "insula", p = 1), df)),
      "requires the.+sf.+package"
    )
  })
})

#' Plot brain atlas regions
#'
#' Colour brain regions by your own values. Give `geom_brain()` an atlas like
#' `dk()` and a data frame, and it matches your values to the right regions and
#' lays out the brain views for you. No data? It just draws the atlas.
#'
#' @param mapping Set of aesthetic mappings created by [ggplot2::aes()].
#' @param data A data.frame containing variables to map. If `NULL`, the atlas
#'   is plotted without user data. Group it with [dplyr::group_by()] to facet.
#' @param atlas A `ggseg_atlas` object (e.g. `dk()`, `aseg()`, `tracula()`).
#' @param hemi Character vector of hemispheres to include (e.g. `"left"`,
#'   `"right"`). Defaults to all hemispheres in the atlas.
#' @param view Character vector of views to include, as recorded in the atlas
#'   data. For cortical atlases: `"lateral"`, `"medial"`. For subcortical/tract
#'   atlases: slice identifiers like `"axial_3"`. Defaults to all views.
#' @param position Position adjustment, either as a string or the result of
#'   a call to [position_brain()].
#' @param context Keep the rest of the brain as a soft grey backdrop (`TRUE`,
#'   the default), or show only the regions you're plotting (`FALSE`).
#' @param show.legend Logical. Should this layer be included in the legends?
#' @param inherit.aes Logical. If `FALSE`, overrides the default aesthetics
#'   rather than combining with them.
#' @param ... Additional arguments passed to [ggplot2::geom_polygon()].
#'
#' @return A list of ggplot2 layer and coord objects.
#' @rdname ggbrain
#' @export
#'
#' @examples
#' library(ggplot2)
#'
#' ggplot() +
#'   geom_brain(atlas = dk())
geom_brain <- function(
  mapping = aes(),
  data = NULL,
  atlas,
  hemi = NULL,
  view = NULL,
  position = position_brain(),
  context = TRUE,
  show.legend = NA,
  inherit.aes = TRUE,
  ...
) {
  dots <- list(...)
  if ("side" %in% names(dots)) {
    cli::cli_warn(c(
      "The {.arg side} argument is deprecated.",
      "i" = "Use {.arg view} instead. Your value has been passed to view."
    ))
    if (is.null(view)) {
      view <- dots$side
    }
    dots$side <- NULL
  }

  do.call(
    geom_brain_polygon,
    c(
      list(
        mapping = mapping,
        data = data,
        atlas = atlas,
        hemi = hemi,
        view = view,
        position = position,
        context = context,
        show.legend = show.legend,
        inherit.aes = inherit.aes
      ),
      dots
    )
  )
}

#' Deprecated sf brain geom
#'
#' @description
#' `r lifecycle::badge("deprecated")`
#'
#' The sf rendering path is deprecated. `geom_brain_sf()` renders an atlas via
#' [ggplot2::geom_sf()] and [coord_sf()][ggplot2::coord_sf]. For new code, use
#' [geom_brain()] (the polygon default), or convert the atlas with
#' `as_sf_atlas()` and use [ggplot2::geom_sf()] directly for the full sf
#' toolkit (labels, other sf layers).
#'
#' @inheritParams geom_brain
#' @return A list of ggplot2 layer and coord objects.
#' @export
#' @keywords internal
#' @importFrom ggplot2 aes coord_sf scale_fill_manual
geom_brain_sf <- function(
  mapping = aes(),
  data = NULL,
  atlas,
  hemi = NULL,
  view = NULL,
  position = NULL,
  show.legend = NA,
  inherit.aes = TRUE,
  ...
) {
  lifecycle::deprecate_warn(
    "2.2.0",
    "geom_brain_sf()",
    details = paste(
      "Use `geom_brain()` for the polygon default, or `as_sf_atlas()` with",
      "`ggplot2::geom_sf()` for an sf workflow."
    )
  )
  require_sf("geom_brain_sf()")
  if (is.null(position)) {
    position <- make_position_brain_sf()
  }
  dots <- list(...)
  if ("side" %in% names(dots)) {
    cli::cli_warn(c(
      "The {.arg side} argument is deprecated.",
      "i" = "Use {.arg view} instead. Your value has been passed to view."
    ))
    if (is.null(view)) {
      view <- dots$side
    }
    dots$side <- NULL
  }

  result <- list(
    layer_brain(
      geom = GeomBrain,
      data = data,
      mapping = mapping,
      stat = "sf",
      position = position,
      show.legend = show.legend,
      inherit.aes = inherit.aes,
      params = c(
        list(na.rm = FALSE, atlas = atlas, hemi = hemi, view = view),
        dots
      )
    ),
    coord_sf(default = TRUE, clip = "off")
  )

  has_fill_aes <- "fill" %in% names(mapping)
  if (!is.null(atlas$palette) && !has_fill_aes) {
    result <- c(
      result,
      list(
        scale_fill_manual(values = atlas$palette, na.value = "grey")
      )
    )
  }

  result
}


#' @section GeomBrain ggproto:
#' `GeomBrain` is a [ggplot2::Geom] ggproto object that handles rendering
#' of brain atlas polygons. It is used internally by [geom_brain()] and
#' should not typically be called directly.
#'
#' @export
#' @rdname ggbrain
#' @usage NULL
#' @format NULL
#' @importFrom ggplot2 Geom aes ggproto draw_key_polygon
GeomBrain <- ggproto(
  "GeomBrain",
  Geom,
  default_aes = aes(
    shape = NULL,
    colour = NULL,
    fill = NULL,
    size = NULL,
    linetype = 1,
    alpha = NA,
    stroke = 0.5
  ),
  draw_panel = function(
    data,
    atlas,
    hemi,
    view,
    panel_params,
    coord,
    legend = NULL,
    lineend = "butt",
    linejoin = "round",
    linemitre = 10,
    na.rm = TRUE
  ) {
    if (!inherits(coord, "CoordSf")) {
      cli::cli_abort("{.fn geom_brain} must be used with {.fn coord_sf}.")
    }

    coord <- coord$transform(data, panel_params)
    brain_grob(
      coord,
      lineend = lineend,
      linejoin = linejoin,
      linemitre = linemitre,
      na.rm = na.rm
    )
  },
  draw_key = function(data, params, size) {
    draw_key_polygon(data, params, size)
  }
)


#' Build a grid grob from transformed brain coordinates
#'
#' Adapted from `ggplot2::sf_grob`. Converts transformed coordinate
#' data into a grid graphical object with polygon fill and stroke.
#'
#' @param x Data.frame of transformed coordinates with columns
#'   `geometry`, `alpha`, `colour`, `fill`, `size`, and `linetype`.
#' @param lineend Line end style passed to [grid::gpar()].
#' @param linejoin Line join style passed to [grid::gpar()].
#' @param linemitre Line mitre limit passed to [grid::gpar()].
#' @param na.rm If `TRUE`, silently remove missing values.
#'
#' @return A grid [grob][grid::grid.grob] object.
#' @keywords internal
#' @importFrom ggplot2 GeomPolygon alpha .pt
#' @noRd
brain_grob <- function(
  x,
  lineend = "butt",
  linejoin = "round",
  linemitre = 10,
  na.rm = TRUE
) {
  # nolint: object_name_linter
  defaults <- modify_list(
    GeomPolygon$default_aes,
    list(colour = "grey35", size = 0.2)
  )

  alpha <- if (!is.null(x$alpha)) x$alpha else defaults$alpha
  col <- if (!is.null(x$colour)) x$colour else defaults$colour

  fill <- if (!is.null(x$fill)) x$fill else defaults$fill
  fill <- alpha(fill, alpha)
  size <- if (!is.null(x$size)) x$size else defaults$size

  lwd <- size * .pt
  lty <- if (!is.null(x$linetype)) x$linetype else defaults$linetype
  gp <- grid::gpar(
    col = col,
    fill = fill,
    lwd = lwd,
    lty = lty,
    lineend = lineend,
    linejoin = linejoin,
    linemitre = linemitre
  )
  sf::st_as_grob(x$geometry, gp = gp)
}

#' Merge values from one list into another
#'
#' @param old List to update.
#' @param new List whose elements overwrite matching names in `old`.
#'
#' @return Updated list.
#' @keywords internal
#' @noRd
modify_list <- function(old, new) {
  for (i in names(new)) {
    old[[i]] <- new[[i]]
  }
  old
}

describe("modify_list", {
  it("merges new values into old list", {
    old <- list(a = 1, b = 2, c = 3)
    new <- list(b = 20, d = 40)
    result <- modify_list(old, new)
    expect_identical(result$a, 1)
    expect_identical(result$b, 20)
    expect_identical(result$c, 3)
    expect_identical(result$d, 40)
  })

  it("returns old list when new is empty", {
    old <- list(a = 1)
    result <- modify_list(old, list())
    expect_identical(result, old)
  })
})

describe("gap", {
  it("returns midpoint of range", {
    expect_identical(gap(c(2, 8)), 5)
    expect_identical(gap(c(-10, 10)), 0)
    expect_identical(gap(c(0, 0)), 0)
  })
})

#' Plot brain parcellations (defunct)
#'
#' `ggseg()` is defunct as of version 2.0.0. Use
#' `ggplot() + geom_brain()` instead.
#'
#' @param ... Ignored.
#'
#' @return Does not return; always raises an error.
#' @seealso [geom_brain()] for the replacement API.
#' @export
#' @examples
#' \dontrun{
#' ggseg()
#' }
ggseg <- function(
  ...
) {
  lifecycle::deprecate_stop(
    "2.0.0",
    "ggseg()",
    details = "Please use `ggplot() + geom_brain()` instead."
  )
}

#' Themes for brain atlas plots
#'
#' @description
#' A set of ggplot2 themes designed for brain atlas visualisations. All themes
#' remove axis ticks and grid lines for a clean presentation.
#'
#' @param text.size Text size in points (default: `12`).
#' @param text.colour Text colour (`theme_custombrain` and `theme_brain2` only).
#' @param text.family Font family (default: `"mono"`).
#' @param plot.background Background fill colour (`theme_custombrain` and
#'   `theme_brain2` only).
#'
#' @details
#' \describe{
#'
#' \item{`theme_brain`}{
#' Default theme. Transparent background, no axes, no grid.}
#'
#' \item{`theme_darkbrain`}{
#' Dark theme with black background and light text.}
#'
#' \item{`theme_custombrain`}{
#' Fully customisable background, text colour, size, and font.}
#'
#' \item{`theme_brain2`}{
#' Like `theme_custombrain` but with axis text removed entirely.}
#'
#' }
#'
#' @return A [ggplot2::theme] object.
#' @seealso [geom_brain()], [ggplot2::theme()]
#' @export
#' @importFrom ggplot2 theme element_blank element_rect element_text
#' @examples
#' library(ggplot2)
#'
#' p <- ggplot() +
#'   geom_brain(atlas = dk())
#'
#' p +
#'   theme_brain()
#'
#' p +
#'   theme_darkbrain()
#'
theme_brain <- function(text.size = 12, text.family = "mono") {
  theme(
    axis.ticks = element_blank(),
    panel.grid = element_blank(),
    panel.background = element_blank(),
    plot.background = element_blank(),
    legend.background = element_blank(),
    text = element_text(family = text.family, size = text.size),
    axis.text = element_text(family = text.family, size = text.size)
  )
}

#' @export
#' @rdname theme_brain
#' @importFrom ggplot2 theme element_blank element_rect element_text
theme_darkbrain <- function(text.size = 12, text.family = "mono") {
  theme(
    axis.ticks = element_blank(),
    panel.grid = element_blank(),
    panel.background = element_blank(),
    legend.background = element_rect(fill = "black"),
    plot.background = element_rect(fill = "black"),
    text = element_text(
      colour = "lightgrey",
      family = text.family,
      size = text.size
    ),
    axis.text = element_text(
      colour = "lightgrey",
      family = text.family,
      size = text.size
    )
  )
}

#' @export
#' @rdname theme_brain
#' @importFrom ggplot2 theme element_blank element_rect element_text
theme_custombrain <- function(
  plot.background = "white",
  text.colour = "darkgrey",
  text.size = 12,
  text.family = "mono"
) {
  theme(
    axis.ticks = element_blank(),
    panel.grid = element_blank(),
    panel.background = element_blank(),
    legend.background = element_rect(fill = plot.background),
    plot.background = element_rect(fill = plot.background),
    text = element_text(
      colour = text.colour,
      family = text.family,
      size = text.size
    ),
    axis.text = element_text(
      colour = text.colour,
      family = text.family,
      size = text.size
    )
  )
}

#' @export
#' @rdname theme_brain
#' @importFrom ggplot2 theme element_blank element_rect element_text
theme_brain2 <- function(
  plot.background = "white",
  text.colour = "darkgrey",
  text.size = 12,
  text.family = "mono"
) {
  theme(
    axis.ticks = element_blank(),
    panel.grid = element_blank(),
    panel.background = element_blank(),
    legend.background = element_rect(fill = plot.background),
    plot.background = element_rect(fill = plot.background),
    text = element_text(
      colour = text.colour,
      family = text.family,
      size = text.size
    ),
    axis.text = element_blank()
  )
}

describe("theme_brain", {
  p <- ggplot() + geom_brain(atlas = dk(), show.legend = FALSE)

  it("returns a ggplot theme", {
    expect_s3_class(theme_brain(), "theme")
  })

  it("applies to a plot", {
    result <- p + theme_brain()
    expect_s3_class(result, "gg")
  })

  it("accepts custom text size and family", {
    th <- theme_brain(text.size = 16, text.family = "sans")
    expect_s3_class(th, "theme")
  })

  it("has blank axis ticks", {
    th <- theme_brain()
    expect_s3_class(th$axis.ticks, "element_blank")
  })
})

describe("theme_darkbrain", {
  it("returns a ggplot theme", {
    expect_s3_class(theme_darkbrain(), "theme")
  })

  it("has black plot background", {
    th <- theme_darkbrain()
    expect_identical(th$plot.background$fill, "black")
  })

  it("accepts custom text size and family", {
    th <- theme_darkbrain(text.size = 8, text.family = "sans")
    expect_s3_class(th, "theme")
  })
})

describe("theme_custombrain", {
  it("returns a ggplot theme", {
    expect_s3_class(theme_custombrain(), "theme")
  })

  it("uses custom background colour", {
    th <- theme_custombrain(plot.background = "pink")
    expect_identical(th$plot.background$fill, "pink")
  })

  it("uses custom text colour", {
    th <- theme_custombrain(text.colour = "blue")
    expect_identical(th$text$colour, "blue")
  })

  it("accepts all custom parameters", {
    th <- theme_custombrain(
      plot.background = "grey90",
      text.colour = "red",
      text.size = 14,
      text.family = "serif"
    )
    expect_s3_class(th, "theme")
  })
})

describe("theme_brain2", {
  it("returns a ggplot theme", {
    expect_s3_class(theme_brain2(), "theme")
  })

  it("has blank axis text", {
    th <- theme_brain2()
    expect_s3_class(th$axis.text, "element_blank")
  })

  it("accepts custom parameters", {
    th <- theme_brain2(
      plot.background = "grey90",
      text.colour = "navy",
      text.size = 10,
      text.family = "serif"
    )
    expect_s3_class(th, "theme")
  })
})

#' Create a brain atlas ggplot2 layer
#'
#' Thin wrapper around [ggplot2::layer()] that substitutes the
#' custom [LayerBrain] class for atlas-specific setup logic.
#'
#' @param geom A ggplot2 Geom ggproto object.
#' @param stat A ggplot2 Stat ggproto object or string.
#' @param data Data to display (data.frame or waiver).
#' @param mapping Set of aesthetic mappings.
#' @param position Position adjustment.
#' @param params Additional parameters for the geom/stat.
#' @param inherit.aes Whether to inherit aesthetics from the plot.
#' @param check.aes Whether to check aesthetics.
#' @param check.param Whether to check parameters.
#' @param show.legend Whether to include in legends.
#'
#' @return A [ggplot2::layer()] object of class `LayerBrain`.
#' @importFrom ggplot2 layer
#' @keywords internal
#' @noRd
layer_brain <- function(
  geom = NULL,
  stat = NULL,
  data = NULL,
  mapping = NULL,
  position = NULL,
  params = list(),
  inherit.aes = TRUE,
  check.aes = TRUE,
  check.param = TRUE,
  show.legend = NA
) {
  layer(
    geom = geom,
    stat = stat,
    data = data,
    mapping = mapping,
    position = position,
    params = params,
    inherit.aes = inherit.aes,
    check.aes = check.aes,
    check.param = check.param,
    show.legend = show.legend,
    layer_class = LayerBrain
  )
}

# ggplot2 does not export `Layer`, so we reach into its namespace once
# to get the parent ggproto. Using `getFromNamespace()` rather than `:::`
# keeps `R CMD check` clean.
ggplot2_Layer <- function() {
  utils::getFromNamespace("Layer", "ggplot2")
}

#' Custom ggplot2 Layer for brain atlas data
#'
#' A [ggplot2::ggproto()] Layer subclass that handles atlas
#' validation, hemisphere/view filtering, data joining, and
#' automatic aesthetic mapping in `setup_layer()`. Subclasses
#' ggplot2's non-exported `Layer` ggproto via
#' [utils::getFromNamespace()], avoiding `:::` usage.
#'
#' @importFrom utils capture.output getFromNamespace
#' @importFrom ggplot2 ggproto ggproto_parent
#' @keywords internal
#' @noRd
LayerBrain <- ggproto(
  "LayerBrain",
  ggplot2_Layer(),
  setup_layer = function(self, data, plot) {
    dt <- ggproto_parent(ggplot2_Layer(), self)$setup_layer(data, plot)

    atlas_obj <- self$geom_params$atlas

    if (is.null(atlas_obj)) {
      cli::cli_abort(
        "No atlas supplied, please provide a brain atlas to the geom."
      )
    }

    atlas <- as.data.frame(atlas_obj)

    if (nrow(atlas) == 0) {
      cli::cli_abort("Atlas has no data to plot.")
    }

    if (!is.null(self$geom_params$hemi)) {
      hemi <- self$geom_params$hemi
      invalid <- setdiff(hemi, unique(atlas$hemi))
      if (length(invalid) > 0) {
        avail <- unique(atlas$hemi)
        cli::cli_abort(
          "Invalid hemisphere(s): {.val {invalid}}. Available: {.val {avail}}"
        )
      }
      atlas <- atlas[atlas$hemi %in% hemi, ]
    }

    if (!is.null(self$geom_params$view)) {
      view <- self$geom_params$view
      invalid <- setdiff(view, unique(atlas$view))
      if (length(invalid) > 0) {
        avail <- unique(atlas$view)
        cli::cli_abort(
          "Invalid view(s): {.val {invalid}}. Available: {.val {avail}}"
        )
      }
      atlas <- atlas[atlas$view %in% view, ]
    }

    if (class(dt)[1] != "waiver") {
      if (!dplyr::is.grouped_df(dt)) {
        facet_vars <- plot$facet$vars()
        group_vars <- intersect(facet_vars, names(dt))
        group_vars <- setdiff(group_vars, names(atlas))
        if (length(group_vars) > 0) {
          dt <- dplyr::group_by(dt, dplyr::across(dplyr::all_of(group_vars)))
        }
      }

      data <- brain_join(dt, atlas)

      merge_errs <- vapply(
        data$geometry,
        function(x) length(!is.na(x)) > 0,
        logical(1)
      )

      if (!all(merge_errs)) {
        k <- data[!merge_errs, ]
        k <- k[, !vapply(k, anyNA, logical(1))]
        k$geometry <- NULL
        k <- paste(utils::capture.output(k), collapse = "\n")

        cli::cli_warn(sprintf(
          "Some data not merged. Check for spelling mistakes in:\n%s",
          k
        ))
        data <- data[merge_errs, ]
      }
    } else {
      data <- atlas
    }

    data <- sf::st_as_sf(data)

    needs_mapping <- function(aes_name) {
      self_map <- self$computed_mapping[[aes_name]]
      plot_map <- plot$computed_mapping[[aes_name]]
      if (isTRUE(self$inherit.aes)) {
        is.null(self_map) && is.null(plot_map)
      } else {
        is.null(self_map)
      }
    }

    if (needs_mapping("geometry") && inherits(data, "sf")) {
      geometry_col <- attr(data, "sf_column")
      self$computed_mapping$geometry <- as.name(geometry_col)
    }

    if (needs_mapping("hemi")) {
      self$computed_mapping$hemi <- as.name("hemi")
    }

    if (needs_mapping("view")) {
      self$computed_mapping$view <- as.name("view")
    }

    if (needs_mapping("type")) {
      self$computed_mapping$type <- as.name("type")
    }

    if (needs_mapping("fill")) {
      self$computed_mapping$fill <- as.name("label")
    }

    self$computed_mapping$label <- as.name("label")

    self$geom_params$legend <- "polygon"

    data
  }
)

# sf-optional brain renderer over flat polygon data ----
#
# Renders a brain atlas using ggplot2::geom_polygon over the polygon
# representation from ggseg.formats::atlas_polygons() (see ggseg.formats). No
# sf objects, no GDAL/GEOS/PROJ system libraries needed — enables wasm and
# air-gapped builds. First piece of Epic ggsegverse/ggseg#128.
#
# Scope of this iteration: simple per-view stacking from the polygons'
# pre-positioned coordinates. position_brain_sf() is not yet wired up for
# the polygon path; users wanting custom layouts should keep using
# geom_brain_sf() for now.

#' Plot brain atlas regions from the polygon representation (sf-optional)
#'
#' Renders a `ggseg_atlas` via `ggplot2::geom_polygon` using the polygon
#' representation from `ggseg.formats::atlas_polygons()`. Works on any atlas;
#' a polygon-backed atlas (e.g. from `ggseg.formats::as_polygon_atlas()`)
#' renders without requiring the `sf` package or its GDAL/GEOS/PROJ system
#' libraries.
#'
#' Hole geometry round-trips correctly via the `subgroup` aesthetic
#' (`grid::pathGrob` even-odd fill).
#'
#' @param mapping Set of aesthetic mappings created by `ggplot2::aes()`.
#' @param data A data.frame containing variables to map. If `NULL`, the
#'   atlas is plotted without user data.
#' @param atlas A `ggseg_atlas` object with 2D geometry (sf or polygons).
#' @param hemi Character vector of hemispheres to include.
#' @param view Character vector of views to include.
#' @param position Position adjustment. Defaults to [position_brain_polygon()],
#'   which lays views out horizontally without sf. Pass `"identity"` to use
#'   the polygons' raw coordinates. Per-view zoom is controlled here via
#'   [position_brain_polygon()]'s `zoom` argument.
#' @param context Logical. When `TRUE` (default), context regions (atlas rows
#'   with no `region` label, drawn grey) are kept. When `FALSE`, they are
#'   dropped so only true atlas regions are plotted.
#' @param show.legend Logical. Should this layer be included in legends?
#' @param inherit.aes Logical. If `FALSE`, overrides the default aesthetics.
#' @param ... Additional arguments passed to `ggplot2::geom_polygon()`.
#'
#' @return A list of ggplot2 layer and coord objects.
#' @keywords internal
#' @noRd
#' @importFrom ggplot2 aes geom_polygon scale_fill_manual
#' @importFrom rlang .data
#'
#' @examples
#' \dontrun{
#' library(ggplot2)
#' poly <- ggseg.formats::as_polygon_atlas(dk())
#' ggplot() + geom_brain_polygon(atlas = poly)
#' }
geom_brain_polygon <- function(
  mapping = aes(),
  data = NULL,
  atlas,
  hemi = NULL,
  view = NULL,
  position = position_brain_polygon(),
  context = TRUE,
  show.legend = NA,
  inherit.aes = TRUE,
  ...
) {
  zoom_spec <- if (is_polygon_position(position)) position$zoom else NULL
  focus <- resolve_zoom_focus(zoom_spec, data, atlas)

  flat <- prepare_polygon_atlas(
    atlas,
    hemi = hemi,
    view = view,
    position = position,
    context = context,
    focus = focus
  )

  if (!is.null(data)) {
    flat <- brain_join_polygon(data, flat)
  }

  base_mapping <- aes(
    x = .data$x,
    y = .data$y,
    group = .data$.feature_id,
    subgroup = .data$subgroup
  )
  if (!"fill" %in% names(mapping)) {
    base_mapping$fill <- quote(.data$label)
  }
  user_mapping <- utils::modifyList(base_mapping, as.list(mapping))
  class(user_mapping) <- "uneval"

  dots <- list(...)
  if (!"colour" %in% names(dots) && !"color" %in% names(dots)) {
    dots$colour <- "grey35"
  }
  if (!"linewidth" %in% names(dots) && !"size" %in% names(dots)) {
    dots$linewidth <- 0.2
  }

  layer <- rlang::exec(
    geom_polygon,
    mapping = user_mapping,
    data = flat,
    position = "identity",
    show.legend = show.legend,
    inherit.aes = inherit.aes,
    !!!dots
  )

  result <- list(layer, coord_brain())

  if (!is.null(atlas$palette) && !"fill" %in% names(mapping)) {
    result <- c(
      result,
      list(scale_fill_manual(values = atlas$palette, na.value = "grey"))
    )
  }

  result
}


#' Flatten a polygon atlas into a row-per-point data.frame for geom_polygon
#'
#' Unnests `ggseg.formats::atlas_polygons(atlas)`, joins with `atlas$core`,
#' applies optional
#' hemi/view filters, and adds the `.feature_id` helper column that the
#' polygon renderer maps to `group` so each (label, view, group) polygon
#' piece renders as one ring set.
#'
#' @keywords internal
#' @noRd
prepare_polygon_atlas <- function(
  atlas,
  hemi = NULL,
  view = NULL,
  position = NULL,
  context = TRUE,
  focus = NULL
) {
  if (is.null(ggseg.formats::atlas_geom(atlas))) {
    cli::cli_abort(c(
      "{.arg atlas} has no 2D geometry.",
      "i" = "Provide an atlas with {.field geom} (sf or polygons) in its data."
    ))
  }

  flat <- tidyr::unnest(
    ggseg.formats::atlas_polygons(atlas),
    cols = "geometry"
  )
  # Namespace the polygon-ring grouping as `.group` before joining `core`:
  # some atlases (e.g. tracula) carry their own `group` column in `core`,
  # which would otherwise collide and suffix this one away.
  names(flat)[names(flat) == "group"] <- ".group"
  flat <- dplyr::left_join(
    flat,
    atlas$core,
    by = "label",
    relationship = "many-to-many"
  )

  if (!is.null(view)) {
    avail <- unique(flat$view)
    invalid <- setdiff(view, avail)
    if (length(invalid)) {
      cli::cli_abort(
        "Invalid view(s): {.val {invalid}}. Available: {.val {avail}}"
      )
    }
    flat <- flat[flat$view %in% view, , drop = FALSE]
  }

  if (!is.null(hemi)) {
    avail <- unique(flat$hemi)
    invalid <- setdiff(hemi, avail)
    if (length(invalid)) {
      cli::cli_abort(
        "Invalid hemisphere(s): {.val {invalid}}. Available: {.val {avail}}"
      )
    }
    flat <- flat[flat$hemi %in% hemi, , drop = FALSE]
  }

  if (!context && "region" %in% names(flat)) {
    flat <- flat[!is.na(flat$region), , drop = FALSE]
  }

  flat$atlas <- atlas$atlas
  flat$type <- atlas$type

  if (atlas$type == "cortical") {
    if (!"hemi" %in% names(flat)) {
      flat$hemi <- NA_character_
    }
    missing_hemi <- is.na(flat$hemi)
    flat$hemi[missing_hemi] <- dplyr::case_when(
      grepl("^lh[_.]", flat$label[missing_hemi]) ~ "left",
      grepl("^rh[_.]", flat$label[missing_hemi]) ~ "right",
      .default = NA_character_
    )
  }

  flat$.feature_id <- as.integer(factor(
    paste(flat$label, flat$view, flat$.group, sep = "@@")
  ))

  if (is_polygon_position(position)) {
    flat <- frame_2_position_flat(
      flat,
      position$position,
      nrow = position$nrow,
      ncol = position$ncol,
      views = position$views,
      focus = focus,
      zoom_pad = if (is.null(position$zoom_pad)) 0.05 else position$zoom_pad
    )
  }

  flat
}


#' Polygon-path version of `brain_join()` — joins user data onto flat rows
#'
#' Matches on `region` or `label` (and `hemi` if both data and atlas carry
#' it). Polygon rows without a matching data row keep `NA` for the joined
#' columns; the renderer paints them grey via `na.value` on the fill scale.
#'
#' Grouped data (via [dplyr::group_by()]) is replicated like [brain_join()]:
#' the full atlas is repeated once per group, with the grouping columns set on
#' every row — including unmatched context regions — so [ggplot2::facet_wrap()]
#' shows the complete atlas in each panel.
#'
#' @keywords internal
#' @noRd
brain_join_polygon <- function(data, flat) {
  by <- intersect(
    c("region", "label", "hemi"),
    intersect(names(data), names(flat))
  )
  if (!length(by)) {
    cli::cli_abort(c(
      "{.arg data} has no columns in common with the atlas.",
      "i" = paste0(
        "Need {.field region} or {.field label} ",
        "(and optionally {.field hemi})."
      )
    ))
  }

  if (!dplyr::is.grouped_df(data)) {
    return(dplyr::left_join(flat, data, by = by, suffix = c("", ".user")))
  }

  group_cols <- dplyr::group_vars(data)
  nested <- tidyr::nest(data)
  pieces <- lapply(seq_len(nrow(nested)), function(i) {
    piece <- dplyr::left_join(
      flat,
      nested$data[[i]],
      by = by,
      suffix = c("", ".user")
    )
    for (g in group_cols) {
      piece[[g]] <- nested[[g]][[i]]
    }
    piece
  })
  dplyr::bind_rows(pieces)
}

describe("annotate_brain_polygon()", {
  it("returns a ggplot2 annotation layer", {
    poly <- ggseg.formats::as_polygon_atlas(dk())
    pos <- position_brain_polygon(hemi ~ view)
    ann <- annotate_brain_polygon(atlas = poly, position = pos)
    expect_s3_class(ann, "Layer")
  })

  it("renders with geom_brain_polygon to produce labelled views", {
    poly <- ggseg.formats::as_polygon_atlas(dk())
    pos <- position_brain_polygon(hemi ~ view)
    p <- ggplot2::ggplot() +
      geom_brain_polygon(atlas = poly, position = pos, show.legend = FALSE) +
      annotate_brain_polygon(atlas = poly, position = pos)
    g <- ggplot2::ggplot_build(p)
    expect_gte(length(g$data), 2L)
    expect_gt(nrow(g$data[[1]]), 0)
  })

  it("filters by hemi", {
    poly <- ggseg.formats::as_polygon_atlas(dk())
    pos <- position_brain_polygon(hemi ~ view)
    p <- ggplot2::ggplot() +
      geom_brain_polygon(
        atlas = poly,
        position = pos,
        hemi = "left",
        show.legend = FALSE
      ) +
      annotate_brain_polygon(atlas = poly, position = pos, hemi = "left")
    g <- ggplot2::ggplot_build(p)
    expect_true(all(g$data[[1]]$hemi == "left" | is.na(g$data[[1]]$hemi)))
  })

  it("works for subcortical aseg", {
    poly_aseg <- ggseg.formats::as_polygon_atlas(aseg())
    p <- ggplot2::ggplot() +
      geom_brain_polygon(atlas = poly_aseg, show.legend = FALSE) +
      annotate_brain_polygon(atlas = poly_aseg)
    expect_no_error(ggplot2::ggplot_build(p))
  })
})

describe("compute_label_positions_flat()", {
  it("produces one row per (hemi, view) for cortical atlases", {
    poly <- ggseg.formats::as_polygon_atlas(dk())
    flat <- prepare_polygon_atlas(
      poly,
      position = position_brain_polygon(hemi ~ view)
    )
    labels <- compute_label_positions_flat(flat)
    expect_true(all(c("x", "y", "label") %in% names(labels)))
    expect_gt(nrow(labels), 1)
  })

  it("produces one row per view for subcortical atlases", {
    poly_aseg <- ggseg.formats::as_polygon_atlas(aseg())
    flat <- prepare_polygon_atlas(poly_aseg)
    labels <- compute_label_positions_flat(flat)
    expect_equal(nrow(labels), length(unique(flat$view)))
  })
})

describe("geom_brain_polygon()", {
  it("renders a polygon atlas without requiring sf in the data path", {
    skip_if_not_installed("vdiffr")
    poly <- ggseg.formats::as_polygon_atlas(dk())
    p <- ggplot2::ggplot() + geom_brain_polygon(atlas = poly)
    g <- ggplot2::ggplot_build(p)
    expect_true(length(g$data) >= 1)
    expect_gt(nrow(g$data[[1]]), 0)
  })

  it("renders an sf-backed atlas via on-the-fly polygon conversion", {
    skip_if_not_installed("sf")
    sf_atlas <- ggseg.formats::as_sf_atlas(dk())
    p <- ggplot2::ggplot() + geom_brain_polygon(atlas = sf_atlas)
    g <- ggplot2::ggplot_build(p)
    expect_gt(nrow(g$data[[1]]), 0)
  })

  it("errors when the atlas has no 2D geometry", {
    no_geom <- atlas_without_2d_geometry()
    expect_error(
      ggplot2::ggplot() + geom_brain_polygon(atlas = no_geom),
      "no 2D geometry"
    )
  })

  it("filters by view", {
    poly <- ggseg.formats::as_polygon_atlas(dk())
    p <- ggplot2::ggplot() +
      geom_brain_polygon(atlas = poly, view = "lateral")
    g <- ggplot2::ggplot_build(p)
    expect_true(all(g$data[[1]]$view == "lateral" | is.na(g$data[[1]]$view)))
  })

  it("rejects invalid views with a clear error", {
    poly <- ggseg.formats::as_polygon_atlas(dk())
    expect_error(
      ggplot2::ggplot() + geom_brain_polygon(atlas = poly, view = "nope"),
      "Invalid view"
    )
  })

  it("filters by hemi", {
    poly <- ggseg.formats::as_polygon_atlas(dk())
    p <- ggplot2::ggplot() +
      geom_brain_polygon(atlas = poly, hemi = "left")
    g <- ggplot2::ggplot_build(p)
    hemis <- unique(g$data[[1]]$hemi)
    expect_true(all(hemis %in% c("left", NA)))
  })

  it("joins user data on region", {
    poly <- ggseg.formats::as_polygon_atlas(dk())
    regs <- unique(poly$core$region)
    regs <- regs[!is.na(regs)]
    user <- data.frame(
      region = regs,
      measure = seq_along(regs) / length(regs)
    )
    p <- ggplot2::ggplot() +
      geom_brain_polygon(
        data = user,
        atlas = poly,
        ggplot2::aes(fill = measure)
      )
    g <- ggplot2::ggplot_build(p)
    expect_true(
      "measure" %in% names(g$data[[1]]) || "fill" %in% names(g$data[[1]])
    )
  })

  it("works on subcortical aseg via the polygon path", {
    poly_aseg <- ggseg.formats::as_polygon_atlas(aseg())
    p <- ggplot2::ggplot() + geom_brain_polygon(atlas = poly_aseg)
    g <- ggplot2::ggplot_build(p)
    expect_gt(nrow(g$data[[1]]), 0)
  })

  it("bundles a fixed-aspect default coord so shapes are not stretched", {
    poly <- ggseg.formats::as_polygon_atlas(dk())
    p <- ggplot2::ggplot() + geom_brain_polygon(atlas = poly)
    expect_equal(p$coordinates$ratio, 1)
    expect_true(isTRUE(p$coordinates$default))
  })

  it("lets a user coord override the bundled one without warning", {
    poly <- ggseg.formats::as_polygon_atlas(dk())
    p <- ggplot2::ggplot() +
      geom_brain_polygon(atlas = poly) +
      ggplot2::coord_fixed(ratio = 2)
    expect_no_message(ggplot2::ggplot_build(p))
    expect_equal(p$coordinates$ratio, 2)
  })

  it("drops context regions when context = FALSE", {
    poly <- ggseg.formats::as_polygon_atlas(aseg())
    full <- prepare_polygon_atlas(poly)
    no_ctx <- prepare_polygon_atlas(poly, context = FALSE)
    expect_true(any(is.na(full$region)))
    expect_false(any(is.na(no_ctx$region)))
    expect_lt(nrow(no_ctx), nrow(full))
  })

  it("context = FALSE re-gathers views into a tighter extent", {
    poly <- ggseg.formats::as_polygon_atlas(aseg())
    rng <- function(p) {
      diff(range(ggplot2::ggplot_build(p)$data[[1]]$x))
    }
    p_full <- ggplot2::ggplot() + geom_brain_polygon(atlas = poly)
    p_ctx <- ggplot2::ggplot() +
      geom_brain_polygon(atlas = poly, context = FALSE)
    expect_lt(rng(p_ctx), rng(p_full))
  })

  it("zoom = TRUE crops each view onto the focus regions", {
    poly <- ggseg.formats::as_polygon_atlas(aseg())
    p_full <- ggplot2::ggplot() + geom_brain_polygon(atlas = poly)
    p_zoom <- ggplot2::ggplot() +
      geom_brain_polygon(
        atlas = poly,
        position = position_brain_polygon(zoom = TRUE)
      )
    span <- function(p) diff(range(ggplot2::ggplot_build(p)$data[[1]]$x))
    expect_lt(span(p_zoom), span(p_full))
  })

  it("zoom = TRUE focuses on regions present in user data", {
    poly <- ggseg.formats::as_polygon_atlas(aseg())
    regs <- unique(poly$core$region)
    regs <- regs[!is.na(regs)][1:2]
    user <- data.frame(region = regs, measure = c(1, 2))
    p <- ggplot2::ggplot() +
      geom_brain_polygon(
        data = user,
        atlas = poly,
        ggplot2::aes(fill = measure),
        position = position_brain_polygon(zoom = TRUE)
      )
    expect_gt(nrow(ggplot2::ggplot_build(p)$data[[1]]), 0)
  })
})

describe("prepare_polygon_atlas()", {
  it("flattens to row-per-point with the expected columns", {
    poly <- ggseg.formats::as_polygon_atlas(dk())
    flat <- prepare_polygon_atlas(poly)
    expect_true(all(
      c("label", "view", "x", "y", ".group", "subgroup", ".feature_id") %in%
        names(flat)
    ))
    expect_gt(nrow(flat), nrow(ggseg.formats::atlas_polygons(poly)))
  })

  it("renames the polygon-ring group to .group to avoid user collision", {
    poly <- ggseg.formats::as_polygon_atlas(dk())
    flat <- prepare_polygon_atlas(poly)
    expect_false("group" %in% names(flat))
    expect_true(".group" %in% names(flat))
  })

  it("assigns one .feature_id per (label, view, .group)", {
    poly <- ggseg.formats::as_polygon_atlas(dk())
    flat <- prepare_polygon_atlas(poly)
    keys <- unique(paste(flat$label, flat$view, flat$.group, sep = "@@"))
    expect_equal(length(unique(flat$.feature_id)), length(keys))
  })
})

describe("brain_join_polygon() faceting", {
  it("replicates the full atlas per group for grouped data", {
    poly <- ggseg.formats::as_polygon_atlas(dk())
    flat <- prepare_polygon_atlas(poly)
    data <- dplyr::group_by(
      data.frame(
        region = c("insula", "precentral"),
        p = c(0.1, 0.2),
        group = c("A", "B"),
        stringsAsFactors = FALSE
      ),
      group
    )
    joined <- brain_join_polygon(data, flat)

    expect_setequal(unique(joined$group), c("A", "B"))
    a <- joined[joined$group == "A", ]
    b <- joined[joined$group == "B", ]
    expect_equal(nrow(a), nrow(flat))
    expect_equal(nrow(b), nrow(flat))
    expect_true(any(!is.na(a$p)))
    expect_true(any(is.na(a$p)))
  })

  it("joins by label when data carries label but not region", {
    poly <- ggseg.formats::as_polygon_atlas(dk())
    flat <- prepare_polygon_atlas(poly)
    lbl <- ggseg.formats::atlas_labels(dk())[1]
    data <- data.frame(label = lbl, val = 1.5, stringsAsFactors = FALSE)
    joined <- brain_join_polygon(data, flat)
    expect_true("val" %in% names(joined))
    expect_equal(unique(joined$val[joined$label %in% lbl]), 1.5)
  })

  it("keeps a user column named group without colliding", {
    poly <- ggseg.formats::as_polygon_atlas(dk())
    flat <- prepare_polygon_atlas(poly)
    data <- data.frame(
      region = "insula",
      group = "cohort1",
      stringsAsFactors = FALSE
    )
    joined <- brain_join_polygon(data, flat)
    expect_true("group" %in% names(joined))
    expect_equal(unique(joined$group[joined$region %in% "insula"]), "cohort1")
  })
})

describe("geom_brain() backwards-compatibility with sf atlases", {
  # The polygon flip kept geom_brain() working on atlas objects that pre-date
  # the polygon representation: it decomposes their sf geometry on the fly.
  # These lock that in so a later refactor can't silently break it.
  it("renders a legacy ggseg_atlas object", {
    skip_if_not_installed("sf")
    legacy <- ggseg.formats::as_ggseg_atlas(dk())
    expect_s3_class(legacy, "ggseg_atlas")
    p <- ggplot2::ggplot() + geom_brain(atlas = legacy, show.legend = FALSE)
    g <- ggplot2::ggplot_build(p)
    expect_gt(nrow(g$data[[1]]), 0)
  })

  it("renders an sf-only atlas (no polygon representation)", {
    skip_if_not_installed("sf")
    sf_only <- ggseg.formats::as_sf_atlas(dk())
    expect_true(ggseg.formats::is_atlas_sf(sf_only))
    expect_false(ggseg.formats::is_atlas_polygon(sf_only))
    p <- ggplot2::ggplot() + geom_brain(atlas = sf_only, show.legend = FALSE)
    g <- ggplot2::ggplot_build(p)
    expect_gt(nrow(g$data[[1]]), 0)
  })
})

# Colour and fill ----
#' Colour and fill scales from brain atlas palettes
#'
#' @description
#' `r lifecycle::badge("deprecated")`
#'
#' Atlas palettes are now applied automatically by [geom_brain()].
#' Use [scale_fill_brain_manual()] for custom palettes.
#'
#' @param name String name of the atlas palette (e.g. `"dk"`, `"aseg"`).
#' @param na.value Colour for `NA` entries (default: `"grey"`).
#' @param aesthetics Which aesthetic to scale: `"fill"`, `"colour"`, or
#'   `"color"`.
#' @param ... Additional arguments passed to [ggseg.formats::atlas_palette()].
#'
#' @return A ggplot2 scale object.
#' @rdname scale_brain
#' @export
#'
#' @importFrom ggplot2 scale_color_manual scale_colour_manual scale_fill_manual
#' @importFrom ggseg.formats atlas_palette
#' @examples
#' \dontrun{
#' library(ggplot2)
#' ggplot() +
#'   geom_brain(atlas = dk(), aes(fill = region), show.legend = FALSE) +
#'   scale_brain("dk")
#' }
scale_brain <- function(
  name = "dk",
  na.value = "grey",
  ...,
  aesthetics = c("fill", "colour", "color")
) {
  lifecycle::deprecate_warn(
    "2.0.0",
    "scale_brain()",
    details = "Atlas palettes are now applied automatically by `geom_brain()`."
  )
  pal <- atlas_palette_by_name(name, ...)
  aesthetics <- match.arg(aesthetics)
  func <- switch(
    aesthetics,
    color = scale_color_manual,
    colour = scale_colour_manual,
    fill = scale_fill_manual
  )
  func(values = pal, na.value = na.value)
}

#' @rdname scale_brain
#' @export
scale_colour_brain <- function(name = "dk", na.value = "grey", ...) {
  lifecycle::deprecate_warn(
    "2.0.0",
    "scale_colour_brain()",
    details = "Atlas palettes are now applied automatically by `geom_brain()`."
  )
  pal <- atlas_palette_by_name(name, ...)
  scale_colour_manual(values = pal, na.value = na.value)
}

#' @rdname scale_brain
#' @export
scale_color_brain <- function(name = "dk", na.value = "grey", ...) {
  lifecycle::deprecate_warn(
    "2.0.0",
    "scale_color_brain()",
    details = "Atlas palettes are now applied automatically by `geom_brain()`."
  )
  pal <- atlas_palette_by_name(name, ...)
  scale_color_manual(values = pal, na.value = na.value)
}

#' @export
#' @rdname scale_brain
scale_fill_brain <- function(name = "dk", na.value = "grey", ...) {
  lifecycle::deprecate_warn(
    "2.0.0",
    "scale_fill_brain()",
    details = "Atlas palettes are now applied automatically by `geom_brain()`."
  )
  pal <- atlas_palette_by_name(name, ...)
  scale_fill_manual(values = pal, na.value = na.value)
}

atlas_palette_by_name <- function(name, ...) {
  atlas_palette(match.fun(name)(), ...)
}

#' Manual colour and fill scales for brain plots
#'
#' @description
#' Apply a custom named colour palette to brain atlas plots. Use this
#' when you want to override the atlas default colours with your own
#' colour mapping.
#'
#' @param palette Named character vector mapping region names to colours.
#' @param na.value Colour for `NA` entries (default: `"grey"`).
#' @param aesthetics Which aesthetic to scale: `"fill"`, `"colour"`, or
#'   `"color"`.
#' @param ... Additional arguments (unused).
#'
#' @return A ggplot2 scale object.
#' @rdname scale_brain_manual
#' @export
#' @examples
#' library(ggplot2)
#'
#' pal <- c("insula" = "red", "precentral" = "blue")
#' ggplot() +
#'   geom_brain(atlas = dk(), aes(fill = region), show.legend = FALSE) +
#'   scale_fill_brain_manual(palette = pal)
#'
scale_brain_manual <- function(
  palette,
  na.value = "grey",
  ...,
  aesthetics = c("fill", "colour", "color")
) {
  aesthetics <- match.arg(aesthetics)
  func <- switch(
    aesthetics,
    color = ggplot2::scale_color_manual,
    colour = ggplot2::scale_colour_manual,
    fill = ggplot2::scale_fill_manual
  )
  func(values = palette, na.value = na.value)
}

#' @rdname scale_brain_manual
#' @export
scale_colour_brain_manual <- function(...) {
  scale_brain_manual(..., aesthetics = "colour")
}

#' @rdname scale_brain_manual
#' @export
scale_color_brain_manual <- function(...) {
  scale_brain_manual(..., aesthetics = "color")
}

#' @export
#' @rdname scale_brain_manual
scale_fill_brain_manual <- function(...) {
  scale_brain_manual(..., aesthetics = "fill")
}

# Deprecated scale_brain2 variants ----
#' @rdname scale_brain2-deprecated
#' @export
scale_brain2 <- function(...) {
  lifecycle::deprecate_warn("1.7.0", "scale_brain2()", "scale_brain_manual()")
  scale_brain_manual(...)
}

#' @rdname scale_brain2-deprecated
#' @export
scale_colour_brain2 <- function(...) {
  lifecycle::deprecate_warn(
    "1.7.0",
    "scale_colour_brain2()",
    "scale_colour_brain_manual()"
  )
  scale_colour_brain_manual(...)
}

#' @rdname scale_brain2-deprecated
#' @export
scale_color_brain2 <- function(...) {
  lifecycle::deprecate_warn(
    "1.7.0",
    "scale_color_brain2()",
    "scale_color_brain_manual()"
  )
  scale_color_brain_manual(...)
}

#' @rdname scale_brain2-deprecated
#' @export
scale_fill_brain2 <- function(...) {
  lifecycle::deprecate_warn(
    "1.7.0",
    "scale_fill_brain2()",
    "scale_fill_brain_manual()"
  )
  scale_fill_brain_manual(...)
}

#' Deprecated scale functions
#'
#' @description
#' `r lifecycle::badge("deprecated")`
#'
#' These functions have been renamed for clarity:
#' - `scale_brain2()` -> [scale_brain_manual()]
#' - `scale_fill_brain2()` -> [scale_fill_brain_manual()]
#' - `scale_colour_brain2()` -> [scale_colour_brain_manual()]
#' - `scale_color_brain2()` -> [scale_color_brain_manual()]
#'
#' @param ... Arguments passed to the replacement function.
#'
#' @return A ggplot2 scale object.
#' @name scale_brain2-deprecated
#' @examples
#' pal <- c("transverse temporal" = "#FF0000", "insula" = "#00FF00")
#' suppressWarnings(scale_fill_brain_manual(palette = pal))
NULL


# Axis scales ----
#' Axis and label scales for brain atlas plots
#'
#' @description
#' Add axis labels and tick labels corresponding to brain atlas regions.
#' These scales add hemisphere or view labels to the x and y axes based on
#' the atlas layout.
#'
#' @inheritParams brain_join
#' @param position Layout style: `"dispersed"` (default) or `"stacked"`.
#' @param aesthetics Which axis to scale: `"x"`, `"y"`, or `"labs"`.
#' @param ... Additional arguments passed to [adapt_scales()].
#'
#' @return A ggplot2 scale or labs object.
#' @rdname scale_continous_brain
#' @export
#' @examples
#' \donttest{
#' library(ggplot2)
#'
#' ggplot() +
#'   geom_brain(atlas = dk()) +
#'   scale_x_brain() +
#'   scale_y_brain() +
#'   scale_labs_brain()
#' }
#'
scale_continous_brain <- function(
  atlas = dk(),
  position = "dispersed",
  aesthetics = c("y", "x")
) {
  positions <- adapt_scales(atlas, position, aesthetics)
  aesthetics <- match.arg(aesthetics)
  func <- switch(
    aesthetics,
    y = ggplot2::scale_y_continuous,
    x = ggplot2::scale_x_continuous
  )
  func(breaks = positions$breaks, labels = positions$labels)
}

#' @export
#' @rdname scale_continous_brain
scale_x_brain <- function(...) {
  scale_continous_brain(..., aesthetics = "x")
}

#' @export
#' @rdname scale_continous_brain
scale_y_brain <- function(...) {
  scale_continous_brain(..., aesthetics = "y")
}

#' @export
#' @rdname scale_continous_brain
#' @importFrom ggplot2 labs
scale_labs_brain <- function(
  atlas = dk(),
  position = "dispersed",
  aesthetics = "labs"
) {
  positions <- adapt_scales(atlas, position, aesthetics)

  aesthetics <- match.arg(aesthetics)
  func <- switch(aesthetics, labs = labs)
  func(x = positions$x, y = positions$y)
}

describe("scale_brain", {
  it("returns a scale for fill by default", {
    lifecycle::expect_deprecated(scale <- scale_brain())
    expect_s3_class(scale, "Scale")
    expect_identical(scale$aesthetics, "fill")
  })

  it("returns a scale for colour", {
    lifecycle::expect_deprecated(scale <- scale_brain(aesthetics = "colour"))
    expect_s3_class(scale, "Scale")
    expect_identical(scale$aesthetics, "colour")
  })

  it("returns a scale for color (aliased to colour)", {
    lifecycle::expect_deprecated(scale <- scale_brain(aesthetics = "color"))
    expect_s3_class(scale, "Scale")
    expect_identical(scale$aesthetics, "colour")
  })

  it("uses custom na.value", {
    lifecycle::expect_deprecated(scale <- scale_brain(na.value = "red"))
    expect_identical(scale$na.value, "red")
  })

  it("accepts atlas name argument", {
    lifecycle::expect_deprecated(scale <- scale_brain(name = "dk"))
    expect_s3_class(scale, "Scale")
  })
})

describe("scale_colour_brain", {
  it("returns a colour scale", {
    lifecycle::expect_deprecated(scale <- scale_colour_brain())
    expect_s3_class(scale, "Scale")
    expect_identical(scale$aesthetics, "colour")
  })
})

describe("scale_color_brain", {
  it("returns a color scale (aliased to colour)", {
    lifecycle::expect_deprecated(scale <- scale_color_brain())
    expect_s3_class(scale, "Scale")
    expect_identical(scale$aesthetics, "colour")
  })
})

describe("scale_fill_brain", {
  it("returns a fill scale", {
    lifecycle::expect_deprecated(scale <- scale_fill_brain())
    expect_s3_class(scale, "Scale")
    expect_identical(scale$aesthetics, "fill")
  })
})

describe("scale_brain_manual", {
  pal <- c("region1" = "#FF0000", "region2" = "#00FF00")

  it("returns a scale for fill by default with custom palette", {
    scale <- scale_brain_manual(palette = pal)
    expect_s3_class(scale, "Scale")
    expect_identical(scale$aesthetics, "fill")
  })

  it("returns a scale for colour", {
    scale <- scale_brain_manual(palette = pal, aesthetics = "colour")
    expect_s3_class(scale, "Scale")
    expect_identical(scale$aesthetics, "colour")
  })

  it("returns a scale for color (aliased to colour)", {
    scale <- scale_brain_manual(palette = pal, aesthetics = "color")
    expect_s3_class(scale, "Scale")
    expect_identical(scale$aesthetics, "colour")
  })

  it("uses custom na.value", {
    scale <- scale_brain_manual(palette = pal, na.value = "blue")
    expect_identical(scale$na.value, "blue")
  })
})

describe("scale_colour_brain_manual", {
  it("returns a colour scale", {
    pal <- c("region1" = "#FF0000")
    scale <- scale_colour_brain_manual(palette = pal)
    expect_s3_class(scale, "Scale")
    expect_identical(scale$aesthetics, "colour")
  })
})

describe("scale_color_brain_manual", {
  it("returns a color scale (aliased to colour)", {
    pal <- c("region1" = "#FF0000")
    scale <- scale_color_brain_manual(palette = pal)
    expect_s3_class(scale, "Scale")
    expect_identical(scale$aesthetics, "colour")
  })
})

describe("scale_fill_brain_manual", {
  it("returns a fill scale", {
    pal <- c("region1" = "#FF0000")
    scale <- scale_fill_brain_manual(palette = pal)
    expect_s3_class(scale, "Scale")
    expect_identical(scale$aesthetics, "fill")
  })
})

describe("deprecated scale_brain2 variants", {
  pal <- c("region1" = "#FF0000", "region2" = "#00FF00")

  it("scale_brain2 warns and delegates", {
    lifecycle::expect_deprecated(scale_brain2(palette = pal))
  })

  it("scale_fill_brain2 warns and delegates", {
    lifecycle::expect_deprecated(scale_fill_brain2(palette = pal))
  })

  it("scale_colour_brain2 warns and delegates", {
    lifecycle::expect_deprecated(scale_colour_brain2(palette = pal))
  })

  it("scale_color_brain2 warns and delegates", {
    lifecycle::expect_deprecated(scale_color_brain2(palette = pal))
  })
})

describe("scale_continous_brain", {
  it("returns y scale by default", {
    skip_if_not_installed("sf")
    atlas <- unnest(sf2coords(as.data.frame(dk())), ggseg)
    scale <- scale_continous_brain(atlas = atlas)
    expect_s3_class(scale, "Scale")
  })

  it("returns x scale", {
    skip_if_not_installed("sf")
    atlas <- unnest(sf2coords(as.data.frame(dk())), ggseg)
    scale <- scale_continous_brain(atlas = atlas, aesthetics = "x")
    expect_s3_class(scale, "Scale")
  })
})

describe("scale_x_brain", {
  it("returns x scale", {
    skip_if_not_installed("sf")
    dk_df <- as.data.frame(dk())
    dk_coords <- sf2coords(dk_df)
    atlas <- unnest(dk_coords, ggseg)
    scale <- scale_x_brain(atlas = atlas)
    expect_s3_class(scale, "Scale")
  })
})

describe("scale_y_brain", {
  it("returns y scale", {
    skip_if_not_installed("sf")
    dk_df <- as.data.frame(dk())
    dk_coords <- sf2coords(dk_df)
    atlas <- unnest(dk_coords, ggseg)
    scale <- scale_y_brain(atlas = atlas)
    expect_s3_class(scale, "Scale")
  })
})

describe("scale_labs_brain", {
  it("returns labs scale", {
    skip_if_not_installed("sf")
    dk_df <- as.data.frame(dk())
    dk_coords <- sf2coords(dk_df)
    atlas <- unnest(dk_coords, ggseg)
    scale <- scale_labs_brain(atlas = atlas)
    expect_s3_class(scale, "gg")
  })
})

#' Compute axis scale positions for brain atlas plots
#'
#' Returns axis breaks, labels, and lab strings based on atlas layout.
#' Used internally by [scale_continous_brain()] and related functions.
#'
#' @param geobrain A data.frame containing atlas information with columns
#'   `hemi`, `view`, `type`, `.lat`, and `.long`.
#' @inheritParams reposition_brain
#' @inheritParams scale_brain
#' @return A list with scale components (breaks, labels, or axis titles).
#' @keywords internal
#' @noRd
#' @importFrom dplyr group_by summarise
adapt_scales <- function(
  geobrain,
  position = "dispersed",
  aesthetics = "labs"
) {
  if (!is.data.frame(geobrain)) {
    geobrain <- sf2coords(as.data.frame(geobrain))
    geobrain <- tidyr::unnest(geobrain, ggseg)
  }

  atlas_type <- unique(geobrain$type)
  if (atlas_type == "cortical") {
    adapt_scales_cortical(geobrain, position, aesthetics)
  } else if (atlas_type %in% c("subcortical", "tract")) {
    adapt_scales_subcortical(geobrain, position, aesthetics)
  }
}


#' @keywords internal
#' @noRd
adapt_scales_cortical <- function(geobrain, position, aesthetics) {
  stk_y <- dplyr::summarise(dplyr::group_by(geobrain, hemi), val = gap(.lat))
  stk_x <- dplyr::summarise(dplyr::group_by(geobrain, view), val = gap(.long))
  disp <- dplyr::summarise_at(
    dplyr::group_by(geobrain, hemi),
    dplyr::vars(.long, .lat),
    list(gap)
  )

  ad_scale <- list(
    stacked = list(
      x = list(breaks = stk_x$val, labels = stk_x$view),
      y = list(breaks = stk_y$val, labels = stk_y$hemi),
      labs = list(y = "hemisphere", x = "view")
    ),
    dispersed = list(
      x = list(breaks = disp$.long, labels = disp$hemi),
      y = list(breaks = NULL, labels = NULL),
      labs = list(y = NULL, x = "hemisphere")
    )
  )

  ad_scale[[position]][[aesthetics]]
}


#' @keywords internal
#' @noRd
adapt_scales_subcortical <- function(geobrain, position, aesthetics) {
  stk_y <- dplyr::summarise(dplyr::group_by(geobrain, view), val = gap(.lat))
  disp <- dplyr::summarise_at(
    dplyr::group_by(geobrain, view),
    dplyr::vars(.long, .lat),
    list(gap)
  )

  ad_scale <- list(
    stacked = list(
      x = list(breaks = NULL, labels = NULL),
      y = list(breaks = stk_y$val, labels = stk_y$view),
      labs = list(y = "view", x = NULL)
    ),
    dispersed = list(
      x = list(breaks = disp$.long, labels = disp$view),
      y = list(breaks = NULL, labels = NULL),
      labs = list(y = NULL, x = "view")
    )
  )

  ad_scale[[position]][[aesthetics]]
}

skip_if_not_installed("sf")

describe("adapt_scales", {
  dk_df <- as.data.frame(dk())
  dk_coords <- sf2coords(dk_df)
  dk_geo <- unnest(dk_coords, ggseg)

  aseg_df <- as.data.frame(aseg())
  aseg_coords <- sf2coords(aseg_df)
  aseg_geo <- unnest(aseg_coords, ggseg)

  it("returns labs scale for cortical atlas", {
    result <- adapt_scales(dk_geo)
    expect_type(result, "list")
    expect_true("y" %in% names(result))
    expect_true("x" %in% names(result))
  })

  it("returns x scale for cortical atlas dispersed", {
    result <- adapt_scales(dk_geo, position = "dispersed", aesthetics = "x")
    expect_type(result, "list")
    expect_true("breaks" %in% names(result))
    expect_true("labels" %in% names(result))
  })

  it("returns y scale for cortical atlas dispersed", {
    result <- adapt_scales(dk_geo, position = "dispersed", aesthetics = "y")
    expect_type(result, "list")
  })

  it("returns labs for cortical atlas stacked", {
    result <- adapt_scales(dk_geo, position = "stacked", aesthetics = "labs")
    expect_type(result, "list")
    expect_true("y" %in% names(result))
    expect_true("x" %in% names(result))
  })

  it("returns x scale for cortical atlas stacked", {
    result <- adapt_scales(dk_geo, position = "stacked", aesthetics = "x")
    expect_type(result, "list")
  })

  it("returns y scale for cortical atlas stacked", {
    result <- adapt_scales(dk_geo, position = "stacked", aesthetics = "y")
    expect_type(result, "list")
  })

  it("converts atlas object to data.frame automatically", {
    result <- adapt_scales(dk())
    expect_type(result, "list")
    expect_true("y" %in% names(result))
    expect_true("x" %in% names(result))
  })

  it("converts subcortical atlas object automatically", {
    result <- adapt_scales(aseg(), aesthetics = "x")
    expect_type(result, "list")
    expect_true("breaks" %in% names(result))
  })

  it("returns labs scale for subcortical atlas", {
    result <- adapt_scales(aseg_geo)
    expect_type(result, "list")
  })

  it("returns x scale for subcortical atlas dispersed", {
    result <- adapt_scales(aseg_geo, position = "dispersed", aesthetics = "x")
    expect_type(result, "list")
  })

  it("returns y scale for subcortical atlas stacked", {
    result <- adapt_scales(aseg_geo, position = "stacked", aesthetics = "y")
    expect_type(result, "list")
  })
})

#' Extract polygon coordinates from sf geometry
#'
#' Converts an sf geometry column into a data.frame of polygon
#' vertex coordinates with unique sub-ID and ordering columns.
#'
#' @param x An sf geometry object (e.g. `sfc_MULTIPOLYGON`).
#' @param n Integer row index used to generate unique `.subid` values.
#'
#' @return Data.frame with columns `.long`, `.lat`, `.subid`, `.id`,
#'   `.poly`, and `.order`.
#' @importFrom dplyr as_tibble group_by mutate row_number ungroup
#' @keywords internal
#' @noRd
to_coords <- function(x, n) {
  cols <- c(".long", ".lat", ".subid", ".id", ".poly", ".order")
  if (length(x) == 0) {
    k <- data.frame(matrix(
      nrow = 0,
      ncol = length(cols)
    ))
    names(k) <- cols
    return(k)
  }

  k <- sf::st_combine(x)
  k <- sf::st_coordinates(k)
  k <- as_tibble(k)
  k$L2 <- n * 10000 + k$L2
  k <- group_by(k, L2)
  k <- mutate(k, .order = row_number())
  k <- ungroup(k)
  names(k) <- cols

  k
}

#' Convert sf atlas data.frame to coordinate list-column
#'
#' Extracts vertex coordinates from each row's geometry into a
#' nested `ggseg` list-column, then drops the geometry column.
#'
#' @param x An sf data.frame with a `geometry` column.
#'
#' @return Data.frame with a `ggseg` list-column of coordinate
#'   data.frames and no `geometry` column.
#' @importFrom dplyr mutate row_number as_tibble
#' @keywords internal
#' @noRd
sf2coords <- function(x) {
  dt <- x
  dt$ggseg <- lapply(
    seq_len(nrow(x)),
    function(i) to_coords(x$geometry[[i]], i)
  )
  dt$geometry <- NULL
  dt
}

describe("to_coords", {
  it("returns empty data frame for empty input", {
    result <- to_coords(list(), 1)
    expect_s3_class(result, "data.frame")
    expect_identical(nrow(result), 0L)
    expect_named(result, c(".long", ".lat", ".subid", ".id", ".poly", ".order"))
  })

  it("converts geometry to coordinate data frame", {
    skip_if_not_installed("sf")
    atlas_df <- as.data.frame(dk())
    geom <- atlas_df$geometry[[1]]
    result <- to_coords(geom, 1)
    expect_s3_class(result, "data.frame")
    expect_gt(nrow(result), 0)
    expect_named(result, c(".long", ".lat", ".subid", ".id", ".poly", ".order"))
  })
})

describe("sf2coords", {
  it("converts sf data to coords format", {
    skip_if_not_installed("sf")
    sf_data <- atlas_sf(dk())
    result <- sf2coords(sf_data)
    expect_true("ggseg" %in% names(result))
    expect_type(result$ggseg, "list")
    expect_length(result$ggseg, nrow(sf_data))
  })

  it("removes geometry column", {
    skip_if_not_installed("sf")
    result <- sf2coords(atlas_sf(dk()))
    expect_false("geometry" %in% names(result))
  })
})

#' ggseg: Visualise brain atlas data with ggplot2
#'
#' Provides a ggplot2 geom and position for visualising brain region data on
#' cortical, subcortical, and white matter tract atlases. Brain atlas
#' geometries are stored as simple features (sf), enabling seamless
#' integration with the ggplot2 ecosystem including faceting, custom scales,
#' and themes.
#'
#' The main entry point is [geom_brain()], which accepts a `ggseg_atlas`
#' object and optional user data. Use [position_brain()] to control the
#' layout of brain slices/views.
#'
#' @name ggseg
#' @docType package
#' @keywords internal
#' @importFrom ggseg.formats atlas_palette dk aseg tracula suit
"_PACKAGE"

#' @export
ggseg.formats::dk

#' @export
ggseg.formats::aseg

#' @export
ggseg.formats::tracula

#' @export
ggseg.formats::suit

utils::globalVariables(c(
  ".id",
  ".lat",
  ".lat_sd",
  ".long",
  ".long_min",
  ".long_sd",
  ".subid",
  "hemi",
  "L2",
  "label",
  "view"
))

# Shared, coordinate-agnostic view layout ----
#
# The sf path (position-brain.R) and the polygon path (position-brain-polygon.R)
# arrange an atlas's view groups into a uniform grid with the *same* algorithm:
# gather each group to the cell origin, centre it in a cell sized to the largest
# group, and shift it to its grid position. That is a single translation per
# group, derivable from the group's bounding box alone — so the math lives here
# once and each path supplies only how to read a bbox and how to translate.

#' Per-group `(dx, dy)` offsets that tile view groups into uniform cells
#'
#' Combines the three steps the old `gather_*` / `center_view*` / `stack_*`
#' helpers did — gather to origin, centre in a uniform cell (sized to the
#' largest group, +20\% separation), and shift to the `(row, col)` grid cell —
#' into one translation per group. 1-D layouts pass a constant row or col index.
#'
#' @param bboxes Numeric matrix, one row per group, columns `xmin`, `ymin`,
#'   `xmax`, `ymax`.
#' @param row_idx,col_idx Integer 1-based cell indices, one per group.
#' @return Numeric matrix with columns `dx`, `dy`, one row per group.
#' @keywords internal
#' @noRd
layout_cell_offsets <- function(bboxes, row_idx, col_idx) {
  w <- bboxes[, "xmax"] - bboxes[, "xmin"]
  h <- bboxes[, "ymax"] - bboxes[, "ymin"]
  cell_w <- max(w)
  cell_h <- max(h)
  sep_x <- cell_w * 1.2
  sep_y <- cell_h * 1.2
  cbind(
    dx = -bboxes[, "xmin"] + (cell_w - w) / 2 + (col_idx - 1) * sep_x,
    dy = -bboxes[, "ymin"] + (cell_h - h) / 2 + (row_idx - 1) * sep_y
  )
}


#' Padded overall bounding box from per-group post-translation bboxes
#'
#' 1\% of the largest extent, anchored at the negative pad (matching the old
#' `get_box()` / `get_box_flat()`).
#'
#' @param bboxes Numeric matrix of post-translation group bboxes.
#' @return Named numeric `c(xmin, ymin, xmax, ymax)`.
#' @keywords internal
#' @noRd
layout_box <- function(bboxes) {
  pad <- max(bboxes) * 0.01
  c(
    xmin = -pad,
    ymin = -pad,
    xmax = max(bboxes[, "xmax"]) + pad,
    ymax = max(bboxes[, "ymax"]) + pad
  )
}


#' Resolve per-group cell indices and the row-major binding order
#'
#' @param groups List of per-view group data.frames.
#' @param posi `"rows"`, `"columns"`, or `"grid"`.
#' @param rows,columns Grid column names (grid mode only).
#' @return List with integer `row`/`col` (per group) and `order` (row-major
#'   bind order, dropping no groups — empty cells simply have no group).
#' @keywords internal
#' @noRd
cell_indices <- function(groups, posi, rows = NULL, columns = NULL) {
  n <- length(groups)
  if (posi == "columns") {
    return(list(row = rep(1L, n), col = seq_len(n), order = seq_len(n)))
  }
  if (posi == "rows") {
    return(list(row = seq_len(n), col = rep(1L, n), order = seq_len(n)))
  }
  lookup <- grid_lookup(groups, rows, columns)
  ri <- match(lookup$df_rows, lookup$row_vals)
  ci <- match(lookup$df_cols, lookup$col_vals)
  list(row = ri, col = ci, order = order(ri, ci))
}


#' Lay out view groups into uniform cells (sf- and polygon-agnostic)
#'
#' Shared driver for [frame_2_position()] (sf) and [frame_2_position_flat()]
#' (polygon). Reads each group's bbox via `bbox_of`, computes the offsets with
#' [layout_cell_offsets()], applies them via `translate`, and returns the groups
#' in row-major bind order plus the padded overall box.
#'
#' @param groups List of per-view group objects.
#' @param posi `"rows"`, `"columns"`, or `"grid"`.
#' @param rows,columns Grid column names (grid mode only).
#' @param bbox_of `function(group)` returning `c(xmin, ymin, xmax, ymax)`.
#' @param translate `function(group, dx, dy)` returning the shifted group.
#' @return List with `data` (translated groups, bind order) and `box`.
#' @keywords internal
#' @noRd
position_groups <- function(groups, posi, rows, columns, bbox_of, translate) {
  bboxes <- t(vapply(groups, bbox_of, numeric(4)))
  colnames(bboxes) <- c("xmin", "ymin", "xmax", "ymax")

  idx <- cell_indices(groups, posi, rows, columns)
  offs <- layout_cell_offsets(bboxes, idx$row, idx$col)

  moved <- lapply(seq_along(groups), function(i) {
    translate(groups[[i]], offs[i, "dx"], offs[i, "dy"])
  })
  # post-translation bboxes: dx shifts xmin/xmax, dy shifts ymin/ymax
  post <- bboxes + offs[, c("dx", "dy", "dx", "dy")]

  ord <- idx$order
  list(data = moved[ord], box = layout_box(post[ord, , drop = FALSE]))
}

describe("layout_cell_offsets", {
  it("gathers, centres in a uniform cell, and grid-shifts each group", {
    # group 1 is 10x10 at origin; group 2 is 4x4 at origin
    bboxes <- rbind(
      c(xmin = 0, ymin = 0, xmax = 10, ymax = 10),
      c(xmin = 0, ymin = 0, xmax = 4, ymax = 4)
    )
    offs <- layout_cell_offsets(bboxes, row_idx = c(1, 1), col_idx = c(1, 2))
    # cell is 10x10 (largest), sep_x = 12
    # g1: already fills the cell at origin -> no shift
    expect_equal(unname(offs[1, ]), c(0, 0))
    # g2: centred in the 10-wide cell ((10-4)/2 = 3) then +1 column (sep 12)
    expect_equal(unname(offs[2, ]), c(3 + 12, 3))
  })

  it("shifts down the rows for a vertical layout", {
    bboxes <- rbind(
      c(xmin = 0, ymin = 0, xmax = 10, ymax = 10),
      c(xmin = 0, ymin = 0, xmax = 10, ymax = 10)
    )
    offs <- layout_cell_offsets(bboxes, row_idx = c(1, 2), col_idx = c(1, 1))
    expect_equal(unname(offs[, "dx"]), c(0, 0))
    # second group drops one row; vertical separation is 12
    expect_equal(unname(offs[, "dy"]), c(0, 12))
  })

  it("folds a non-zero origin into the offset (gather)", {
    bboxes <- rbind(c(xmin = 100, ymin = 50, xmax = 110, ymax = 60))
    offs <- layout_cell_offsets(bboxes, row_idx = 1, col_idx = 1)
    expect_equal(unname(offs[1, ]), c(-100, -50))
  })
})

describe("layout_box", {
  it("pads the overall extent by 1% and anchors at the negative pad", {
    bboxes <- rbind(
      c(xmin = 0, ymin = 0, xmax = 10, ymax = 10),
      c(xmin = 12, ymin = 0, xmax = 22, ymax = 10)
    )
    box <- layout_box(bboxes)
    pad <- 22 * 0.01
    expect_equal(unname(box), c(-pad, -pad, 22 + pad, 10 + pad))
    expect_named(box, c("xmin", "ymin", "xmax", "ymax"))
  })
})

describe("cell_indices", {
  it("lays groups left-to-right for columns", {
    groups <- vector("list", 3)
    idx <- cell_indices(groups, "columns")
    expect_equal(idx$row, c(1, 1, 1))
    expect_equal(idx$col, c(1, 2, 3))
    expect_equal(idx$order, 1:3)
  })

  it("stacks groups top-to-bottom for rows", {
    groups <- vector("list", 3)
    idx <- cell_indices(groups, "rows")
    expect_equal(idx$row, c(1, 2, 3))
    expect_equal(idx$col, c(1, 1, 1))
  })

  it("maps grid groups to (row, col) indices and returns row-major order", {
    mk <- function(r, c) data.frame(.grid_row = r, .grid_col = c, x = 0, y = 0)
    # indices are positions in the first-appearance unique values
    groups <- list(mk(1, 1), mk(1, 2), mk(2, 1))
    idx <- cell_indices(groups, "grid", ".grid_row", ".grid_col")
    expect_equal(idx$row, c(1, 1, 2))
    expect_equal(idx$col, c(1, 2, 1))
    # already in row-major (row, col) order
    expect_equal(idx$order, c(1, 2, 3))
  })
})

describe("position_groups", {
  it("applies the offsets via the supplied translate + returns padded box", {
    g1 <- data.frame(x = c(0, 10), y = c(0, 10))
    g2 <- data.frame(x = c(0, 10), y = c(0, 10))
    res <- position_groups(
      list(g1, g2),
      "columns",
      NULL,
      NULL,
      bbox_of = bbox_flat,
      translate = function(g, dx, dy) {
        g$x <- g$x + dx
        g$y <- g$y + dy
        g
      }
    )
    expect_length(res$data, 2)
    # second group shifted one cell to the right (sep_x = 12)
    expect_equal(min(res$data[[2]]$x), 12)
    expect_named(res$box, c("xmin", "ymin", "xmax", "ymax"))
  })
})

#' Reposition brain slices
#'
#' Repositions pre-joined sf atlas data (i.e. data and atlas already joined to
#' a single sf data frame) for control over final plot layout. For even more
#' detailed control, convert the "hemi" and "view" columns into factors ordered
#' by wanted order of appearance.
#'
#' This is the sf layout helper. It requires the `sf` package (an optional
#' dependency); for the sf-free default, build a layout with [position_brain()]
#' and pass it to [geom_brain()].
#'
#' @param data sf-data.frame of joined brain atlas and data
#' @param position Position formula for slices. For cortical atlases, use
#'   formulas like `hemi ~ view`. For subcortical/tract atlases, use
#'   "horizontal", "vertical", or `type ~ .` for type-based layout.
#' @param nrow Number of rows for grid layout (subcortical/tract only)
#' @param ncol Number of columns for grid layout (subcortical/tract only)
#' @param views Character vector specifying view order (subcortical/tract only)
#'
#' @return sf-data.frame with re-positioned slices
#' @export
#'
#' @examplesIf requireNamespace("sf", quietly = TRUE)
#' reposition_brain(dk(), hemi ~ view)
#' reposition_brain(dk(), view ~ hemi)
#' reposition_brain(dk(), hemi + view ~ .)
#' reposition_brain(dk(), . ~ hemi + view)
#'
#' \donttest{
#' reposition_brain(aseg(), nrow = 2)
#' reposition_brain(aseg(), views = c("sagittal", "axial_3"))
#' }
reposition_brain <- function(
  data,
  position = "horizontal",
  nrow = NULL,
  ncol = NULL,
  views = NULL
) {
  require_sf("reposition_brain()")
  data <- as.data.frame(data, stringsAsFactors = FALSE)
  frame_2_position(
    data,
    position,
    nrow = nrow,
    ncol = ncol,
    views = views
  )
}


#' Arrange brain atlas views
#'
#' Controls how an atlas's hemispheres and views are arranged in the plot --
#' side by side, stacked, or in a grid -- and can zoom each view in on the
#' regions you care about. Pass the result to the `position` argument of
#' [geom_brain()] (or [annotate_brain()]).
#'
#' @param position Formula describing the rows ~ columns organisation for
#'   cortical atlases (e.g., `hemi ~ view`). For subcortical/tract atlases,
#'   can be "horizontal", "vertical", or a formula with `type ~ .` where type
#'   is extracted from view names like "axial_1" -> "axial".
#' @param nrow Number of rows for grid layout. If NULL (default), calculated
#'   automatically. Only used for subcortical/tract atlases when position is
#'   not a formula.
#' @param ncol Number of columns for grid layout. If NULL (default), calculated
#'   automatically. Only used for subcortical/tract atlases when position is
#'   not a formula.
#' @param views Character vector specifying which views to include and their
#'   order. If NULL (default), all views are included in their original order.
#'   Only applies to subcortical/tract atlases.
#' @param zoom Controls per-view zoom. `NULL`/`FALSE` (default) draws each view
#'   at full extent. `TRUE` zooms each view onto its focus regions; a character
#'   vector names the focus regions explicitly.
#' @param zoom_pad Fractional padding added around the focus window when `zoom`
#'   is active. Defaults to `0.05` (5\%).
#'
#' @export
#' @return A layout specification to hand to [geom_brain()]'s `position`
#'   argument.
#' @examples
#' library(ggplot2)
#'
#' # Cortical atlas with formula
#' ggplot() +
#'   geom_brain(
#'     atlas = dk(), aes(fill = region),
#'     position = position_brain(. ~ view + hemi),
#'     show.legend = FALSE
#'   )
#'
#' ggplot() +
#'   geom_brain(
#'     atlas = dk(), aes(fill = region),
#'     position = position_brain(view ~ hemi),
#'     show.legend = FALSE
#'   )
#'
#' ggplot() +
#'   geom_brain(
#'     atlas = aseg(), aes(fill = region),
#'     position = position_brain(nrow = 2)
#'   )
#'
#' ggplot() +
#'   geom_brain(
#'     atlas = aseg(), aes(fill = region),
#'     position = position_brain(
#'       views = c("sagittal", "axial_3", "coronal_2"),
#'       nrow = 1
#'     )
#'   )
#'
#' ggplot() +
#'   geom_brain(
#'     atlas = aseg(), aes(fill = region),
#'     position = position_brain(type ~ .)
#'   )
position_brain <- function(
  position = "horizontal",
  nrow = NULL,
  ncol = NULL,
  views = NULL,
  zoom = NULL,
  zoom_pad = 0.05
) {
  position_brain_polygon(
    position = position,
    nrow = nrow,
    ncol = ncol,
    views = views,
    zoom = zoom,
    zoom_pad = zoom_pad
  )
}

#' Deprecated sf brain-view layout
#'
#' @description
#' `r lifecycle::badge("deprecated")`
#'
#' The sf rendering path is deprecated. `position_brain_sf()` returns the
#' legacy `PositionBrain` ggproto for use with [geom_brain_sf()]. For new
#' code, use [position_brain()] (the polygon default), or convert the atlas
#' with `as_sf_atlas()` and use [ggplot2::geom_sf()] directly.
#'
#' @inheritParams position_brain
#' @export
#' @return A `PositionBrain` ggproto object.
#' @importFrom ggplot2 ggproto
#' @keywords internal
position_brain_sf <- function(
  position = "horizontal",
  nrow = NULL,
  ncol = NULL,
  views = NULL
) {
  lifecycle::deprecate_warn(
    "2.2.0",
    "position_brain_sf()",
    details = paste(
      "Use `position_brain()` for the polygon default, or `as_sf_atlas()`",
      "with `ggplot2::geom_sf()` for an sf workflow."
    )
  )
  require_sf("position_brain_sf()")
  make_position_brain_sf(position, nrow = nrow, ncol = ncol, views = views)
}

#' Construct a PositionBrain ggproto without the deprecation warning
#'
#' @keywords internal
#' @noRd
#' @importFrom ggplot2 ggproto
make_position_brain_sf <- function(
  position = "horizontal",
  nrow = NULL,
  ncol = NULL,
  views = NULL
) {
  ggproto(
    NULL,
    PositionBrain,
    position = position,
    nrow = nrow,
    ncol = ncol,
    views = views
  )
}

#' ggproto Position class for brain atlas layout
#'
#' Handles coordinate repositioning of brain views/slices during
#' the ggplot2 rendering pipeline. Created by [position_brain()].
#'
#' @keywords internal
#' @importFrom ggplot2 ggproto Position
#' @noRd
PositionBrain <- ggproto(
  "PositionBrain",
  Position,
  position = hemi + view ~ .,
  nrow = NULL,
  ncol = NULL,
  views = NULL,
  setup_params = function(self, data) {
    list(
      position = self$position,
      nrow = self$nrow,
      ncol = self$ncol,
      views = self$views
    )
  },
  compute_layer = function(self, data, params, layout) {
    df3 <- frame_2_position(
      data,
      params$position,
      nrow = params$nrow,
      ncol = params$ncol,
      views = params$views
    )
    bbx <- sf::st_bbox(df3$geometry)

    if (is.null(layout$coord$limits$y)) {
      layout$coord$limits$y <- bbx[c(2, 4)]
    }

    if (is.null(layout$coord$limits$x)) {
      layout$coord$limits$x <- bbx[c(1, 3)]
    }

    df3
  }
)

# geometry movers ----

#' Extract and validate variable names from a position formula
#'
#' @param pos A formula describing the layout.
#' @return Character vector of variable names (excluding `.`).
#' @keywords internal
#' @noRd
parse_formula_vars <- function(pos) {
  chosen <- all.vars(pos, unique = FALSE)
  chosen <- chosen[!grepl(".", chosen, fixed = TRUE)]

  if (anyDuplicated(chosen)) {
    cli::cli_abort(
      "Cannot position brain with the same data as columns and rows"
    )
  }
  chosen
}

#' Detect stacking direction from a formula with `+`
#'
#' For formulas like `hemi + view ~ .` or `. ~ hemi + view`,
#' determines whether the stacked layout is row-based or column-based.
#'
#' @param pos A formula containing `+` on one side and `.` on the other.
#' @return `"rows"` or `"columns"`.
#' @keywords internal
#' @noRd
stacking_direction <- function(pos) {
  if (grepl("~\\s*\\.", deparse(pos))) "rows" else "columns"
}

#' Validate that a single-direction formula includes both `.` and `~`
#'
#' @param pos A formula.
#' @param position The resolved position (`"rows"`, `"columns"`, or vars).
#' @keywords internal
#' @noRd
validate_stacking_formula <- function(pos, position) {
  is_single <- length(position) == 1 && position %in% c("rows", "columns")
  if (!is_single) {
    return(invisible())
  }

  has_both <- sum(grepl("\\.|~", pos)) == 2
  if (!has_both) {
    cli::cli_abort(
      "Formula for a single row or column must contain both a '.' and '~'"
    )
  }
}

#' Parse a position formula into layout instructions
#'
#' Interprets a formula like `hemi ~ view` into row/column
#' variable names and validates against the atlas type.
#'
#' @param pos A formula describing the layout.
#' @param data Data.frame with atlas columns (`type`, `hemi`, `view`).
#'
#' @return A list with `position` (character), `chosen` (variable names),
#'   and `data` (possibly modified data.frame).
#' @keywords internal
#' @noRd
position_formula <- function(pos, data) {
  chosen <- parse_formula_vars(pos)
  atlas_type <- unique(data$type)[1]

  if (atlas_type == "cortical") {
    position <- position_cortical(pos, chosen)
  } else {
    result <- position_subcortical(pos, chosen, data)
    position <- result$position
    chosen <- result$chosen
    data <- result$data
  }

  validate_stacking_formula(pos, position)
  list(position = position, chosen = chosen, data = data)
}

#' Resolve position for a cortical atlas formula
#'
#' @param pos A formula.
#' @param chosen Character vector of variable names.
#' @return Position specification: variable names or `"rows"`/`"columns"`.
#' @keywords internal
#' @noRd
position_cortical <- function(pos, chosen) {
  if (length(chosen) < 2) {
    missing_vars <- c("view", "hemi")[!c("view", "hemi") %in% chosen]
    cli::cli_abort(c(
      "Position formula not correct.",
      "x" = paste("Missing:", paste(missing_vars, collapse = " & "))
    ))
  }
  if (any(grepl("+", pos, fixed = TRUE))) {
    chosen <- stacking_direction(pos)
  }
  chosen
}

#' Resolve position for a subcortical/tract atlas formula
#'
#' @param pos A formula.
#' @param chosen Character vector of variable names.
#' @param data Data.frame with atlas columns.
#' @return A list with `position`, `chosen`, and `data`.
#' @keywords internal
#' @noRd
position_subcortical <- function(pos, chosen, data) {
  if ("type" %in% chosen) {
    data$.view_type <- extract_view_type(data$view)
    chosen[chosen == "type"] <- ".view_type"
  }

  position <- if (length(chosen) == 1) {
    stacking_direction(pos)
  } else {
    chosen
  }

  list(
    position = position,
    chosen = chosen,
    data = data
  )
}


#' Extract the type prefix from view names
#'
#' Splits view names like `"axial_3"` on underscore and returns
#' the first part (`"axial"`).
#'
#' @param views Character vector of view names.
#'
#' @return Character vector of type prefixes.
#' @keywords internal
#' @noRd
extract_view_type <- function(views) {
  vapply(
    views,
    function(v) {
      parts <- strsplit(v, "_", fixed = TRUE)[[1]]
      if (length(parts) >= 1) parts[1] else v # nocov
    },
    character(1),
    USE.NAMES = FALSE
  )
}

#' Reposition brain views according to layout specification
#'
#' Main dispatcher that splits data by view/hemisphere, gathers
#' geometry, and delegates to the appropriate stacking function.
#'
#' @param data Data.frame with atlas columns and `geometry`.
#' @param pos Position specification: a formula, `"horizontal"`,
#'   or `"vertical"`.
#' @param nrow Number of grid rows (optional).
#' @param ncol Number of grid columns (optional).
#' @param views Character vector of views to include.
#'
#' @return An sf data.frame with repositioned geometry and
#'   adjusted bounding box.
#' @keywords internal
#' @noRd
frame_2_position <- function(
  data,
  pos,
  nrow = NULL,
  ncol = NULL,
  views = NULL
) {
  if (!is.null(views)) {
    data <- data[data$view %in% views, , drop = FALSE]
    data$view <- factor(data$view, levels = views)
    data <- data[order(data$view), ]
    data$view <- as.character(data$view)
  }

  if (!is.null(nrow) || !is.null(ncol)) {
    dfpos <- split_data_grid(data, nrow, ncol)
  } else {
    dfpos <- split_data(data, pos)
  }

  posi <- if (length(dfpos$position) > 1) "grid" else dfpos$position
  rows <- if (posi == "grid") dfpos$position[1] else NULL
  columns <- if (posi == "grid") dfpos$position[2] else NULL

  res <- position_groups(
    dfpos$data,
    posi,
    rows,
    columns,
    bbox_of = function(g) as.numeric(sf::st_bbox(g$geometry)),
    translate = function(g, dx, dy) {
      g$geometry <- g$geometry + c(dx, dy)
      g
    }
  )

  out <- do.call(rbind, res$data)
  if (posi == "grid") {
    out <- drop_temp_columns(out)
  }
  df4 <- sf::st_as_sf(out)
  box <- res$box
  class(box) <- "bbox"
  attr(sf::st_geometry(df4), "bbox") <- box

  df4
}


#' Split atlas data into a grid of views
#'
#' Assigns grid row/column indices to each view and returns
#' a list of per-view data.frames.
#'
#' @param data Data.frame with a `view` column.
#' @param nrow Number of grid rows (optional, auto-calculated).
#' @param ncol Number of grid columns (optional, auto-calculated).
#'
#' @return A list with `data` (list of data.frames) and
#'   `position` (column names for grid coordinates).
#' @keywords internal
#' @noRd
split_data_grid <- function(data, nrow = NULL, ncol = NULL) {
  view_list <- unique(data$view)
  n_views <- length(view_list)

  if (is.null(nrow) && is.null(ncol)) {
    ncol <- ceiling(sqrt(n_views))
    nrow <- ceiling(n_views / ncol)
  } else if (is.null(nrow)) {
    nrow <- ceiling(n_views / ncol)
  } else if (is.null(ncol)) {
    ncol <- ceiling(n_views / nrow)
  }

  data$.grid_row <- ((seq_along(view_list) - 1) %/% ncol + 1)[
    match(data$view, view_list)
  ]
  data$.grid_col <- ((seq_along(view_list) - 1) %% ncol + 1)[
    match(data$view, view_list)
  ]

  df_list <- lapply(view_list, function(v) {
    data[data$view == v, ]
  })

  list(
    data = df_list,
    position = c(".grid_row", ".grid_col")
  )
}

#' Split atlas data by position specification
#'
#' Routes to formula-based or string-based splitting depending
#' on whether `position` is a formula or character.
#'
#' @param data Data.frame with atlas columns.
#' @param position A formula or character layout specification.
#'
#' @return A list with `data` (list of data.frames) and
#'   `position` (layout direction or variable names).
#' @keywords internal
#' @noRd
split_data <- function(data, position) {
  if (inherits(position, "formula")) {
    split_data_formula(data, position)
  } else {
    split_data_string(data, position)
  }
}

#' Split atlas data by a position formula (e.g. `hemi ~ view`)
#'
#' @return A list with `data` (one data.frame per group) and `position`
#'   (the resolved layout direction or variable names).
#' @keywords internal
#' @noRd
split_data_formula <- function(data, position) {
  pos <- position_formula(position, data)
  if (!is.null(pos$data)) {
    data <- pos$data
  }
  groups <- dplyr::group_split(dplyr::group_by_at(data, pos$chosen))
  list(data = groups, position = pos$position)
}

#' Split atlas data by a string layout
#'
#' `position` is either `"horizontal"`/`"vertical"` (expanded to the atlas's
#' default view order) or pre-built `"hemi view"` / `"view"` identifiers.
#'
#' @inherit split_data_formula return
#' @keywords internal
#' @noRd
split_data_string <- function(data, position) {
  layout_direction <- "columns"
  if (length(position) == 1 && position %in% c("horizontal", "vertical")) {
    layout_direction <- if (position == "vertical") "rows" else "columns"
    position <- default_order(data)
  }

  pos <- as.data.frame(
    strsplit(position, " ", fixed = TRUE),
    stringsAsFactors = FALSE
  )

  groups <- if (identical(unique(data$type)[1], "cortical")) {
    split_cortical_pairs(data, pos)
  } else {
    lapply(pos, function(view) data[data$view == view, ])
  }

  list(data = groups, position = layout_direction)
}

#' Subset cortical data into its existing hemi/view pairs
#'
#' Columns of `pos` are `c(hemi, view)` pairs; keep only the pairs the atlas
#' actually contains, then subset the data to each.
#' @keywords internal
#' @noRd
split_cortical_pairs <- function(data, pos) {
  has_pair <- (pos[1, ] %in% data$hemi) & (pos[2, ] %in% data$view)
  pos <- pos[has_pair]
  lapply(pos, function(pair) {
    data[data$hemi == pair[1] & data$view == pair[2], ]
  })
}

#' Build a lookup of row/column values per data.frame element
#'
#' @param df List of data.frames from a split atlas.
#' @param rows Column name for the row variable.
#' @param columns Column name for the column variable.
#'
#' @return A list with `df_rows`, `df_cols` (per-element values),
#'   and `row_vals`, `col_vals` (unique levels).
#' @keywords internal
#' @noRd
grid_lookup <- function(df, rows, columns) {
  as_char <- function(x) {
    if (is.numeric(x)) as.character(x) else x
  }
  df_rows <- vapply(
    df,
    function(x) as_char(unique(x[[rows]])),
    character(1)
  )
  df_cols <- vapply(
    df,
    function(x) as_char(unique(x[[columns]])),
    character(1)
  )
  list(
    df_rows = df_rows,
    df_cols = df_cols,
    row_vals = unique(df_rows),
    col_vals = unique(df_cols)
  )
}

#' Remove temporary columns added during grid layout
#'
#' @param df A data.frame.
#' @return The data.frame with temp columns removed.
#' @keywords internal
#' @noRd
drop_temp_columns <- function(df) {
  temp <- c(
    "xmin",
    "xmax",
    "ymin",
    "ymax",
    ".grid_row",
    ".grid_col",
    ".view_type"
  )
  temp <- temp[temp %in% names(df)]
  if (length(temp) > 0) {
    df[, temp] <- NULL
  }
  df
}

#' Generate the default view ordering for an atlas
#'
#' For cortical atlases, returns `"hemi view"` pairs (left first).
#' For subcortical/tract atlases, returns unique views as-is.
#'
#' @param data Data.frame with `type`, `view`, and `hemi` columns.
#'
#' @return Character vector of ordered view identifiers.
#' @keywords internal
#' @noRd
default_order <- function(data) {
  if (unique(data$type) != "cortical") {
    return(unique(data$view))
  }
  sides <- unique(data$view)
  left_sides <- sides[sides %in% unique(data$view[data$hemi == "left"])]
  right_sides <- sides[sides %in% unique(data$view[data$hemi == "right"])]
  left_views <- paste("left", left_sides)
  right_views <- paste("right", right_sides)
  c(left_views, right_views)
}

test_that("position_formula works", {
  expect_error(
    position_formula(hemi ~ hemi, as.data.frame(dk())),
    "Cannot position brain"
  )

  expect_error(
    position_formula(bla ~ ., as.data.frame(dk())),
    "formula not correct"
  )

  k <- position_formula(hemi ~ view, as.data.frame(dk()))
  expect_true(all(c("position", "chosen") %in% names(k)))
  expect_identical(k$position, c("hemi", "view"))
  expect_identical(k$chosen, c("hemi", "view"))

  k <- position_formula(view ~ hemi, as.data.frame(dk()))
  expect_true(all(c("position", "chosen") %in% names(k)))
  expect_identical(k$position, c("view", "hemi"))
  expect_identical(k$chosen, c("view", "hemi"))

  k <- position_formula(. ~ hemi + view, as.data.frame(dk()))
  expect_true(all(c("position", "chosen") %in% names(k)))
  expect_identical(k$position, "columns")
  expect_identical(k$chosen, c("hemi", "view"))

  k <- position_formula(hemi + view ~ ., as.data.frame(dk()))
  expect_true(all(c("position", "chosen") %in% names(k)))
  expect_identical(k$position, "rows")
  expect_identical(k$chosen, c("hemi", "view"))
})

describe("reposition_brain", {
  it("converts data to data.frame", {
    skip_if_not_installed("sf")
    result <- reposition_brain(dk(), hemi ~ view)
    expect_s3_class(result, "sf")
  })

  it("works with formula position", {
    skip_if_not_installed("sf")
    result <- reposition_brain(dk(), view ~ hemi)
    expect_s3_class(result, "sf")
  })

  it("works with hemi + view formula", {
    skip_if_not_installed("sf")
    result <- reposition_brain(dk(), hemi + view ~ .)
    expect_s3_class(result, "sf")
  })
})

describe("position_brain", {
  it("returns a polygon layout spec", {
    pos <- position_brain()
    expect_s3_class(pos, "position_brain_polygon_spec")
  })

  it("accepts formula position", {
    pos <- position_brain(hemi ~ view)
    expect_s3_class(pos, "position_brain_polygon_spec")
  })

  it("accepts horizontal position string", {
    pos <- position_brain("horizontal")
    expect_s3_class(pos, "position_brain_polygon_spec")
  })

  it("accepts vertical position string", {
    pos <- position_brain("vertical")
    expect_s3_class(pos, "position_brain_polygon_spec")
  })
})

describe("position_brain_sf (deprecated)", {
  it("warns and returns a PositionBrain ggproto", {
    lifecycle::expect_deprecated(pos <- position_brain_sf(hemi ~ view))
    expect_s3_class(pos, "PositionBrain")
    expect_identical(pos$position, hemi ~ view)
  })
})

describe("split_data", {
  it("works with horizontal character position", {
    data <- as.data.frame(dk())
    result <- split_data(data, "horizontal")
    expect_type(result, "list")
    expect_named(result, c("data", "position"))
  })

  it("works with vertical character position", {
    data <- as.data.frame(dk())
    result <- split_data(data, "vertical")
    expect_type(result, "list")
    expect_identical(result$position, "rows")
  })
})

describe("default_order", {
  it("returns order for cortical data", {
    data <- as.data.frame(dk())
    result <- default_order(data)
    expect_type(result, "character")
    expect_true(any(grepl("left", result, fixed = TRUE)))
    expect_true(any(grepl("right", result, fixed = TRUE)))
  })

  it("returns views for subcortical data", {
    data <- as.data.frame(aseg())
    result <- default_order(data)
    expect_type(result, "character")
  })
})

describe("split_data with subcortical", {
  it("works with subcortical atlas positions", {
    data <- as.data.frame(aseg())
    result <- split_data(data, "horizontal")
    expect_type(result, "list")
    expect_named(result, c("data", "position"))
  })
})


describe("position_formula edge cases", {
  it("errors when formula missing '.' for single row/column", {
    data <- as.data.frame(dk())
    expect_error(
      position_formula(hemi + view ~ foo, data),
      "must contain both"
    )
  })
})


describe("position_brain with nrow/ncol", {
  it("accepts nrow parameter", {
    pos <- position_brain(nrow = 2)
    expect_s3_class(pos, "position_brain_polygon_spec")
    expect_identical(pos$nrow, 2)
  })

  it("accepts ncol parameter", {
    pos <- position_brain(ncol = 3)
    expect_s3_class(pos, "position_brain_polygon_spec")
    expect_identical(pos$ncol, 3)
  })

  it("accepts both nrow and ncol", {
    pos <- position_brain(nrow = 2, ncol = 3)
    expect_s3_class(pos, "position_brain_polygon_spec")
    expect_identical(pos$nrow, 2)
    expect_identical(pos$ncol, 3)
  })

  it("accepts views parameter", {
    pos <- position_brain(views = c("axial_3", "sagittal"))
    expect_s3_class(pos, "position_brain_polygon_spec")
    expect_identical(pos$views, c("axial_3", "sagittal"))
  })
})


describe("split_data_grid", {
  it("creates grid layout for subcortical data", {
    data <- as.data.frame(aseg())
    result <- split_data_grid(data, nrow = 2)
    expect_type(result, "list")
    expect_named(result, c("data", "position"))
    expect_identical(result$position, c(".grid_row", ".grid_col"))
  })

  it("respects ncol parameter", {
    data <- as.data.frame(aseg())
    result <- split_data_grid(data, ncol = 3)
    expect_type(result, "list")
    expect_identical(result$position, c(".grid_row", ".grid_col"))
  })
})


describe("reposition_brain with subcortical", {
  it("works with nrow parameter", {
    skip_if_not_installed("sf")
    data <- as.data.frame(aseg())
    result <- reposition_brain(data, nrow = 2)
    expect_s3_class(result, "sf")
  })

  it("works with views parameter", {
    skip_if_not_installed("sf")
    data <- as.data.frame(aseg())
    views <- unique(data$view)[1:3]
    result <- reposition_brain(data, views = views)
    expect_s3_class(result, "sf")
    expect_true(all(result$view %in% views))
  })

  it("respects view order", {
    skip_if_not_installed("sf")
    data <- as.data.frame(aseg())
    views <- rev(unique(data$view)[1:3])
    result <- reposition_brain(data, views = views)
    result_views <- unique(result$view)
    expect_identical(result_views, views)
  })
})


describe("position_formula with subcortical", {
  it("handles type ~ . formula", {
    data <- as.data.frame(aseg())
    k <- position_formula(type ~ ., data)
    expect_true("position" %in% names(k))
    expect_identical(k$position, "rows")
  })

  it("handles . ~ type formula", {
    data <- as.data.frame(aseg())
    k <- position_formula(. ~ type, data)
    expect_true("position" %in% names(k))
    expect_identical(k$position, "columns")
  })

  it("handles view ~ . formula for subcortical", {
    data <- as.data.frame(aseg())
    k <- position_formula(view ~ ., data)
    expect_identical(k$position, "rows")
  })

  it("handles . ~ view formula for subcortical", {
    data <- as.data.frame(aseg())
    k <- position_formula(. ~ view, data)
    expect_identical(k$position, "columns")
  })
})

describe("split_data_grid with defaults", {
  it("auto-calculates nrow and ncol when both NULL", {
    data <- as.data.frame(aseg())
    result <- split_data_grid(data)
    expect_type(result, "list")
    expect_identical(result$position, c(".grid_row", ".grid_col"))
  })
})

describe("reposition_brain subcortical formula", {
  it("works with type ~ . formula", {
    skip_if_not_installed("sf")
    data <- as.data.frame(aseg())
    result <- reposition_brain(data, type ~ .)
    expect_s3_class(result, "sf")
  })

  it("works with . ~ type formula", {
    skip_if_not_installed("sf")
    data <- as.data.frame(aseg())
    result <- reposition_brain(data, . ~ type)
    expect_s3_class(result, "sf")
  })
})

describe("position_formula subcortical multi-var", {
  it("handles two-variable formula for subcortical", {
    data <- as.data.frame(aseg())
    data$hemi <- "left"
    k <- position_formula(view ~ hemi, data)
    expect_identical(k$position, c("view", "hemi"))
  })
})

describe("stack_grid numeric sorting", {
  it("works with numeric grid positions", {
    skip_if_not_installed("sf")
    data <- as.data.frame(aseg())
    result <- reposition_brain(data, nrow = 2, ncol = 4)
    expect_s3_class(result, "sf")
  })
})

describe("extract_view_type", {
  it("extracts type from view names", {
    views <- c("axial_1", "axial_2", "coronal_1", "sagittal")
    types <- extract_view_type(views)
    expect_identical(types, c("axial", "axial", "coronal", "sagittal"))
  })

  it("handles views without underscore", {
    views <- c("sagittal", "coronal")
    types <- extract_view_type(views)
    expect_identical(types, c("sagittal", "coronal"))
  })
})

describe("parse_formula_vars", {
  it("extracts variable names from a formula", {
    expect_identical(parse_formula_vars(hemi ~ view), c("hemi", "view"))
  })

  it("excludes dot placeholder", {
    expect_identical(parse_formula_vars(hemi + view ~ .), c("hemi", "view"))
    expect_identical(parse_formula_vars(. ~ hemi + view), c("hemi", "view"))
  })

  it("errors on duplicate variables", {
    expect_error(parse_formula_vars(hemi ~ hemi), "Cannot position brain")
  })
})

describe("stacking_direction", {
  it("returns rows when dot is on RHS", {
    expect_identical(stacking_direction(hemi + view ~ .), "rows")
  })

  it("returns columns when dot is on LHS", {
    expect_identical(stacking_direction(. ~ hemi + view), "columns")
  })
})

describe("validate_stacking_formula", {
  it("passes silently for grid positions", {
    expect_invisible(validate_stacking_formula(hemi ~ view, c("hemi", "view")))
  })

  it("passes for valid stacking formula", {
    expect_silent(validate_stacking_formula(hemi + view ~ ., "rows"))
  })

  it("errors when dot or tilde is missing", {
    expect_error(
      validate_stacking_formula(hemi + view ~ foo, "rows"),
      "must contain both"
    )
  })
})

describe("position_cortical", {
  it("returns variable names for two-var formula", {
    expect_identical(
      position_cortical(hemi ~ view, c("hemi", "view")),
      c("hemi", "view")
    )
  })

  it("detects stacking direction with plus", {
    expect_identical(
      position_cortical(hemi + view ~ ., c("hemi", "view")),
      "rows"
    )
    expect_identical(
      position_cortical(. ~ hemi + view, c("hemi", "view")),
      "columns"
    )
  })

  it("errors with fewer than two variables", {
    expect_error(position_cortical(hemi ~ ., "hemi"), "formula not correct")
  })
})

describe("position_subcortical", {
  it("remaps type to .view_type", {
    data <- as.data.frame(aseg())
    result <- position_subcortical(type ~ ., "type", data)
    expect_identical(result$chosen, ".view_type")
    expect_true(".view_type" %in% names(result$data))
  })

  it("detects stacking direction for single var", {
    data <- as.data.frame(aseg())
    result <- position_subcortical(view ~ ., "view", data)
    expect_identical(result$position, "rows")
  })

  it("returns chosen vars for two-var formula", {
    data <- as.data.frame(aseg())
    data$hemi <- "left"
    result <- position_subcortical(view ~ hemi, c("view", "hemi"), data)
    expect_identical(result$position, c("view", "hemi"))
  })
})

describe("grid_lookup", {
  it("extracts row and column values from split data", {
    data <- as.data.frame(aseg())
    split_result <- split_data_grid(data, nrow = 2)
    lookup <- grid_lookup(split_result$data, ".grid_row", ".grid_col")
    expect_named(lookup, c("df_rows", "df_cols", "row_vals", "col_vals"))
    expect_length(lookup$df_rows, length(split_result$data))
    expect_length(lookup$df_cols, length(split_result$data))
  })

  it("coerces numeric values to character", {
    data <- as.data.frame(aseg())
    split_result <- split_data_grid(data, nrow = 2)
    lookup <- grid_lookup(split_result$data, ".grid_row", ".grid_col")
    expect_type(lookup$df_rows, "character")
    expect_type(lookup$df_cols, "character")
  })
})

describe("drop_temp_columns", {
  it("removes temporary grid columns", {
    df <- data.frame(
      x = 1:3,
      .grid_row = 1:3,
      .grid_col = 1:3,
      .view_type = "a"
    )
    result <- drop_temp_columns(df)
    expect_named(result, "x")
  })

  it("leaves data unchanged when no temp columns present", {
    df <- data.frame(x = 1:3, y = 4:6)
    result <- drop_temp_columns(df)
    expect_identical(result, df)
  })
})

describe("extract_position_params", {
  it("extracts from PositionBrain object", {
    skip_if_not_installed("sf")
    withr::local_options(lifecycle_verbosity = "quiet")
    pos <- position_brain_sf(hemi ~ view, nrow = 2, ncol = 3, views = "lateral")
    params <- extract_position_params(pos)
    expect_identical(params$position, hemi ~ view)
    expect_identical(params$nrow, 2)
    expect_identical(params$ncol, 3)
    expect_identical(params$views, "lateral")
  })

  it("wraps raw string position", {
    params <- extract_position_params("horizontal")
    expect_identical(params$position, "horizontal")
    expect_null(params$nrow)
    expect_null(params$ncol)
    expect_null(params$views)
  })

  it("wraps raw formula position", {
    params <- extract_position_params(hemi ~ view)
    expect_identical(params$position, hemi ~ view)
    expect_null(params$nrow)
  })
})

describe("compute_label_positions", {
  it("produces hemi + view labels for cortical", {
    skip_if_not_installed("sf")
    repositioned <- reposition_brain(dk(), hemi ~ view)
    label_df <- compute_label_positions(repositioned)

    expect_s3_class(label_df, "data.frame")
    expect_named(label_df, c("x", "y", "label"))
    dk_df <- as.data.frame(dk())
    n_combos <- length(unique(paste(dk_df$hemi, dk_df$view)))
    expect_identical(nrow(label_df), n_combos)
    expect_true(all(
      c("left lateral", "left medial", "right lateral", "right medial") %in%
        label_df$label
    ))
  })

  it("produces view labels for subcortical", {
    skip_if_not_installed("sf")
    repositioned <- reposition_brain(aseg())
    label_df <- compute_label_positions(repositioned)

    expect_s3_class(label_df, "data.frame")
    views <- unique(as.data.frame(aseg())$view)
    expect_identical(nrow(label_df), length(views))
    expect_true(all(views %in% label_df$label))
  })

  it("produces view labels for tract", {
    skip_if_not_installed("sf")
    repositioned <- reposition_brain(tracula())
    label_df <- compute_label_positions(repositioned)

    views <- unique(as.data.frame(tracula())$view)
    expect_identical(nrow(label_df), length(views))
    expect_true(all(views %in% label_df$label))
  })
})

describe("annotate_brain dispatch", {
  it("returns a polygon-path layer by default", {
    layer <- annotate_brain(atlas = dk())
    expect_s3_class(layer, "LayerInstance")
  })

  it("pairs with geom_brain_polygon() by default", {
    p <- ggplot() +
      geom_brain_polygon(atlas = dk(), show.legend = FALSE) +
      annotate_brain(atlas = dk())
    expect_silent(ggplot2::ggplot_build(p))
  })

  it("routes to the sf path when given a position_brain_sf()", {
    skip_if_not_installed("sf")
    withr::local_options(lifecycle_verbosity = "quiet")
    layer <- annotate_brain(atlas = dk(), position = position_brain_sf())
    expect_s3_class(layer, "LayerInstance")
  })
})

describe("annotate_brain (polygon path)", {
  it("returns an annotation layer", {
    layer <- annotate_brain(atlas = dk(), position = position_brain())
    expect_s3_class(layer, "LayerInstance")
  })

  it("builds a valid plot with cortical atlas", {
    p <- ggplot() +
      geom_brain(atlas = dk(), show.legend = FALSE) +
      annotate_brain(atlas = dk(), position = position_brain())
    expect_s3_class(p, "gg")
    expect_silent(ggplot2::ggplot_build(p))
  })

  it("builds a valid plot with subcortical atlas", {
    p <- ggplot() +
      geom_brain(atlas = aseg(), show.legend = FALSE) +
      annotate_brain(atlas = aseg(), position = position_brain())
    expect_s3_class(p, "gg")
    expect_silent(ggplot2::ggplot_build(p))
  })

  it("respects hemi filtering", {
    layer <- annotate_brain(
      atlas = dk(),
      hemi = "left",
      position = position_brain()
    )
    built <- ggplot() +
      geom_brain(atlas = dk(), hemi = "left", show.legend = FALSE) +
      layer
    bd <- ggplot2::ggplot_build(built)
    labels <- bd$data[[2]]$label
    expect_true(all(grepl("^left", labels)))
  })

  it("respects view filtering", {
    layer <- annotate_brain(
      atlas = dk(),
      view = "lateral",
      position = position_brain()
    )
    built <- ggplot() +
      geom_brain(atlas = dk(), view = "lateral", show.legend = FALSE) +
      layer
    bd <- ggplot2::ggplot_build(built)
    labels <- bd$data[[2]]$label
    expect_true(all(grepl("lateral$", labels)))
  })

  it("works with position formula", {
    p <- ggplot() +
      geom_brain(
        atlas = dk(),
        position = position_brain(hemi ~ view),
        show.legend = FALSE
      ) +
      annotate_brain(
        atlas = dk(),
        position = position_brain(hemi ~ view)
      )
    expect_silent(ggplot2::ggplot_build(p))
  })

  it("works with nrow/ncol for subcortical", {
    p <- ggplot() +
      geom_brain(
        atlas = aseg(),
        position = position_brain(nrow = 2),
        show.legend = FALSE
      ) +
      annotate_brain(
        atlas = aseg(),
        position = position_brain(nrow = 2)
      )
    expect_silent(ggplot2::ggplot_build(p))
  })

  it("passes styling arguments", {
    layer <- annotate_brain(
      atlas = dk(),
      position = position_brain(),
      size = 5,
      colour = "red",
      fontface = "bold"
    )
    expect_s3_class(layer, "LayerInstance")
  })
})

describe("annotate_brain visual", {
  it("dk default with labels", {
    expect_doppelganger(
      "dk default labels",
      ggplot() +
        geom_brain(atlas = dk(), show.legend = FALSE) +
        annotate_brain(atlas = dk(), position = position_brain())
    )
  })

  it("dk hemi ~ view with labels", {
    expect_doppelganger(
      "dk hemi view labels",
      ggplot() +
        geom_brain(
          atlas = dk(),
          position = position_brain(hemi ~ view),
          show.legend = FALSE
        ) +
        annotate_brain(
          atlas = dk(),
          position = position_brain(hemi ~ view)
        )
    )
  })

  it("aseg default with labels", {
    expect_doppelganger(
      "aseg default labels",
      ggplot() +
        geom_brain(atlas = aseg(), show.legend = FALSE) +
        annotate_brain(atlas = aseg(), position = position_brain())
    )
  })

  it("aseg nrow 2 with labels", {
    expect_doppelganger(
      "aseg nrow 2 labels",
      ggplot() +
        geom_brain(
          atlas = aseg(),
          position = position_brain(nrow = 2),
          show.legend = FALSE
        ) +
        annotate_brain(
          atlas = aseg(),
          position = position_brain(nrow = 2)
        )
    )
  })
})

#' Midpoint of a numeric range
#'
#' @param x Numeric vector.
#'
#' @return Single numeric value, the mean of `min(x)` and `max(x)`.
#' @keywords internal
#' @noRd
gap <- function(x) {
  (min(x) + max(x)) / 2
}


#' Keep brain shapes undistorted
#'
#' Fixes the aspect ratio so brains aren't stretched by the shape of the
#' plotting window. [geom_brain()] adds `coord_brain()` for you, so you rarely
#' need to call it yourself -- reach for it only to adjust the `ratio` or
#' `clip` behaviour. You can safely add your own coord or stack several
#' `geom_brain()` layers; it steps aside cleanly.
#'
#' @param ratio Aspect ratio, expressed as `y / x`. Defaults to `1`, which
#'   keeps brain polygons undistorted.
#' @param clip Should drawing be clipped to the panel extent (`"on"`) or
#'   allowed to overflow (`"off"`)? Defaults to `"off"` so region outlines at
#'   the panel edge are not cut.
#' @param ... Additional arguments passed to [ggplot2::coord_fixed()].
#'
#' @return A ggplot2 coordinate system that registers as a default.
#' @export
#' @importFrom ggplot2 coord_fixed
#' @examples
#' library(ggplot2)
#' \dontrun{
#' poly <- ggseg.formats::as_polygon_atlas(dk())
#' # Equivalent to the default; shown explicitly:
#' ggplot() +
#'   geom_brain(atlas = poly) +
#'   coord_brain()
#' }
coord_brain <- function(ratio = 1, clip = "off", ...) {
  coord <- coord_fixed(ratio = ratio, clip = clip, ...)
  coord$default <- TRUE
  coord
}

describe("gap", {
  it("returns midpoint of range", {
    expect_identical(gap(c(0, 10)), 5)
    expect_identical(gap(c(-5, 5)), 0)
    expect_identical(gap(c(3, 3)), 3)
  })
})

describe("coord_brain", {
  it("returns a fixed-aspect ggplot2 coord", {
    co <- coord_brain()
    expect_s3_class(co, "Coord")
    expect_equal(co$ratio, 1)
  })

  it("registers as a default coord so it can be overridden silently", {
    co <- coord_brain()
    expect_true(isTRUE(co$default))
  })

  it("does not clip by default", {
    expect_identical(coord_brain()$clip, "off")
  })

  it("honours a custom ratio and clip", {
    co <- coord_brain(ratio = 2, clip = "on")
    expect_equal(co$ratio, 2)
    expect_identical(co$clip, "on")
  })

  it("does not warn when a user coord is added after it", {
    p <- ggplot2::ggplot(mtcars, ggplot2::aes(wt, mpg)) +
      ggplot2::geom_point() +
      coord_brain() +
      ggplot2::coord_cartesian()
    expect_no_message(ggplot2::ggplot_build(p))
  })
})

# sf-optional position machinery ----
#
# Parallel implementations of the position primitives in position-brain.R for
# row-per-point polygon data (the polygon path produced by
# prepare_polygon_atlas). Where the sf path operates on `df$geometry` as sfc
# and uses `sf::st_bbox`, these operate on `df$x`/`df$y` and use base
# `range()` — no sf needed.
#
# Naming: helpers carry the `_flat` suffix to distinguish them from their sf
# counterparts. Stacking helpers return `list(df = combined, box = c(xmin,
# ymin, xmax, ymax))` so the rest of the pipeline reads the same fields.
#
# Layouts are applied at flatten time (inside prepare_polygon_atlas()) rather
# than via a ggproto Position. That side-steps ggplot2's aesthetic-stripping
# of non-standard columns (type / view / hemi), which the sf path's custom
# LayerBrain avoids by declaring those columns as aesthetics.

#' Bounding box of flat polygon coords
#'
#' @param df Data.frame with `x`, `y` numeric columns.
#' @return Named numeric vector `c(xmin, ymin, xmax, ymax)`.
#' @keywords internal
#' @noRd
bbox_flat <- function(df) {
  c(
    xmin = min(df$x),
    ymin = min(df$y),
    xmax = max(df$x),
    ymax = max(df$y)
  )
}


#' Apply a position layout to flat polygon data
#'
#' Mirror of [frame_2_position()] for the polygon path. Operates on
#' row-per-point data and returns a flat data.frame (no sf class machinery).
#'
#' @param data Data.frame with `x`, `y`, view/hemi/etc. columns.
#' @param pos Position spec (string or formula).
#' @param nrow,ncol Optional grid dimensions (mutually exclusive with `pos`).
#' @param views Optional ordered view filter.
#' @param focus Optional character vector of focus region names. When
#'   supplied, each view is cropped to a common window centred on its focus
#'   regions (see [zoom_views_flat()]).
#' @param zoom_pad Fractional padding added around the focus window.
#' @return Data.frame with repositioned `x`, `y` and a `polygon_bbox`
#'   attribute carrying the overall bounding box.
#' @keywords internal
#' @noRd
frame_2_position_flat <- function(
  data,
  pos,
  nrow = NULL,
  ncol = NULL,
  views = NULL,
  focus = NULL,
  zoom_pad = 0.05
) {
  if (!is.null(views)) {
    data <- data[data$view %in% views, , drop = FALSE]
    data$view <- factor(data$view, levels = views)
    data <- data[order(data$view), ]
    data$view <- as.character(data$view)
  }

  if (!is.null(nrow) || !is.null(ncol)) {
    dfpos <- split_data_grid(data, nrow, ncol)
  } else {
    dfpos <- split_data(data, pos)
  }

  if (!is.null(focus)) {
    dfpos$data <- zoom_views_flat(dfpos$data, focus, zoom_pad)
    keep <- vapply(dfpos$data, function(d) nrow(d) > 0, logical(1))
    dfpos$data <- dfpos$data[keep]
  }

  posi <- if (length(dfpos$position) > 1) "grid" else dfpos$position
  rows <- if (posi == "grid") dfpos$position[1] else NULL
  columns <- if (posi == "grid") dfpos$position[2] else NULL

  res <- position_groups(
    dfpos$data,
    posi,
    rows,
    columns,
    bbox_of = bbox_flat,
    translate = function(g, dx, dy) {
      g$x <- g$x + dx
      g$y <- g$y + dy
      g
    }
  )

  out <- dplyr::bind_rows(res$data)
  if (posi == "grid") {
    out <- drop_temp_columns(out)
  }
  attr(out, "polygon_bbox") <- res$box
  out
}


#' Position spec for the sf-optional polygon path
#'
#' Mirror of [position_brain()] for use with [geom_brain_polygon()]. Returns
#' a lightweight spec (not a ggproto Position) so the layout can be applied
#' inside [prepare_polygon_atlas()] before data flows through ggplot2's
#' aesthetic machinery — avoiding the column-stripping that would lose the
#' `type`, `view`, and `hemi` columns the layout needs.
#'
#' @param position Formula describing the rows ~ columns organisation for
#'   cortical atlases (e.g., `hemi ~ view`). For subcortical/tract atlases,
#'   can be "horizontal", "vertical", or a formula with `type ~ .` where type
#'   is extracted from view names like "axial_1" -> "axial".
#' @param nrow Number of rows for grid layout. If NULL (default), calculated
#'   automatically. Only used for subcortical/tract atlases when position is
#'   not a formula.
#' @param ncol Number of columns for grid layout. If NULL (default), calculated
#'   automatically. Only used for subcortical/tract atlases when position is
#'   not a formula.
#' @param views Character vector specifying which views to include and their
#'   order. If NULL (default), all views are included in their original order.
#'   Only applies to subcortical/tract atlases.
#' @param zoom Controls per-view zoom. `NULL`/`FALSE` (default) draws each
#'   view at full extent. `TRUE` zooms each view onto its focus regions —
#'   the regions present in the user `data` passed to [geom_brain_polygon()],
#'   or the atlas's labelled regions when no data is supplied. A character
#'   vector names the focus regions explicitly. Cropping uses an sf-free
#'   polygon clip so context regions become a clean rectangular frame around
#'   the focus.
#' @param zoom_pad Fractional padding added around the focus window when
#'   `zoom` is active. Defaults to `0.05` (5\%).
#' @return A `position_brain_polygon_spec` list with the layout parameters.
#' @keywords internal
#' @noRd
#' @examples
#' \dontrun{
#' library(ggplot2)
#' poly <- ggseg.formats::as_polygon_atlas(dk())
#' ggplot() +
#'   geom_brain_polygon(
#'     atlas = poly,
#'     position = position_brain_polygon("vertical")
#'   )
#'
#' # Zoom each view onto the regions present in user data
#' ggplot() +
#'   geom_brain_polygon(
#'     atlas = poly,
#'     data = my_data,
#'     position = position_brain_polygon(zoom = TRUE)
#'   )
#' }
position_brain_polygon <- function(
  position = "horizontal",
  nrow = NULL,
  ncol = NULL,
  views = NULL,
  zoom = NULL,
  zoom_pad = 0.05
) {
  structure(
    list(
      position = position,
      nrow = nrow,
      ncol = ncol,
      views = views,
      zoom = zoom,
      zoom_pad = zoom_pad
    ),
    class = "position_brain_polygon_spec"
  )
}


#' Resolve a zoom spec into a concrete set of focus region names
#'
#' Translates the `zoom` argument of [position_brain_polygon()] into the
#' character vector of regions used to build each view's focus window.
#' `TRUE` resolves to the regions present in the user `data` (those the user
#' supplied values for), falling back to the atlas's labelled regions when no
#' data is supplied.
#'
#' @param zoom The `zoom` spec: `NULL`/`FALSE`, `TRUE`, or a character vector.
#' @param data Optional user data.frame passed to [geom_brain_polygon()].
#' @param atlas The `ggseg_atlas` being rendered.
#' @return `NULL` when zoom is off, otherwise a character vector of regions.
#' @keywords internal
#' @noRd
resolve_zoom_focus <- function(zoom, data, atlas) {
  if (is.null(zoom) || isFALSE(zoom)) {
    return(NULL)
  }

  if (is.character(zoom)) {
    return(zoom)
  }

  if (!isTRUE(zoom)) {
    cli::cli_abort(c(
      "{.arg zoom} must be {.code NULL}, a logical, or a character vector.",
      "x" = "You supplied {.obj_type_friendly {zoom}}."
    ))
  }

  if (!is.null(data) && "region" %in% names(data)) {
    regs <- unique(data$region)
    regs <- regs[!is.na(regs)]
    if (length(regs)) {
      return(regs)
    }
  }

  regs <- unique(atlas$core$region)
  regs[!is.na(regs)]
}


#' Crop each view onto a common window centred on its focus regions
#'
#' Computes a single window size from the largest focus bounding box across
#' views (plus `pad`), then clips every view to that window centred on the
#' view's own focus centroid. Keeping the window size constant means every
#' view still occupies the same allotted cell after stacking. Views with no
#' focus regions are clipped to the same window centred on the view itself,
#' so they render as context-only frames rather than disappearing.
#'
#' @param view_list List of per-view flat data.frames.
#' @param focus Character vector of focus region names.
#' @param pad Fractional padding around the focus window.
#' @return List of clipped per-view data.frames.
#' @keywords internal
#' @noRd
zoom_views_flat <- function(view_list, focus, pad = 0.05) {
  focus_bboxes <- lapply(view_list, function(df) {
    fr <- df[!is.na(df$region) & df$region %in% focus, , drop = FALSE]
    if (nrow(fr) == 0) {
      return(NULL)
    }
    bbox_flat(fr)
  })

  has_focus <- !vapply(focus_bboxes, is.null, logical(1))
  if (!any(has_focus)) {
    cli::cli_warn(c(
      "No focus regions found in any view; {.arg zoom} had no effect.",
      "i" = "Check that the focus regions exist in the atlas/data."
    ))
    return(view_list)
  }

  dims <- vapply(
    focus_bboxes[has_focus],
    function(b) c(b[["xmax"]] - b[["xmin"]], b[["ymax"]] - b[["ymin"]]),
    numeric(2)
  )
  win_w <- max(dims[1, ]) * (1 + pad)
  win_h <- max(dims[2, ]) * (1 + pad)

  Map(
    function(df, b) {
      if (is.null(b)) {
        b <- bbox_flat(df)
      }
      cx <- (b[["xmin"]] + b[["xmax"]]) / 2
      cy <- (b[["ymin"]] + b[["ymax"]]) / 2
      box <- c(cx - win_w / 2, cx + win_w / 2, cy - win_h / 2, cy + win_h / 2)
      clip_view_flat(df, box)
    },
    view_list,
    focus_bboxes
  )
}


#' Clip all polygon rings in a flat view to a rectangle
#'
#' Splits the flat data into rings (one per `.feature_id` × `subgroup`),
#' clips each with [clip_ring_to_box()], and rebuilds the row-per-point
#' data.frame preserving all metadata columns. Rings fully outside the box
#' are dropped; rings spanning the box gain vertices on the boundary.
#'
#' @param df Flat view data.frame with `x`, `y`, `.feature_id`, `subgroup`.
#' @param box Numeric `c(xmin, xmax, ymin, ymax)`.
#' @return Clipped flat data.frame (possibly zero rows).
#' @keywords internal
#' @noRd
clip_view_flat <- function(df, box) {
  if (nrow(df) == 0) {
    return(df)
  }

  ring <- interaction(df$.feature_id, df$subgroup, drop = TRUE)
  pieces <- split(seq_len(nrow(df)), ring)

  out <- lapply(pieces, function(idx) {
    sub <- df[idx, , drop = FALSE]
    clipped <- clip_ring_to_box(sub$x, sub$y, box)
    if (nrow(clipped) == 0) {
      return(NULL)
    }
    rebuilt <- sub[rep(1L, nrow(clipped)), , drop = FALSE]
    rebuilt$x <- clipped[, 1]
    rebuilt$y <- clipped[, 2]
    rebuilt
  })

  out <- Filter(Negate(is.null), out)
  if (length(out) == 0) {
    return(df[0, , drop = FALSE])
  }
  dplyr::bind_rows(out)
}


#' Clip one polygon ring to an axis-aligned rectangle (Sutherland–Hodgman)
#'
#' sf-free polygon clipping against the convex clip rectangle, applied as
#' four successive half-plane clips. Treats the input as a closed ring.
#'
#' @param x,y Numeric vertex coordinates of the ring.
#' @param box Numeric `c(xmin, xmax, ymin, ymax)`.
#' @return Two-column matrix of clipped vertices (zero rows if fully outside).
#' @keywords internal
#' @noRd
clip_ring_to_box <- function(x, y, box) {
  xmin <- box[[1]]
  xmax <- box[[2]]
  ymin <- box[[3]]
  ymax <- box[[4]]

  pts <- cbind(x, y)

  clip_edge <- function(pts, inside, intersect) {
    n <- nrow(pts)
    if (n == 0) {
      return(pts)
    }
    keep <- vector("list", 0)
    prev <- pts[n, ]
    prev_in <- inside(prev)
    for (i in seq_len(n)) {
      cur <- pts[i, ]
      cur_in <- inside(cur)
      if (cur_in) {
        if (!prev_in) {
          keep[[length(keep) + 1L]] <- intersect(prev, cur)
        }
        keep[[length(keep) + 1L]] <- cur
      } else if (prev_in) {
        keep[[length(keep) + 1L]] <- intersect(prev, cur)
      }
      prev <- cur
      prev_in <- cur_in
    }
    if (length(keep) == 0) {
      matrix(numeric(0), 0, 2)
    } else {
      do.call(rbind, keep)
    }
  }

  cross <- function(a, b, edge, axis) {
    t <- (edge - a[axis]) / (b[axis] - a[axis])
    a + t * (b - a)
  }

  pts <- clip_edge(pts, function(p) p[1] >= xmin, function(a, b) {
    cross(a, b, xmin, 1)
  })
  pts <- clip_edge(pts, function(p) p[1] <= xmax, function(a, b) {
    cross(a, b, xmax, 1)
  })
  pts <- clip_edge(pts, function(p) p[2] >= ymin, function(a, b) {
    cross(a, b, ymin, 2)
  })
  pts <- clip_edge(pts, function(p) p[2] <= ymax, function(a, b) {
    cross(a, b, ymax, 2)
  })

  unname(pts)
}


#' Test whether an object is a polygon-path position spec
#'
#' @param x An object.
#' @return Logical.
#' @keywords internal
#' @noRd
is_polygon_position <- function(x) {
  inherits(x, "position_brain_polygon_spec")
}

describe("position_brain_polygon()", {
  it("returns a position_brain_polygon_spec object", {
    spec <- position_brain_polygon()
    expect_s3_class(spec, "position_brain_polygon_spec")
    expect_equal(spec$position, "horizontal")
    expect_null(spec$nrow)
    expect_null(spec$ncol)
    expect_null(spec$views)
  })

  it("captures formula positions", {
    spec <- position_brain_polygon(hemi ~ view)
    expect_s3_class(spec, "position_brain_polygon_spec")
    expect_s3_class(spec$position, "formula")
  })

  it("default horizontal layout produces a single row of views", {
    poly <- ggseg.formats::as_polygon_atlas(dk())
    flat <- prepare_polygon_atlas(poly, position = position_brain_polygon())
    bbox <- attr(flat, "polygon_bbox")
    expect_true(!is.null(bbox))
    expect_gt(bbox["xmax"] - bbox["xmin"], bbox["ymax"] - bbox["ymin"])
  })

  it("vertical layout produces a single column of views", {
    poly <- ggseg.formats::as_polygon_atlas(dk())
    flat <- prepare_polygon_atlas(
      poly,
      position = position_brain_polygon("vertical")
    )
    bbox <- attr(flat, "polygon_bbox")
    expect_gt(bbox["ymax"] - bbox["ymin"], bbox["xmax"] - bbox["xmin"])
  })

  it("formula layout `hemi ~ view` produces a grid", {
    poly <- ggseg.formats::as_polygon_atlas(dk())
    flat <- prepare_polygon_atlas(
      poly,
      position = position_brain_polygon(hemi ~ view)
    )
    bbox <- attr(flat, "polygon_bbox")
    expect_true(!is.null(bbox))
    expect_true(all(is.finite(bbox)))
  })
})

describe("flat-coord helpers", {
  it("bbox_flat returns named numeric of length 4", {
    df <- data.frame(x = c(0, 1, 2), y = c(0, 1, 2))
    b <- bbox_flat(df)
    expect_named(b, c("xmin", "ymin", "xmax", "ymax"))
    expect_equal(unname(b), c(0, 0, 2, 2))
  })
})

describe("geom_brain_polygon() with position", {
  it("renders with default horizontal position", {
    poly <- ggseg.formats::as_polygon_atlas(dk())
    p <- ggplot2::ggplot() + geom_brain_polygon(atlas = poly)
    g <- ggplot2::ggplot_build(p)
    expect_true(all(is.finite(range(g$data[[1]]$x))))
  })

  it("renders with formula position", {
    poly <- ggseg.formats::as_polygon_atlas(dk())
    p <- ggplot2::ggplot() +
      geom_brain_polygon(
        atlas = poly,
        position = position_brain_polygon(hemi ~ view)
      )
    g <- ggplot2::ggplot_build(p)
    expect_true(all(is.finite(range(g$data[[1]]$x))))
    expect_true(all(is.finite(range(g$data[[1]]$y))))
  })

  it("renders with grid (nrow/ncol) position", {
    poly <- ggseg.formats::as_polygon_atlas(dk())
    p <- ggplot2::ggplot() +
      geom_brain_polygon(
        atlas = poly,
        position = position_brain_polygon(nrow = 2)
      )
    g <- ggplot2::ggplot_build(p)
    expect_true(all(is.finite(range(g$data[[1]]$y))))
  })

  it("stores zoom and zoom_pad in the spec", {
    spec <- position_brain_polygon(zoom = TRUE, zoom_pad = 0.1)
    expect_true(spec$zoom)
    expect_equal(spec$zoom_pad, 0.1)
  })

  it("defaults zoom off with 5% padding", {
    spec <- position_brain_polygon()
    expect_null(spec$zoom)
    expect_equal(spec$zoom_pad, 0.05)
  })
})

describe("clip_ring_to_box", {
  it("clips a ring that straddles the box to the box bounds", {
    sq <- clip_ring_to_box(c(0, 10, 10, 0), c(0, 0, 10, 10), c(2, 8, 2, 8))
    expect_equal(range(sq[, 1]), c(2, 8))
    expect_equal(range(sq[, 2]), c(2, 8))
  })

  it("leaves a fully-contained ring's extent unchanged", {
    tri <- clip_ring_to_box(c(3, 5, 4), c(3, 3, 5), c(0, 10, 0, 10))
    expect_equal(nrow(tri), 3)
  })

  it("drops a ring fully outside the box", {
    out <- clip_ring_to_box(c(20, 22, 21), c(20, 20, 22), c(0, 10, 0, 10))
    expect_equal(nrow(out), 0)
  })

  it("fills the box when the ring contains it", {
    big <- clip_ring_to_box(
      c(-5, 15, 15, -5),
      c(-5, -5, 15, 15),
      c(0, 10, 0, 10)
    )
    expect_equal(range(big[, 1]), c(0, 10))
    expect_equal(range(big[, 2]), c(0, 10))
  })
})

describe("clip_view_flat", {
  it("clips every ring in a flat view and preserves metadata columns", {
    df <- data.frame(
      x = c(0, 10, 10, 0, 100, 110, 110, 100),
      y = c(0, 0, 10, 10, 0, 0, 10, 10),
      .feature_id = c(1, 1, 1, 1, 2, 2, 2, 2),
      subgroup = 1L,
      region = c(rep("a", 4), rep("b", 4)),
      stringsAsFactors = FALSE
    )
    clipped <- clip_view_flat(df, c(-1, 11, -1, 11))
    expect_true(all(clipped$region == "a"))
    expect_setequal(unique(clipped$.feature_id), 1)
  })
})

describe("resolve_zoom_focus", {
  it("returns NULL when zoom is off", {
    expect_null(resolve_zoom_focus(NULL, NULL, aseg()))
    expect_null(resolve_zoom_focus(FALSE, NULL, aseg()))
  })

  it("returns explicit region names unchanged", {
    expect_equal(resolve_zoom_focus(c("x", "y"), NULL, aseg()), c("x", "y"))
  })

  it("uses regions present in data when zoom = TRUE", {
    atlas <- aseg()
    data <- data.frame(region = c("thalamus proper", "caudate"))
    expect_setequal(
      resolve_zoom_focus(TRUE, data, atlas),
      c("thalamus proper", "caudate")
    )
  })

  it("falls back to labelled atlas regions when zoom = TRUE and no data", {
    atlas <- aseg()
    labelled <- unique(atlas$core$region)
    labelled <- labelled[!is.na(labelled)]
    expect_setequal(resolve_zoom_focus(TRUE, NULL, atlas), labelled)
  })

  it("errors on an unsupported zoom value", {
    expect_error(resolve_zoom_focus(1L, NULL, aseg()), "must be")
  })
})

describe("zoom_views_flat", {
  it("warns and returns input unchanged when no focus regions are present", {
    df <- data.frame(
      x = c(0, 1, 1),
      y = c(0, 0, 1),
      .feature_id = 1L,
      subgroup = 1L,
      region = NA_character_
    )
    expect_warning(
      out <- zoom_views_flat(list(df), focus = "missing"),
      "No focus regions"
    )
    expect_identical(out, list(df))
  })
})

# sf-optional annotate_brain ----
#
# Mirror of annotate_brain() for the polygon path. Computes label positions
# from flat polygon data using range() instead of sf::st_bbox.

#' Add view labels to a polygon-path brain plot
#'
#' Annotates each brain view with a text label positioned above the view's
#' bounding box. For cortical atlases, labels show hemisphere and view
#' (e.g. "left lateral"). For subcortical and tract atlases, labels show
#' the view name directly.
#'
#' Labels respect the layout applied by [position_brain_polygon()], so the
#' same `position` argument should be passed to both [geom_brain_polygon()]
#' and `annotate_brain_polygon()`.
#'
#' @param atlas A `ggseg_atlas` object with 2D geometry (sf or polygons).
#' @param position A [position_brain_polygon()] spec matching the one used
#'   in [geom_brain_polygon()].
#' @param hemi Character vector of hemispheres to include. If `NULL`
#'   (default), all hemispheres are included.
#' @param view Character vector of views to include. If `NULL` (default),
#'   all views are included.
#' @param size Text size in mm (default `3`).
#' @param colour Text colour (default `"grey30"`).
#' @param family Font family (default `"mono"`).
#' @param padding Vertical gap between each label and its view, as a
#'   fraction of the plot's total height (default `0.05`). Labels are also
#'   bottom-anchored (`vjust = 0`) so they sit clear of the geometry.
#' @param nudge_y Additional absolute vertical offset for labels
#'   (default `0`).
#' @param ... Additional arguments passed to [ggplot2::annotate()].
#'
#' @return A ggplot2 annotation layer.
#' @keywords internal
#' @noRd
#' @examples
#' \dontrun{
#' library(ggplot2)
#' poly <- ggseg.formats::as_polygon_atlas(dk())
#' pos <- position_brain_polygon(hemi ~ view)
#' ggplot() +
#'   geom_brain_polygon(atlas = poly, position = pos, show.legend = FALSE) +
#'   annotate_brain_polygon(atlas = poly, position = pos)
#' }
annotate_brain_polygon <- function(
  atlas,
  position = position_brain_polygon(),
  hemi = NULL,
  view = NULL,
  size = 3,
  colour = "grey30",
  family = "mono",
  padding = 0.05,
  nudge_y = 0,
  ...
) {
  flat <- prepare_polygon_atlas(
    atlas,
    hemi = hemi,
    view = view,
    position = position
  )

  label_df <- compute_label_positions_flat(flat)

  y_range <- diff(range(flat$y))
  label_df$y <- label_df$y + y_range * padding + nudge_y

  annotate_brain_labels(label_df, size, colour, family, ...)
}


#' Compute label text and positions from flat polygon data
#'
#' Mirror of [compute_label_positions()] for row-per-point data. Groups by
#' (hemi, view) for cortical atlases, just `view` otherwise, then returns
#' one centred label per group using base `range()` instead of
#' `sf::st_bbox`.
#'
#' @param flat A row-per-point data.frame returned by
#'   [prepare_polygon_atlas()].
#' @return A data.frame with columns `x`, `y`, `label`.
#' @keywords internal
#' @noRd
compute_label_positions_flat <- function(flat) {
  atlas_type <- unique(flat$type)[1]

  if (atlas_type == "cortical") {
    groups <- split(
      flat,
      list(flat$hemi, flat$view),
      drop = TRUE
    )
    label_fn <- function(df) paste(unique(df$hemi), unique(df$view))
  } else {
    groups <- split(flat, flat$view, drop = TRUE)
    label_fn <- function(df) unique(df$view)
  }

  do.call(
    rbind,
    lapply(groups, function(df) {
      data.frame(
        x = (min(df$x) + max(df$x)) / 2,
        y = max(df$y),
        label = label_fn(df),
        stringsAsFactors = FALSE,
        row.names = NULL
      )
    })
  )
}

#' Join user data with a brain atlas
#'
#' Matches your data to a brain atlas by a shared column (usually `region`),
#' keeping every atlas region whether or not you have a value for it. Grouped
#' data (via [dplyr::group_by()]) gives one complete atlas per group. You
#' rarely need this directly -- [geom_brain()] joins your data for you; reach
#' for `brain_join()` when you want the joined sf object yourself.
#'
#' @param data A data.frame with a column matching an atlas column
#'   (typically `"region"`). Can be grouped with [dplyr::group_by()].
#' @param atlas A `ggseg_atlas` object or data.frame containing atlas data.
#' @param by Character vector of column names to join by. If `NULL` (default),
#'   columns are detected automatically.
#'
#' @return An `sf` object if the atlas contains geometry, otherwise a tibble.
#' @export
#' @importFrom dplyr is.grouped_df full_join as_tibble
#' @importFrom tidyr nest unnest
#' @importFrom utils capture.output
#' @examplesIf requireNamespace("sf", quietly = TRUE)
#' someData <- data.frame(
#'   region = c(
#'     "transverse temporal", "insula",
#'     "precentral", "superior parietal"
#'   ),
#'   p = sample(seq(0, .5, .001), 4),
#'   stringsAsFactors = FALSE
#' )
#'
#' brain_join(someData, dk())
#' brain_join(someData, dk(), "region")
#'
brain_join <- function(data, atlas, by = NULL) {
  atlas <- as.data.frame(atlas)

  if (is.null(by)) {
    by <- names(data)[names(data) %in% names(atlas)]
    cli::cli_inform("Merging atlas and data by {.field {by}}.")
  }

  if (is.grouped_df(data)) {
    data2 <- nest(data)
    data2$data <- lapply(seq_len(nrow(data2)), function(x) {
      full_join(atlas, data2$data[[x]], by = by)
    })

    dt <- unnest(data2, data)
  } else {
    dt <- full_join(atlas, data, by = by)
  }

  errs <- dt[is.na(dt$atlas), ]

  if (nrow(errs) > 0) {
    errs <- dplyr::select(errs, -dplyr::starts_with("."))
    errs <- dplyr::as_tibble(errs)

    cli::cli_warn(c(
      "Some data not merged properly.",
      "i" = "Check for naming errors in data:",
      " " = paste(capture.output(errs)[-1], collapse = "\n")
    ))
  }

  if ("geometry" %in% names(dt)) {
    require_sf("brain_join()")
    sf::st_as_sf(dt)
  } else {
    as_tibble(dt)
  }
}

describe("brain_join", {
  some_data <- data.frame(
    region = c(
      "transverse temporal",
      "insula",
      "precentral",
      "superior parietal",
      "transverse temporal",
      "insula",
      "precentral",
      "superior parietal"
    ),
    p = seq(0.1, 0.8, by = 0.1),
    grp = c(rep("G1", 4), rep("G2", 4)),
    stringsAsFactors = FALSE
  )

  it("joins ungrouped data to atlas", {
    skip_if_not_installed("sf")
    single <- some_data[some_data$grp == "G1", ]
    expect_message(
      result <- brain_join(single, dk()),
      "Merging"
    )
    expect_s3_class(result, "sf")
  })

  it("auto-detects join columns", {
    skip_if_not_installed("sf")
    single <- some_data[some_data$grp == "G1", ]
    expect_message(
      brain_join(single, dk()),
      "region"
    )
  })

  it("respects explicit by argument", {
    skip_if_not_installed("sf")
    single <- some_data[some_data$grp == "G1", ]
    result <- brain_join(single, dk(), by = "region")
    expect_s3_class(result, "sf")
  })

  it("joins grouped data producing one atlas per group", {
    skip_if_not_installed("sf")
    grouped <- group_by(some_data, grp)
    expect_message(
      result <- brain_join(grouped, dk()),
      "Merging"
    )
    expect_s3_class(result, "sf")
    expect_true("grp" %in% names(result))
    expect_identical(sort(unique(result$grp)), c("G1", "G2"))
    atlas_rows <- nrow(as.data.frame(dk()))
    rows_per_grp <- tapply(result$grp, result$grp, length)
    expect_true(all(rows_per_grp == atlas_rows))
  })

  it("warns when data has unmatched regions", {
    skip_if_not_installed("sf")
    bad_data <- data.frame(
      region = "not a real region",
      p = 0.5,
      stringsAsFactors = FALSE
    )
    expect_warning(
      expect_message(brain_join(bad_data, dk())),
      "not merged"
    )
  })

  it("returns tibble when atlas has no geometry", {
    atlas_df <- as.data.frame(dk())
    atlas_df$geometry <- NULL
    single <- some_data[some_data$grp == "G1", ]
    expect_message(
      result <- brain_join(single, atlas_df),
      "Merging"
    )
    expect_s3_class(result, "tbl_df")
  })
})

#' Add view labels to brain atlas plots
#'
#' Annotates each brain view with a text label positioned above the
#' view's bounding box. For cortical atlases, labels show hemisphere
#' and view (e.g., "left lateral"). For subcortical and tract atlases,
#' labels show the view name directly (e.g., "axial_1", "sagittal").
#'
#' Pass the same `position` you gave [geom_brain()] and the labels line up with
#' the views automatically.
#'
#' @param atlas A `brain_atlas` object (e.g. `dk()`, `aseg()`).
#' @param position The same layout you passed to [geom_brain()], from
#'   [position_brain()].
#' @param hemi Character vector of hemispheres to include. If `NULL`
#'   (default), all hemispheres are included.
#' @param view Character vector of views to include. If `NULL`
#'   (default), all views are included.
#' @param size Text size in mm (default: `3`).
#' @param colour Text colour (default: `"grey30"`).
#' @param family Font family (default: `"mono"`).
#' @param padding Vertical gap between each label and its view, as a
#'   fraction of the plot's total height (default: `0.05`). Labels are also
#'   bottom-anchored (`vjust = 0`) so they sit clear of the geometry.
#' @param nudge_y Additional absolute vertical offset for labels
#'   (default: `0`).
#' @param ... Additional arguments passed to [ggplot2::annotate()].
#'
#' @return A ggplot2 annotation layer.
#' @export
#'
#' @examples
#' library(ggplot2)
#'
#' pos <- position_brain(hemi ~ view)
#' ggplot() +
#'   geom_brain(atlas = dk(), position = pos, show.legend = FALSE) +
#'   annotate_brain(atlas = dk(), position = pos)
#'
#' ggplot() +
#'   geom_brain(atlas = dk(), show.legend = FALSE) +
#'   annotate_brain(atlas = dk())
annotate_brain <- function(
  atlas,
  position = position_brain(),
  hemi = NULL,
  view = NULL,
  size = 3,
  colour = "grey30",
  family = "mono",
  padding = 0.05,
  nudge_y = 0,
  ...
) {
  annotate_fn <- if (is_polygon_position(position)) {
    annotate_brain_polygon
  } else {
    annotate_brain_sf
  }
  annotate_fn(
    atlas,
    position = position,
    hemi = hemi,
    view = view,
    size = size,
    colour = colour,
    family = family,
    padding = padding,
    nudge_y = nudge_y,
    ...
  )
}


#' sf implementation behind [annotate_brain()]
#'
#' @inheritParams annotate_brain
#' @return A ggplot2 annotation layer.
#' @keywords internal
#' @noRd
annotate_brain_sf <- function(
  atlas,
  position = position_brain(),
  hemi = NULL,
  view = NULL,
  size = 3,
  colour = "grey30",
  family = "mono",
  padding = 0.05,
  nudge_y = 0,
  ...
) {
  require_sf("annotate_brain()")
  data <- as.data.frame(atlas)

  if (!is.null(hemi)) {
    data <- data[data$hemi %in% hemi, ]
  }
  if (!is.null(view)) {
    data <- data[data$view %in% view, ]
  }

  params <- extract_position_params(position)
  repositioned <- reposition_brain(
    data,
    params$position,
    nrow = params$nrow,
    ncol = params$ncol,
    views = params$views
  )

  label_df <- compute_label_positions(repositioned)

  overall_bbox <- sf::st_bbox(repositioned$geometry)
  y_range <- unname(overall_bbox["ymax"] - overall_bbox["ymin"])
  label_df$y <- label_df$y + y_range * padding + nudge_y

  annotate_brain_labels(label_df, size, colour, family, ...)
}


#' Build the text-annotation layer shared by the annotate helpers
#'
#' @param label_df A data.frame with `x`, `y`, `label` columns.
#' @param size,colour,family Text styling passed to [ggplot2::annotate()].
#' @param ... Extra arguments for [ggplot2::annotate()]; `vjust = 0` is the
#'   default so labels are anchored above their point.
#' @return A ggplot2 annotation layer.
#' @keywords internal
#' @noRd
annotate_brain_labels <- function(label_df, size, colour, family, ...) {
  dots <- list(...)
  if (!"vjust" %in% names(dots)) {
    dots$vjust <- 0
  }
  do.call(
    ggplot2::annotate,
    c(
      list(
        "text",
        x = label_df$x,
        y = label_df$y,
        label = label_df$label,
        size = size,
        colour = colour,
        family = family
      ),
      dots
    )
  )
}


#' Extract position parameters from a PositionBrain object
#'
#' @param pos A PositionBrain ggproto object or raw position value.
#' @return A list with position, nrow, ncol, views.
#' @keywords internal
#' @noRd
extract_position_params <- function(pos) {
  if (inherits(pos, "PositionBrain")) {
    list(
      position = pos$position,
      nrow = pos$nrow,
      ncol = pos$ncol,
      views = pos$views
    )
  } else {
    list(position = pos, nrow = NULL, ncol = NULL, views = NULL)
  }
}


#' Compute label text and positions from repositioned brain data
#'
#' @param repositioned An sf data.frame returned by [reposition_brain()].
#' @return A data.frame with columns x, y, label.
#' @keywords internal
#' @noRd
compute_label_positions <- function(repositioned) {
  atlas_type <- unique(repositioned$type)[1]

  if (atlas_type == "cortical") {
    groups <- split(
      repositioned,
      list(repositioned$hemi, repositioned$view),
      drop = TRUE
    )
    label_fn <- function(df) paste(unique(df$hemi), unique(df$view))
  } else {
    groups <- split(repositioned, repositioned$view, drop = TRUE)
    label_fn <- function(df) unique(df$view)
  }

  do.call(
    rbind,
    lapply(groups, function(df) {
      bbox <- sf::st_bbox(df$geometry)
      data.frame(
        x = unname((bbox["xmin"] + bbox["xmax"]) / 2),
        y = unname(bbox["ymax"]),
        label = label_fn(df),
        stringsAsFactors = FALSE,
        row.names = NULL
      )
    })
  )
}

test_that("test read_freesurfer_stats works", {
  aseg_file <- test_path("data/bert/stats/aseg.stats")

  aseg_stats <- read_freesurfer_stats(aseg_file)
  expect_named(
    aseg_stats,
    c(
      "Index",
      "SegId",
      "NVoxels",
      "Volume_mm3",
      "label",
      "normMean",
      "normStdDev",
      "normMin",
      "normMax",
      "normRange"
    )
  )
  expect_identical(nrow(aseg_stats), 45L)

  expect_named(
    read_freesurfer_stats(aseg_file, FALSE),
    c(
      "Index",
      "SegId",
      "NVoxels",
      "Volume_mm3",
      "StructName",
      "normMean",
      "normStdDev",
      "normMin",
      "normMax",
      "normRange"
    )
  )

  dkt_file <- test_path("data/bert/stats/lh.aparc.stats")

  dkt_stats <- read_freesurfer_stats(dkt_file)
  expect_named(
    dkt_stats,
    c(
      "label",
      "NumVert",
      "SurfArea",
      "GrayVol",
      "ThickAvg",
      "ThickStd",
      "MeanCurv",
      "GausCurv",
      "FoldInd",
      "CurvInd"
    )
  )
  expect_identical(nrow(dkt_stats), 34L)

  expect_named(
    read_freesurfer_stats(dkt_file, FALSE),
    c(
      "StructName",
      "NumVert",
      "SurfArea",
      "GrayVol",
      "ThickAvg",
      "ThickStd",
      "MeanCurv",
      "GausCurv",
      "FoldInd",
      "CurvInd"
    )
  )
})

test_that("test that read_atlas_files works", {
  dat <- read_atlas_files(test_path("data"), "aparc")

  expect_named(
    dat,
    c(
      "subject",
      "label",
      "NumVert",
      "SurfArea",
      "GrayVol",
      "ThickAvg",
      "ThickStd",
      "MeanCurv",
      "GausCurv",
      "FoldInd",
      "CurvInd"
    )
  )
  expect_identical(nrow(dat), 68L)
  expect_identical(
    unique(dat$label)[1:10],
    c(
      "lh_bankssts",
      "lh_caudalanteriorcingulate",
      "lh_caudalmiddlefrontal",
      "lh_cuneus",
      "lh_entorhinal",
      "lh_fusiform",
      "lh_inferiorparietal",
      "lh_inferiortemporal",
      "lh_isthmuscingulate",
      "lh_lateraloccipital"
    )
  )
})

test_that("test read_freesurfer_table works", {
  file <- test_path("data/aparc.volume.table")
  dat <- read_freesurfer_table(file)

  expect_named(dat, c("subject", "label", "value"))
  expect_identical(nrow(dat), 36L)
  expect_true(any(grepl("volume$", dat$label)))

  dat <- read_freesurfer_table(file, measure = "volume")
  expect_named(dat, c("subject", "label", "volume"))
  expect_false(any(grepl("volume$", dat$label)))
})

---
title: "Getting started with ggseg"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Getting started with ggseg}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r}
#| label: setup
#| include: false
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 6
)
```

If you work with neuroimaging data, you've probably spent time wrestling
region-level results into brain plots. ggseg handles that plumbing for you.
It stores brain atlas geometries and plots them as ggplot2 layers, so you get
brain figures with the same code you'd use for any other ggplot.

```{r}
#| label: load-packages
library(ggseg)
library(ggplot2)
```

## What's in a brain atlas

A `brain_atlas` object bundles four things: a name, a type (cortical,
subcortical, or tract), region geometry, and an optional colour palette.

```{r}
#| label: dk-atlas
dk()
```

The `dk` atlas (Desikan-Killiany) ships with the package. Call `plot()` for a
quick look:

```{r}
#| label: fig-dk-plot
#| fig-cap: "Quick overview of the Desikan-Killiany cortical atlas."
plot(dk())
```

Two helpers pull out the names you'll need for matching data:

```{r}
#| label: dk-names
library(ggseg.formats)

atlas_regions(dk())
atlas_labels(dk())
```

## Your first brain plot

`geom_brain()` works like any ggplot2 geom. Pass it an atlas and it
draws the regions:

```{r}
#| label: fig-basic-brain
#| fig-cap: "Default brain plot using geom_brain() with the dk atlas."
ggplot() +
  geom_brain(atlas = dk(), show.legend = FALSE)
```

## Arranging views

Brain atlases have multiple views (lateral, medial) and hemispheres.
`position_brain()` controls how they're laid out, using formula
syntax borrowed from `facet_grid()`:

```{r}
#| label: fig-position-formula
#| fig-cap: "Brain views arranged using formula syntax in position_brain()."
ggplot() +
  geom_brain(
    atlas = dk(),
    position = position_brain(hemi ~ view),
    show.legend = FALSE
  )
```

Or reorder views with a character vector:

```{r}
#| label: fig-position-character
#| fig-cap: "Brain views arranged using a character vector to specify order."
ggplot() +
  geom_brain(
    atlas = dk(),
    position = position_brain(c(
      "right lateral",
      "right medial",
      "left lateral",
      "left medial"
    )),
    show.legend = FALSE
  )
```

See `vignette("positioning-views")` for the full set of layout options,
including grid layouts for subcortical and tract atlases and per-view zoom.

## Showing only certain hemispheres or views

```{r}
#| label: fig-filter-lateral
#| fig-cap: "Brain plot showing only lateral views."
ggplot() +
  geom_brain(atlas = dk(), view = "lateral", show.legend = FALSE)
```

```{r}
#| label: fig-filter-left
#| fig-cap: "Brain plot showing only the left hemisphere."
ggplot() +
  geom_brain(atlas = dk(), hemi = "left")
```

For subcortical atlases, views are slice identifiers. Check what's available
with `ggseg.formats::atlas_views()`:

```{r}
#| label: aseg-views
atlas_views(aseg())
```

## Plotting your own data

Create a data frame with at least one column that matches the atlas
(typically `region` or `label`). Pass it to `geom_brain()` through
the `data` argument and it joins to the atlas automatically:

```{r}
#| label: fig-external-data
#| fig-cap: "Brain plot coloured by external p-values using a viridis scale."
library(dplyr)

some_data <- tibble(
  region = c(
    "transverse temporal",
    "insula",
    "precentral",
    "superior parietal"
  ),
  p = sample(seq(0, 0.5, 0.001), 4)
)

ggplot() +
  geom_brain(
    atlas = dk(),
    data = some_data,
    position = position_brain(hemi ~ view),
    aes(fill = p)
  ) +
  scale_fill_viridis_c(option = "cividis", direction = -1) +
  theme_void()
```

See `vignette("external-data")` for more on data preparation and matching.

## Faceting

ggplot2 faceting works as you'd expect. Group your data by the faceting
variable and `geom_brain()` replicates the full atlas in each panel:

```{r}
#| label: fig-faceting
#| fig-cap: "Brain plots faceted by group, each panel showing the full atlas."
some_data <- tibble(
  region = rep(
    c(
      "transverse temporal",
      "insula",
      "precentral",
      "superior parietal"
    ),
    2
  ),
  p = sample(seq(0, 0.5, 0.001), 8),
  group = c(rep("A", 4), rep("B", 4))
)

ggplot() +
  geom_brain(
    atlas = dk(),
    data = group_by(some_data, group),
    position = position_brain(hemi ~ view),
    aes(fill = p)
  ) +
  facet_wrap(~group)
```


## Highlighting specific regions

To colour a few regions with their atlas colours, use two columns: one for
the join (`region`) and one for the fill aesthetic:

```{r}
#| label: fig-highlighting
#| fig-cap: "Highlighting selected regions using the atlas colour palette."
data <- data.frame(
  region = atlas_regions(dk())[1:3],
  reg_col = atlas_labels(dk())[1:3]
)

palette <- atlas_palette(dk())[1:3]

ggplot() +
  geom_brain(atlas = dk(), data = data, aes(fill = reg_col)) +
  scale_fill_brain_manual(palette)
```

## Scales and themes

Atlas colour palettes are applied automatically by `geom_brain()`.
For custom palettes, use `scale_fill_brain_manual()`.

Built-in themes strip axes and grids for cleaner figures:

```{r}
#| label: themes-example
#| eval: false
ggplot() +
  geom_brain(atlas = dk()) +
  theme_brain()
```

## Going further

For full control over the data before it reaches ggplot2 — adding region
labels, layering other sf geoms, or using `geom_sf()` directly — see
`vignette("geom-sf")`.

## Finding more atlases

ggseg ships with `dk()` (cortical), `aseg()` (subcortical), `suit()` (cerebellar flamap), and `tracula()`
(white matter tracts). Many more are available through the
[ggsegverse r-universe](https://ggsegverse.r-universe.dev):

```{r}
#| label: install-extra
#| eval: false
install.packages("ggsegYeo2011", repos = "https://ggsegverse.r-universe.dev")
```

---
title: "Plotting external data"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Plotting external data}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r}
#| label: setup
#| include: false
knitr::opts_chunk$set(eval = TRUE, fig.retina = 3, fig.width = 6)
```

Most of the time, you're not plotting empty atlases. You have results --
p-values, cortical thickness, whatever -- and you want them on a brain. This
vignette covers how to get your data into the right shape for ggseg.

```{r}
#| label: load-packages
library(ggseg)
library(dplyr)
library(ggplot2)
```

## How matching works

`geom_brain()` joins your data to the atlas by any columns they share.
That means your data needs at least one column with names that match the
atlas. The two columns you'll use most:

- **region** -- human-readable names like "insula" or "precentral"
- **label** -- FreeSurfer labels like "lh_bankssts"

Check what's available:

```{r}
#| label: dk-regions
ggseg.formats::atlas_regions(dk())
```

```{r}
#| label: dk-labels
ggseg.formats::atlas_labels(dk())
```

Names must match exactly, including case and spacing.

## A minimal example

Three regions, three p-values:

```{r}
#| label: minimal-data
some_data <- tibble(
  region = c("superior temporal", "precentral", "lateral orbitofrontal"),
  p = c(0.03, 0.6, 0.05)
)
some_data
```

Pass the data to `ggplot()` and map `fill` to your variable:

```{r}
#| label: fig-minimal-plot
#| fig-cap: "Brain plot with three regions coloured by p-value."
ggplot(some_data) +
  geom_brain(atlas = dk(), mapping = aes(fill = p))
```

Regions not in your data appear as `NA` (grey by default). Regions in your
data that don't match the atlas are silently dropped, so watch your spelling.

## Constraining matches with extra columns

If your data is hemisphere-specific, add a `hemi` column. The join will use
both `region` and `hemi`, so values only land on the correct side:

```{r}
#| label: fig-hemi-constraint
#| fig-cap: "Brain plot restricted to the left hemisphere using a hemi column."
some_data$hemi <- "left"

ggplot(some_data) +
  geom_brain(atlas = dk(), mapping = aes(fill = p))
```

The same works for any atlas column -- adding `view`, for instance, would
restrict matches to specific views.

## Faceting across groups

If your data has a grouping variable, `facet_wrap()` and `facet_grid()` work
as you'd expect. `geom_brain()` detects the faceting variables and
replicates the full atlas in each panel:

```{r}
#| label: fig-facet-groups
#| fig-cap: "Brain plots faceted by age group with a custom colour gradient."
some_data <- tibble(
  region = rep(
    c(
      "transverse temporal",
      "insula",
      "precentral",
      "superior parietal"
    ),
    2
  ),
  p = sample(seq(0, 0.5, 0.001), 8),
  group = c(rep("Young", 4), rep("Old", 4))
)

ggplot(some_data) +
  geom_brain(atlas = dk(), colour = "white", mapping = aes(fill = p)) +
  facet_wrap(~group, ncol = 1) +
  theme(legend.position = "bottom") +
  scale_fill_gradientn(
    colours = c("royalblue", "firebrick", "goldenrod"),
    na.value = "grey"
  )
```

No need to call `group_by()` first -- the geom handles atlas replication
automatically. (Explicit `group_by()` still works for backward
compatibility.)

## The pre-merged workflow

For full control over faceting or when you need to combine brain data with
other sf layers, convert the atlas to a data frame and join manually:

```{r}
#| label: atlas-columns
atlas_df <- as.data.frame(dk())
names(atlas_df)
```

Then use a standard join and `geom_sf()`:

```{r}
#| label: fig-pre-merged
#| fig-cap: "Brain plot using a manual left_join and geom_sf for full control."
some_data <- tibble(
  region = c("superior temporal", "precentral", "lateral orbitofrontal"),
  p = c(0.03, 0.6, 0.05)
)

atlas_df |>
  left_join(some_data, by = "region") |>
  ggplot() +
  geom_sf(aes(fill = p), colour = "white") +
  facet_grid(hemi ~ view) +
  theme_void()
```

See `vignette("geom-sf")` for more on this approach.

## Troubleshooting

**Regions don't show up.** Check spelling and case. `ggseg.formats::atlas_regions(dk())`
gives you the exact strings the atlas expects.

**Data lands on both hemispheres.** Add a `hemi` column with `"left"` or
`"right"` to constrain the match.

**Extra facet panels appear.** This is handled automatically by
`geom_brain()`. If you're using the `brain_join()` + `geom_sf()` workflow
directly, `group_by()` your data by the faceting variable before joining.

---
title: "Reading FreeSurfer stats files"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Reading FreeSurfer stats files}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r}
#| label: setup
#| include: false
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)
```

FreeSurfer's `recon-all` produces stats files with region-level measures --
cortical thickness, surface area, volume. ggseg includes readers that pull
these files directly into data frames shaped for plotting.

All three functions below come from the `ggseg.formats` package, which ggseg
re-exports.

```{r}
#| label: load-packages
library(ggseg)
library(ggplot2)
library(dplyr)
```

## Reading a single stats file

After `recon-all` finishes, each subject has a `stats/` folder with
parcellation data. `read_freesurfer_stats()` reads one file at a time:

```{r}
#| label: read-single-stats
subjects_dir <- Sys.getenv("SUBJECTS_DIR")
stats_file <- file.path(subjects_dir, "bert/stats/lh.aparc.stats")

data <- read_freesurfer_stats(stats_file)
data
```

The hemisphere is encoded in the filename (`lh.` or `rh.`). To match the
atlas, prepend it to the label column:

```{r}
#| label: fig-single-stats
#| fig-cap: "Brain plot of average cortical thickness from a single stats file."
ggplot() +
  geom_brain(
    atlas = dk(),
    data = data |> mutate(label = paste0("lh_", label)),
    mapping = aes(fill = ThickAvg)
  )
```

## Reading stats files across subjects

`read_atlas_files()` reads all matching stats files from a subjects
directory. Pass a regular expression to select the right files:

```{r}
#| label: read-atlas-files
dat <- read_atlas_files(subjects_dir, "aparc.stats$")
dat
```

The trailing `$` matters -- it ensures you get `aparc.stats` but not
`aparc.a2009s.stats`.

The function adds hemisphere prefixes automatically, so the data is ready
for plotting:

```{r}
#| label: fig-atlas-files
#| fig-cap: "Brain plot of cortical thickness variability across subjects."
ggplot() +
  geom_brain(atlas = dk(), data = dat, mapping = aes(fill = ThickStd))
```

To show all metrics at once, pivot and facet:

```{r}
#| label: fig-all-metrics
#| fig-cap: "All FreeSurfer metrics shown in faceted brain plots."
#| fig-width: 10
library(tidyr)

long <- dat |>
  pivot_longer(-c(subject, label), names_to = "stat", values_to = "val")

ggplot() +
  geom_brain(
    atlas = dk(),
    data = group_by(long, stat),
    mapping = aes(fill = val)
  ) +
  facet_wrap(~stat)
```

## Reading FreeSurfer stats tables

FreeSurfer's `aparcstats2table` and `asegstats2table` commands create
summary tables across subjects. Read these with `read_freesurfer_table()`:

```{r}
#| label: read-fs-table
table_path <- "path/to/aparc.volume.table"

read_freesurfer_table(table_path)
```

The output has three columns: subject, label, and value.

## Cleaning label suffixes

Stats tables append the measure name to each label
(e.g. `lh_bankssts_volume`). The `measure` argument strips that suffix and
renames the value column:

```{r}
#| label: read-fs-table-measure
dat <- read_freesurfer_table(table_path, measure = "volume")
dat
```

## Filtering non-region rows

FreeSurfer tables include summary measures (total volume, ICV) that don't
correspond to atlas regions. Filter them before plotting:

```{r}
#| label: fig-filtered-table
#| fig-cap: "Brain plot of volume data after filtering non-region rows."
ggplot() +
  geom_brain(
    atlas = dk(),
    data = dat |> filter(grepl("lh|rh", label)),
    mapping = aes(fill = volume)
  )
```

The `grepl("lh|rh", label)` pattern keeps only hemisphere-specific rows.

---
title: "Plotting external data"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Plotting external data}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r}
#| label: setup
#| include: false
knitr::opts_chunk$set(eval = TRUE, fig.retina = 3, fig.width = 6)
```

Most of the time, you're not plotting empty atlases. You have results --
p-values, cortical thickness, whatever -- and you want them on a brain. This
vignette covers how to get your data into the right shape for ggseg.

```{r}
#| label: load-packages
library(ggseg)
library(dplyr)
library(ggplot2)
```

## How matching works

`geom_brain()` joins your data to the atlas by `region` (and by
`hemi` too, when both your data and the atlas carry it). That means your data
needs at least one column with names that match the atlas. The two columns
you'll use most:

- **region** -- human-readable names like "insula" or "precentral"
- **label** -- FreeSurfer labels like "lh_bankssts"

Check what's available:

```{r}
#| label: dk-regions
ggseg.formats::atlas_regions(dk())
```

```{r}
#| label: dk-labels
ggseg.formats::atlas_labels(dk())
```

Names must match exactly, including case and spacing.

## A minimal example

Three regions, three p-values:

```{r}
#| label: minimal-data
some_data <- tibble(
  region = c("superior temporal", "precentral", "lateral orbitofrontal"),
  p = c(0.03, 0.6, 0.05)
)
some_data
```

Pass the data to `geom_brain()` through its `data` argument and map
`fill` to your variable:

```{r}
#| label: fig-minimal-plot
#| fig-cap: "Brain plot with three regions coloured by p-value."
ggplot() +
  geom_brain(atlas = dk(), data = some_data, mapping = aes(fill = p))
```

Regions not in your data appear as `NA` (grey by default). Regions in your
data that don't match the atlas are silently dropped, so watch your spelling.

## Constraining matches with extra columns

If your data is hemisphere-specific, add a `hemi` column. The join will use
both `region` and `hemi`, so values only land on the correct side:

```{r}
#| label: fig-hemi-constraint
#| fig-cap: "Brain plot restricted to the left hemisphere using a hemi column."
some_data$hemi <- "left"

ggplot() +
  geom_brain(atlas = dk(), data = some_data, mapping = aes(fill = p))
```

The same works for any atlas column -- adding `view`, for instance, would
restrict matches to specific views.

## Faceting across groups

If your data has a grouping variable, group by it and `facet_wrap()` /
`facet_grid()` work as you'd expect. `geom_brain()` replicates the
full atlas -- context regions included -- in each panel:

```{r}
#| label: fig-facet-groups
#| fig-cap: "Brain plots faceted by age group with a custom colour gradient."
some_data <- tibble(
  region = rep(
    c(
      "transverse temporal",
      "insula",
      "precentral",
      "superior parietal"
    ),
    2
  ),
  p = sample(seq(0, 0.5, 0.001), 8),
  group = c(rep("Young", 4), rep("Old", 4))
)

ggplot() +
  geom_brain(
    atlas = dk(),
    data = group_by(some_data, group),
    colour = "white",
    mapping = aes(fill = p)
  ) +
  facet_wrap(~group, ncol = 1) +
  theme(legend.position = "bottom") +
  scale_fill_gradientn(
    colours = c("royalblue", "firebrick", "goldenrod"),
    na.value = "grey"
  )
```

Grouping the data is what tells the geom how many copies of the atlas to
make -- one per group.

## Troubleshooting

**Regions don't show up.** Check spelling and case. `ggseg.formats::atlas_regions(dk())`
gives you the exact strings the atlas expects.

**Data lands on both hemispheres.** Add a `hemi` column with `"left"` or
`"right"` to constrain the match.

**A facet panel is missing context.** Group your data by the faceting
variable before plotting (`data = my_data |> group_by(group)`) so the full
atlas is replicated in every panel.

## Full control with sf

When you need to layer brain data with other sf geoms, or join the atlas
manually before plotting, work with the atlas as an sf object instead. See
`vignette("geom-sf")` for that workflow.

---
title: "Reading FreeSurfer stats files"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Reading FreeSurfer stats files}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r}
#| label: setup
#| include: false
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)
```

FreeSurfer's `recon-all` produces stats files with region-level measures --
cortical thickness, surface area, volume. ggseg includes readers that pull
these files directly into data frames shaped for plotting.

All three functions below come from the `ggseg.formats` package, which ggseg
re-exports.

```{r}
#| label: load-packages
library(ggseg)
library(ggplot2)
library(dplyr)
```

## Reading a single stats file

After `recon-all` finishes, each subject has a `stats/` folder with
parcellation data. `read_freesurfer_stats()` reads one file at a time:

```{r}
#| label: read-single-stats
subjects_dir <- Sys.getenv("SUBJECTS_DIR")
stats_file <- file.path(subjects_dir, "bert/stats/lh.aparc.stats")

data <- read_freesurfer_stats(stats_file)
data
```

The hemisphere is encoded in the filename (`lh.` or `rh.`). To match the
atlas, prepend it to the label column:

```{r}
#| label: fig-single-stats
#| fig-cap: "Brain plot of average cortical thickness from a single stats file."
ggplot(data |> mutate(label = paste0("lh_", label))) +
  geom_brain(atlas = dk(), mapping = aes(fill = ThickAvg))
```

## Reading stats files across subjects

`read_atlas_files()` reads all matching stats files from a subjects
directory. Pass a regular expression to select the right files:

```{r}
#| label: read-atlas-files
dat <- read_atlas_files(subjects_dir, "aparc.stats$")
dat
```

The trailing `$` matters -- it ensures you get `aparc.stats` but not
`aparc.a2009s.stats`.

The function adds hemisphere prefixes automatically, so the data is ready
for plotting:

```{r}
#| label: fig-atlas-files
#| fig-cap: "Brain plot of cortical thickness variability across subjects."
ggplot(dat) +
  geom_brain(atlas = dk(), mapping = aes(fill = ThickStd))
```

To show all metrics at once, pivot and facet:

```{r}
#| label: fig-all-metrics
#| fig-cap: "All FreeSurfer metrics shown in faceted brain plots."
#| fig-width: 10
library(tidyr)

dat |>
  pivot_longer(-c(subject, label), names_to = "stat", values_to = "val") |>
  ggplot() +
  geom_brain(atlas = dk(), mapping = aes(fill = val)) +
  facet_wrap(~stat)
```

## Reading FreeSurfer stats tables

FreeSurfer's `aparcstats2table` and `asegstats2table` commands create
summary tables across subjects. Read these with `read_freesurfer_table()`:

```{r}
#| label: read-fs-table
table_path <- "path/to/aparc.volume.table"

read_freesurfer_table(table_path)
```

The output has three columns: subject, label, and value.

## Cleaning label suffixes

Stats tables append the measure name to each label
(e.g. `lh_bankssts_volume`). The `measure` argument strips that suffix and
renames the value column:

```{r}
#| label: read-fs-table-measure
dat <- read_freesurfer_table(table_path, measure = "volume")
dat
```

## Filtering non-region rows

FreeSurfer tables include summary measures (total volume, ICV) that don't
correspond to atlas regions. Filter them before plotting:

```{r}
#| label: fig-filtered-table
#| fig-cap: "Brain plot of volume data after filtering non-region rows."
ggplot(dat |> filter(grepl("lh|rh", label))) +
  geom_brain(atlas = dk(), mapping = aes(fill = volume))
```

The `grepl("lh|rh", label)` pattern keeps only hemisphere-specific rows.

---
title: "Getting started with ggseg"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Getting started with ggseg}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r}
#| label: setup
#| include: false
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 6
)
```

If you work with neuroimaging data, you've probably spent time wrestling
region-level results into brain plots. ggseg handles that plumbing for you.
It stores brain atlas geometries as simple features and plots them as ggplot2
layers, so you get brain figures with the same code you'd use for any other
ggplot.

```{r}
#| label: load-packages
library(ggseg)
library(ggplot2)
```

## What's in a brain atlas

A `brain_atlas` object bundles four things: a name, a type (cortical,
subcortical, or tract), region geometry stored as simple features, and an
optional colour palette.

```{r}
#| label: dk-atlas
dk
```

The `dk` atlas (Desikan-Killiany) ships with the package. Call `plot()` for a
quick look:

```{r}
#| label: fig-dk-plot
#| fig-cap: "Quick overview of the Desikan-Killiany cortical atlas."
plot(dk())
```

Two helpers pull out the names you'll need for matching data:

```{r}
#| label: dk-names
ggseg.formats::atlas_regions(dk())
ggseg.formats::atlas_labels(dk())
```

## Your first brain plot

`geom_brain()` works like any ggplot2 geom. Pass it an atlas and it draws
the regions:

```{r}
#| label: fig-basic-brain
#| fig-cap: "Default brain plot using geom_brain() with the dk atlas."
ggplot() +
  geom_brain(atlas = dk())
```

## Arranging views

Brain atlases have multiple views (lateral, medial) and hemispheres.
`position_brain()` controls how they're laid out, using formula syntax
borrowed from `facet_grid()`:

```{r}
#| label: fig-position-formula
#| fig-cap: "Brain views arranged using formula syntax in position_brain()."
ggplot() +
  geom_brain(atlas = dk(), position = position_brain(hemi ~ view))
```

Or reorder views with a character vector:

```{r}
#| label: fig-position-character
#| fig-cap: "Brain views arranged using a character vector to specify order."
ggplot() +
  geom_brain(
    atlas = dk(),
    position = position_brain(c(
      "right lateral",
      "right medial",
      "left lateral",
      "left medial"
    ))
  )
```

See `vignette("positioning-views")` for the full set of layout options,
including grid layouts for subcortical and tract atlases.

## Showing only certain hemispheres or views

```{r}
#| label: fig-filter-lateral
#| fig-cap: "Brain plot showing only lateral views."
ggplot() +
  geom_brain(atlas = dk(), view = "lateral")
```

```{r}
#| label: fig-filter-left
#| fig-cap: "Brain plot showing only the left hemisphere."
ggplot() +
  geom_brain(atlas = dk(), hemi = "left")
```

For subcortical atlases, views are slice identifiers. Check what's available
with `ggseg.formats::atlas_views()`:

```{r}
#| label: aseg-views
ggseg.formats::atlas_views(aseg())
```

## Plotting your own data

Create a data frame with at least one column that matches the atlas
(typically `region` or `label`). `geom_brain()` joins your data to the atlas
automatically:

```{r}
#| label: fig-external-data
#| fig-cap: "Brain plot coloured by external p-values using a viridis scale."
library(dplyr)

some_data <- tibble(
  region = c(
    "transverse temporal",
    "insula",
    "precentral",
    "superior parietal"
  ),
  p = sample(seq(0, 0.5, 0.001), 4)
)

ggplot(some_data) +
  geom_brain(
    atlas = dk(),
    position = position_brain(hemi ~ view),
    aes(fill = p)
  ) +
  scale_fill_viridis_c(option = "cividis", direction = -1) +
  theme_void()
```

See `vignette("external-data")` for more on data preparation and matching.

## Faceting

ggplot2 faceting works as you'd expect. `geom_brain()` detects faceting
variables and replicates the full atlas in each panel:

```{r}
#| label: fig-faceting
#| fig-cap: "Brain plots faceted by group, each panel showing the full atlas."
some_data <- tibble(
  region = rep(
    c(
      "transverse temporal",
      "insula",
      "precentral",
      "superior parietal"
    ),
    2
  ),
  p = sample(seq(0, 0.5, 0.001), 8),
  group = c(rep("A", 4), rep("B", 4))
)

ggplot(some_data) +
  geom_brain(
    atlas = dk(),
    position = position_brain(hemi ~ view),
    aes(fill = p)
  ) +
  facet_wrap(~group)
```

No `group_by()` needed -- the geom handles it.

## Highlighting specific regions

To colour a few regions with their atlas colours, use two columns: one for
the join (`region`) and one for the fill aesthetic:

```{r}
#| label: fig-highlighting
#| fig-cap: "Highlighting selected regions using the atlas colour palette."
data <- data.frame(
  region = ggseg.formats::atlas_regions(dk())[1:3],
  reg_col = ggseg.formats::atlas_regions(dk())[1:3]
)

ggplot(data) +
  geom_brain(atlas = dk(), aes(fill = reg_col)) +
  scale_fill_brain_manual(dk()$palette[data$region])
```

## Scales and themes

Atlas colour palettes are applied automatically by `geom_brain()`. For
custom palettes, use `scale_fill_brain_manual()`.

Built-in themes strip axes and grids for cleaner figures:

```{r}
#| label: themes-example
#| eval: false
ggplot() +
  geom_brain(atlas = dk()) +
  theme_brain()
```

## Finding more atlases

ggseg ships with `dk` (cortical), `aseg` (subcortical), and `tracula`
(white matter tracts). Many more are available through the
[ggsegverse r-universe](https://ggsegverse.r-universe.dev):

```{r}
#| label: install-extra
#| eval: false
install.packages("ggsegYeo2011", repos = "https://ggsegverse.r-universe.dev")
```

---
title: "Arranging brain views"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Arranging brain views}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r}
#| label: setup
#| include: false
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)
```

```{r}
#| label: load-packages
library(ggseg)
library(ggplot2)
```

A brain atlas typically has several views -- lateral and medial for cortical
atlases, or axial, coronal, and sagittal slices for subcortical and tract
atlases. `position_brain()` controls how those views are arranged in the
final plot.

The function works differently depending on the atlas type, so this vignette
covers cortical and subcortical/tract atlases separately.

## Cortical atlases

Cortical atlases like `dk` have two layout dimensions: **hemi** (left or
right) and **view** (lateral, medial, etc.). The formula syntax mirrors
`facet_grid()` -- left side is rows, right side is columns:

```{r}
#| label: fig-hemi-by-view
#| fig-cap: "Cortical atlas with hemispheres as rows and views as columns."
ggplot() +
  geom_brain(
    atlas = dk(),
    position = position_brain(hemi ~ view),
    show.legend = FALSE
  ) +
  theme_void()
```

Flip the formula to transpose the layout:

```{r}
#| label: fig-view-by-hemi
#| fig-cap: "Transposed layout with views as rows and hemispheres as columns."
ggplot() +
  geom_brain(
    atlas = dk(),
    position = position_brain(view ~ hemi),
    show.legend = FALSE
  ) +
  theme_void()
```

### Stacking all views

Use `.` with `+` to collapse everything into a single row or column. This
is handy for compact figures:

```{r}
#| label: fig-single-row
#| fig-cap: "All brain views stacked in a single row."
ggplot() +
  geom_brain(
    atlas = dk(),
    position = position_brain(. ~ hemi + view),
    show.legend = FALSE
  ) +
  theme_void()
```

```{r}
#| label: fig-single-column
#| fig-cap: "All brain views stacked in a single column."
ggplot() +
  geom_brain(
    atlas = dk(),
    position = position_brain(hemi + view ~ .),
    show.legend = FALSE
  ) +
  theme_void()
```

## Subcortical and tract atlases

Subcortical atlases like `aseg` and tract atlases like `tracula` don't have
the hemisphere/view pairing that cortical atlases do. Their views are
individual slices (e.g. `"axial_3"`, `"sagittal"`). That opens up a
different set of positioning tools.

### Horizontal and vertical

The simplest options. `"horizontal"` is the default:

```{r}
#| label: fig-aseg-horizontal
#| fig-cap: "Subcortical atlas views arranged horizontally."
ggplot() +
  geom_brain(
    atlas = aseg(),
    position = position_brain("horizontal"),
    show.legend = FALSE
  ) +
  theme_void()
```

```{r}
#| label: fig-aseg-vertical
#| fig-cap: "Subcortical atlas views arranged vertically."
ggplot() +
  geom_brain(
    atlas = aseg(),
    position = position_brain("vertical"),
    show.legend = FALSE
  ) +
  theme_void()
```

### Grid layouts

When you have many views, a grid keeps things readable. Specify `nrow`,
`ncol`, or both:

```{r}
#| label: fig-aseg-nrow
#| fig-cap: "Subcortical atlas views in a two-row grid."
ggplot() +
  geom_brain(
    atlas = aseg(),
    position = position_brain(nrow = 2),
    show.legend = FALSE
  ) +
  theme_void()
```

```{r}
#| label: fig-aseg-ncol
#| fig-cap: "Subcortical atlas views in a three-column grid."
ggplot() +
  geom_brain(
    atlas = aseg(),
    position = position_brain(ncol = 3),
    show.legend = FALSE
  ) +
  theme_void()
```

### Picking specific views

The `views` parameter lets you select which views to include and in what
order. Check what's available with `ggseg.formats::atlas_views()`:

```{r}
#| label: aseg-views
ggseg.formats::atlas_views(aseg())
```

```{r}
#| label: fig-aseg-select-views
#| fig-cap: "A subset of subcortical views selected by name."
ggplot() +
  geom_brain(
    atlas = aseg(),
    position = position_brain(
      views = c("sagittal", "axial_3", "coronal_3")
    ),
    show.legend = FALSE
  ) +
  theme_void()
```

Combine `views` with `nrow` or `ncol` for a custom grid:

```{r}
#| label: fig-aseg-custom-grid
#| fig-cap: "Custom two-row grid with selected subcortical views."
ggplot() +
  geom_brain(
    atlas = aseg(),
    position = position_brain(
      views = c("sagittal", "axial_3", "axial_5", "coronal_3"),
      nrow = 2
    ),
    show.legend = FALSE
  ) +
  theme_void()
```

### Grouping by slice type

The `type ~ .` formula groups views by their orientation -- all axial slices
together, all coronal slices together, and so on. The type is extracted from
the view name (everything before the first underscore):

```{r}
#| label: aseg-type-group
#| eval: false
ggplot() +
  geom_brain(
    atlas = aseg(),
    position = position_brain(type ~ .),
    show.legend = FALSE
  ) +
  theme_void()
```

## Pre-processing with reposition_brain()

If you're using the `geom_sf()` workflow instead of `geom_brain()`, use
`reposition_brain()` to rearrange the data before plotting. It takes the
same arguments:

```{r}
#| label: fig-reposition-brain
#| fig-cap: "Repositioned subcortical data plotted with geom_sf()."
atlas_data <- as.data.frame(aseg())

repositioned <- reposition_brain(
  atlas_data,
  views = c("sagittal", "axial_3", "coronal_3"),
  nrow = 1
)

ggplot(repositioned) +
  geom_sf(aes(fill = region), show.legend = FALSE) +
  theme_void()
```

## Adding view labels

Use `annotate_brain()` to label each view with its name. For cortical
atlases the label combines hemisphere and view (e.g. "left lateral"); for
subcortical and tract atlases it uses the view name directly.

Store the `position_brain()` specification in an object so both layers share
the same layout:

```{r}
#| label: fig-dk-view-labels
#| fig-cap: "Cortical atlas with view labels."
pos <- position_brain(hemi ~ view)
ggplot() +
  geom_brain(atlas = dk(), position = pos, show.legend = FALSE) +
  annotate_brain(atlas = dk(), position = pos) +
  theme_void()
```

It works with any positioning — horizontal, vertical, grid, and formula
layouts:

```{r}
#| label: fig-aseg-view-labels
#| fig-cap: "Subcortical atlas with view labels in a two-row grid."
pos <- position_brain(nrow = 2)
ggplot() +
  geom_brain(atlas = aseg(), position = pos, show.legend = FALSE) +
  annotate_brain(atlas = aseg(), position = pos) +
  theme_void()
```

Text appearance is customisable through standard `annotate()` arguments:

```{r}
#| label: fig-styled-labels
#| fig-cap: "View labels with custom styling."
ggplot() +
  geom_brain(atlas = dk(), show.legend = FALSE) +
  annotate_brain(
    atlas = dk(),
    size = 2.5,
    colour = "grey50",
    fontface = "italic"
  ) +
  theme_void()
```

## Quick reference

| Atlas type | Options |
|---|---|
| Cortical | `hemi ~ view`, `view ~ hemi`, `. ~ hemi + view`, `hemi + view ~ .` |
| Subcortical / Tract | `"horizontal"`, `"vertical"`, `nrow`, `ncol`, `views`, `type ~ .` |

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom-brain.R
\docType{data}
\name{geom_brain}
\alias{geom_brain}
\alias{GeomBrain}
\title{Plot brain atlas regions}
\usage{
geom_brain(
  mapping = aes(),
  data = NULL,
  atlas,
  hemi = NULL,
  view = NULL,
  position = position_brain(),
  context = TRUE,
  show.legend = NA,
  inherit.aes = TRUE,
  ...
)
}
\arguments{
\item{mapping}{Set of aesthetic mappings created by [ggplot2::aes()].}

\item{data}{A data.frame containing variables to map. If `NULL`, the atlas
is plotted without user data. Group it with [dplyr::group_by()] to facet.}

\item{atlas}{A `ggseg_atlas` object (e.g. `dk()`, `aseg()`, `tracula()`).}

\item{hemi}{Character vector of hemispheres to include (e.g. `"left"`,
`"right"`). Defaults to all hemispheres in the atlas.}

\item{view}{Character vector of views to include, as recorded in the atlas
data. For cortical atlases: `"lateral"`, `"medial"`. For subcortical/tract
atlases: slice identifiers like `"axial_3"`. Defaults to all views.}

\item{position}{Position adjustment, either as a string or the result of
a call to [position_brain()].}

\item{context}{Keep the rest of the brain as a soft grey backdrop (`TRUE`,
the default), or show only the regions you're plotting (`FALSE`).}

\item{show.legend}{Logical. Should this layer be included in the legends?}

\item{inherit.aes}{Logical. If `FALSE`, overrides the default aesthetics
rather than combining with them.}

\item{...}{Additional arguments passed to [ggplot2::geom_polygon()].}
}
\value{
A list of ggplot2 layer and coord objects.
}
\description{
Colour brain regions by your own values. Give `geom_brain()` an atlas like
`dk()` and a data frame, and it matches your values to the right regions and
lays out the brain views for you. No data? It just draws the atlas.
}
\section{GeomBrain ggproto}{

`GeomBrain` is a [ggplot2::Geom] ggproto object that handles rendering
of brain atlas polygons. It is used internally by [geom_brain()] and
should not typically be called directly.
}

\examples{
library(ggplot2)

ggplot() +
  geom_brain(atlas = dk())
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggseg-package.R, R/ggseg.R
\docType{package}
\name{ggseg}
\alias{ggseg-package}
\alias{ggseg}
\title{ggseg: Visualise brain atlas data with ggplot2}
\usage{
ggseg(...)
}
\arguments{
\item{...}{Ignored.}
}
\value{
Does not return; always raises an error.
}
\description{
Provides a ggplot2 geom and position for visualising brain region data on
cortical, subcortical, and white matter tract atlases. Brain atlas
geometries are stored as simple features (sf), enabling seamless
integration with the ggplot2 ecosystem including faceting, custom scales,
and themes.

`ggseg()` is defunct as of version 2.0.0. Use
`ggplot() + geom_brain()` instead.
}
\details{
The main entry point is [geom_brain()], which accepts a `ggseg_atlas`
object and optional user data. Use [position_brain()] to control the
layout of brain slices/views.
}
\examples{
\dontrun{
ggseg()
}
}
\seealso{
Useful links:
\itemize{
  \item \url{https://ggsegverse.github.io/ggseg/}
  \item \url{https://github.com/ggsegverse/ggseg}
  \item Report bugs at \url{https://github.com/ggsegverse/ggseg/issues}
}


[geom_brain()] for the replacement API.
}
\author{
\strong{Maintainer}: Athanasia Mo Mowinckel \email{a.m.mowinckel@psykologi.uio.no} (\href{https://orcid.org/0000-0002-5756-0223}{ORCID})

Authors:
\itemize{
  \item Didac Vidal-Piñeiro \email{d.v.pineiro@psykologi.uio.no} (\href{https://orcid.org/0000-0001-9997-9156}{ORCID})
}

Other contributors:
\itemize{
  \item Ramiro Magno \email{ramiro.magno@gmail.com} (\href{https://orcid.org/0000-0001-5226-3441}{ORCID}) [contributor]
  \item Center for Lifespan Changes in Brain and Cognition, University of Oslo, Norway [copyright holder]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/position-brain.R
\name{position_brain}
\alias{position_brain}
\title{Arrange brain atlas views}
\usage{
position_brain(
  position = "horizontal",
  nrow = NULL,
  ncol = NULL,
  views = NULL,
  zoom = NULL,
  zoom_pad = 0.05
)
}
\arguments{
\item{position}{Formula describing the rows ~ columns organisation for
cortical atlases (e.g., `hemi ~ view`). For subcortical/tract atlases,
can be "horizontal", "vertical", or a formula with `type ~ .` where type
is extracted from view names like "axial_1" -> "axial".}

\item{nrow}{Number of rows for grid layout. If NULL (default), calculated
automatically. Only used for subcortical/tract atlases when position is
not a formula.}

\item{ncol}{Number of columns for grid layout. If NULL (default), calculated
automatically. Only used for subcortical/tract atlases when position is
not a formula.}

\item{views}{Character vector specifying which views to include and their
order. If NULL (default), all views are included in their original order.
Only applies to subcortical/tract atlases.}

\item{zoom}{Controls per-view zoom. `NULL`/`FALSE` (default) draws each view
at full extent. `TRUE` zooms each view onto its focus regions; a character
vector names the focus regions explicitly.}

\item{zoom_pad}{Fractional padding added around the focus window when `zoom`
is active. Defaults to `0.05` (5\%).}
}
\value{
A layout specification to hand to [geom_brain()]'s `position`
  argument.
}
\description{
Controls how an atlas's hemispheres and views are arranged in the plot --
side by side, stacked, or in a grid -- and can zoom each view in on the
regions you care about. Pass the result to the `position` argument of
[geom_brain()] (or [annotate_brain()]).
}
\examples{
library(ggplot2)

# Cortical atlas with formula
ggplot() +
  geom_brain(
    atlas = dk(), aes(fill = region),
    position = position_brain(. ~ view + hemi),
    show.legend = FALSE
  )

ggplot() +
  geom_brain(
    atlas = dk(), aes(fill = region),
    position = position_brain(view ~ hemi),
    show.legend = FALSE
  )

ggplot() +
  geom_brain(
    atlas = aseg(), aes(fill = region),
    position = position_brain(nrow = 2)
  )

ggplot() +
  geom_brain(
    atlas = aseg(), aes(fill = region),
    position = position_brain(
      views = c("sagittal", "axial_3", "coronal_2"),
      nrow = 1
    )
  )

ggplot() +
  geom_brain(
    atlas = aseg(), aes(fill = region),
    position = position_brain(type ~ .)
  )
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom-brain.R
\name{geom_brain_sf}
\alias{geom_brain_sf}
\title{Deprecated sf brain geom}
\usage{
geom_brain_sf(
  mapping = aes(),
  data = NULL,
  atlas,
  hemi = NULL,
  view = NULL,
  position = NULL,
  show.legend = NA,
  inherit.aes = TRUE,
  ...
)
}
\arguments{
\item{mapping}{Set of aesthetic mappings created by [ggplot2::aes()].}

\item{data}{A data.frame containing variables to map. If `NULL`, the atlas
is plotted without user data. Group it with [dplyr::group_by()] to facet.}

\item{atlas}{A `ggseg_atlas` object (e.g. `dk()`, `aseg()`, `tracula()`).}

\item{hemi}{Character vector of hemispheres to include (e.g. `"left"`,
`"right"`). Defaults to all hemispheres in the atlas.}

\item{view}{Character vector of views to include, as recorded in the atlas
data. For cortical atlases: `"lateral"`, `"medial"`. For subcortical/tract
atlases: slice identifiers like `"axial_3"`. Defaults to all views.}

\item{position}{Position adjustment, either as a string or the result of
a call to [position_brain()].}

\item{show.legend}{Logical. Should this layer be included in the legends?}

\item{inherit.aes}{Logical. If `FALSE`, overrides the default aesthetics
rather than combining with them.}

\item{...}{Additional arguments passed to [ggplot2::geom_polygon()].}
}
\value{
A list of ggplot2 layer and coord objects.
}
\description{
`r lifecycle::badge("deprecated")`

The sf rendering path is deprecated. `geom_brain_sf()` renders an atlas via
[ggplot2::geom_sf()] and [coord_sf()][ggplot2::coord_sf]. For new code, use
[geom_brain()] (the polygon default), or convert the atlas with
`as_sf_atlas()` and use [ggplot2::geom_sf()] directly for the full sf
toolkit (labels, other sf layers).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/position-brain.R
\name{position_brain_sf}
\alias{position_brain_sf}
\title{Deprecated sf brain-view layout}
\usage{
position_brain_sf(
  position = "horizontal",
  nrow = NULL,
  ncol = NULL,
  views = NULL
)
}
\arguments{
\item{position}{Formula describing the rows ~ columns organisation for
cortical atlases (e.g., `hemi ~ view`). For subcortical/tract atlases,
can be "horizontal", "vertical", or a formula with `type ~ .` where type
is extracted from view names like "axial_1" -> "axial".}

\item{nrow}{Number of rows for grid layout. If NULL (default), calculated
automatically. Only used for subcortical/tract atlases when position is
not a formula.}

\item{ncol}{Number of columns for grid layout. If NULL (default), calculated
automatically. Only used for subcortical/tract atlases when position is
not a formula.}

\item{views}{Character vector specifying which views to include and their
order. If NULL (default), all views are included in their original order.
Only applies to subcortical/tract atlases.}
}
\value{
A `PositionBrain` ggproto object.
}
\description{
`r lifecycle::badge("deprecated")`

The sf rendering path is deprecated. `position_brain_sf()` returns the
legacy `PositionBrain` ggproto for use with [geom_brain_sf()]. For new
code, use [position_brain()] (the polygon default), or convert the atlas
with `as_sf_atlas()` and use [ggplot2::geom_sf()] directly.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/position-brain.R
\name{reposition_brain}
\alias{reposition_brain}
\title{Reposition brain slices}
\usage{
reposition_brain(
  data,
  position = "horizontal",
  nrow = NULL,
  ncol = NULL,
  views = NULL
)
}
\arguments{
\item{data}{sf-data.frame of joined brain atlas and data}

\item{position}{Position formula for slices. For cortical atlases, use
formulas like `hemi ~ view`. For subcortical/tract atlases, use
"horizontal", "vertical", or `type ~ .` for type-based layout.}

\item{nrow}{Number of rows for grid layout (subcortical/tract only)}

\item{ncol}{Number of columns for grid layout (subcortical/tract only)}

\item{views}{Character vector specifying view order (subcortical/tract only)}
}
\value{
sf-data.frame with re-positioned slices
}
\description{
Repositions pre-joined sf atlas data (i.e. data and atlas already joined to
a single sf data frame) for control over final plot layout. For even more
detailed control, convert the "hemi" and "view" columns into factors ordered
by wanted order of appearance.
}
\details{
This is the sf layout helper. It requires the `sf` package (an optional
dependency); for the sf-free default, build a layout with [position_brain()]
and pass it to [geom_brain()].
}
\examples{
\dontshow{if (requireNamespace("sf", quietly = TRUE)) withAutoprint(\{ # examplesIf}
reposition_brain(dk(), hemi ~ view)
reposition_brain(dk(), view ~ hemi)
reposition_brain(dk(), hemi + view ~ .)
reposition_brain(dk(), . ~ hemi + view)

\donttest{
reposition_brain(aseg(), nrow = 2)
reposition_brain(aseg(), views = c("sagittal", "axial_3"))
}
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scale-brain.R
\name{scale_brain2}
\alias{scale_brain2}
\alias{scale_colour_brain2}
\alias{scale_color_brain2}
\alias{scale_fill_brain2}
\alias{scale_brain2-deprecated}
\title{Deprecated scale functions}
\usage{
scale_brain2(...)

scale_colour_brain2(...)

scale_color_brain2(...)

scale_fill_brain2(...)
}
\arguments{
\item{...}{Arguments passed to the replacement function.}
}
\value{
A ggplot2 scale object.
}
\description{
`r lifecycle::badge("deprecated")`

These functions have been renamed for clarity:
- `scale_brain2()` -> [scale_brain_manual()]
- `scale_fill_brain2()` -> [scale_fill_brain_manual()]
- `scale_colour_brain2()` -> [scale_colour_brain_manual()]
- `scale_color_brain2()` -> [scale_color_brain_manual()]
}
\examples{
pal <- c("transverse temporal" = "#FF0000", "insula" = "#00FF00")
suppressWarnings(scale_fill_brain_manual(palette = pal))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme-brain.R
\name{theme_brain}
\alias{theme_brain}
\alias{theme_darkbrain}
\alias{theme_custombrain}
\alias{theme_brain2}
\title{Themes for brain atlas plots}
\usage{
theme_brain(text.size = 12, text.family = "mono")

theme_darkbrain(text.size = 12, text.family = "mono")

theme_custombrain(
  plot.background = "white",
  text.colour = "darkgrey",
  text.size = 12,
  text.family = "mono"
)

theme_brain2(
  plot.background = "white",
  text.colour = "darkgrey",
  text.size = 12,
  text.family = "mono"
)
}
\arguments{
\item{text.size}{Text size in points (default: `12`).}

\item{text.family}{Font family (default: `"mono"`).}

\item{plot.background}{Background fill colour (`theme_custombrain` and
`theme_brain2` only).}

\item{text.colour}{Text colour (`theme_custombrain` and `theme_brain2` only).}
}
\value{
A [ggplot2::theme] object.
}
\description{
A set of ggplot2 themes designed for brain atlas visualisations. All themes
remove axis ticks and grid lines for a clean presentation.
}
\details{
\describe{

\item{`theme_brain`}{
Default theme. Transparent background, no axes, no grid.}

\item{`theme_darkbrain`}{
Dark theme with black background and light text.}

\item{`theme_custombrain`}{
Fully customisable background, text colour, size, and font.}

\item{`theme_brain2`}{
Like `theme_custombrain` but with axis text removed entirely.}

}
}
\examples{
library(ggplot2)

p <- ggplot() +
  geom_brain(atlas = dk())

p +
  theme_brain()

p +
  theme_darkbrain()

}
\seealso{
[geom_brain()], [ggplot2::theme()]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scale-brain.R
\name{scale_brain}
\alias{scale_brain}
\alias{scale_colour_brain}
\alias{scale_color_brain}
\alias{scale_fill_brain}
\title{Colour and fill scales from brain atlas palettes}
\usage{
scale_brain(
  name = "dk",
  na.value = "grey",
  ...,
  aesthetics = c("fill", "colour", "color")
)

scale_colour_brain(name = "dk", na.value = "grey", ...)

scale_color_brain(name = "dk", na.value = "grey", ...)

scale_fill_brain(name = "dk", na.value = "grey", ...)
}
\arguments{
\item{name}{String name of the atlas palette (e.g. `"dk"`, `"aseg"`).}

\item{na.value}{Colour for `NA` entries (default: `"grey"`).}

\item{...}{Additional arguments passed to [ggseg.formats::atlas_palette()].}

\item{aesthetics}{Which aesthetic to scale: `"fill"`, `"colour"`, or
`"color"`.}
}
\value{
A ggplot2 scale object.
}
\description{
`r lifecycle::badge("deprecated")`

Atlas palettes are now applied automatically by [geom_brain()].
Use [scale_fill_brain_manual()] for custom palettes.
}
\examples{
\dontrun{
library(ggplot2)
ggplot() +
  geom_brain(atlas = dk(), aes(fill = region), show.legend = FALSE) +
  scale_brain("dk")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/brain_join.R
\name{brain_join}
\alias{brain_join}
\title{Join user data with a brain atlas}
\usage{
brain_join(data, atlas, by = NULL)
}
\arguments{
\item{data}{A data.frame with a column matching an atlas column
(typically `"region"`). Can be grouped with [dplyr::group_by()].}

\item{atlas}{A `ggseg_atlas` object or data.frame containing atlas data.}

\item{by}{Character vector of column names to join by. If `NULL` (default),
columns are detected automatically.}
}
\value{
An `sf` object if the atlas contains geometry, otherwise a tibble.
}
\description{
Matches your data to a brain atlas by a shared column (usually `region`),
keeping every atlas region whether or not you have a value for it. Grouped
data (via [dplyr::group_by()]) gives one complete atlas per group. You
rarely need this directly -- [geom_brain()] joins your data for you; reach
for `brain_join()` when you want the joined sf object yourself.
}
\examples{
\dontshow{if (requireNamespace("sf", quietly = TRUE)) withAutoprint(\{ # examplesIf}
someData <- data.frame(
  region = c(
    "transverse temporal", "insula",
    "precentral", "superior parietal"
  ),
  p = sample(seq(0, .5, .001), 4),
  stringsAsFactors = FALSE
)

brain_join(someData, dk())
brain_join(someData, dk(), "region")
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scale-brain.R
\name{scale_continous_brain}
\alias{scale_continous_brain}
\alias{scale_x_brain}
\alias{scale_y_brain}
\alias{scale_labs_brain}
\title{Axis and label scales for brain atlas plots}
\usage{
scale_continous_brain(
  atlas = dk(),
  position = "dispersed",
  aesthetics = c("y", "x")
)

scale_x_brain(...)

scale_y_brain(...)

scale_labs_brain(atlas = dk(), position = "dispersed", aesthetics = "labs")
}
\arguments{
\item{atlas}{A `ggseg_atlas` object or data.frame containing atlas data.}

\item{position}{Layout style: `"dispersed"` (default) or `"stacked"`.}

\item{aesthetics}{Which axis to scale: `"x"`, `"y"`, or `"labs"`.}

\item{...}{Additional arguments passed to [adapt_scales()].}
}
\value{
A ggplot2 scale or labs object.
}
\description{
Add axis labels and tick labels corresponding to brain atlas regions.
These scales add hemisphere or view labels to the x and y axes based on
the atlas layout.
}
\examples{
\donttest{
library(ggplot2)

ggplot() +
  geom_brain(atlas = dk()) +
  scale_x_brain() +
  scale_y_brain() +
  scale_labs_brain()
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/annotate-brain.R
\name{annotate_brain}
\alias{annotate_brain}
\title{Add view labels to brain atlas plots}
\usage{
annotate_brain(
  atlas,
  position = position_brain(),
  hemi = NULL,
  view = NULL,
  size = 3,
  colour = "grey30",
  family = "mono",
  padding = 0.05,
  nudge_y = 0,
  ...
)
}
\arguments{
\item{atlas}{A `brain_atlas` object (e.g. `dk()`, `aseg()`).}

\item{position}{The same layout you passed to [geom_brain()], from
[position_brain()].}

\item{hemi}{Character vector of hemispheres to include. If `NULL`
(default), all hemispheres are included.}

\item{view}{Character vector of views to include. If `NULL`
(default), all views are included.}

\item{size}{Text size in mm (default: `3`).}

\item{colour}{Text colour (default: `"grey30"`).}

\item{family}{Font family (default: `"mono"`).}

\item{padding}{Vertical gap between each label and its view, as a
fraction of the plot's total height (default: `0.05`). Labels are also
bottom-anchored (`vjust = 0`) so they sit clear of the geometry.}

\item{nudge_y}{Additional absolute vertical offset for labels
(default: `0`).}

\item{...}{Additional arguments passed to [ggplot2::annotate()].}
}
\value{
A ggplot2 annotation layer.
}
\description{
Annotates each brain view with a text label positioned above the
view's bounding box. For cortical atlases, labels show hemisphere
and view (e.g., "left lateral"). For subcortical and tract atlases,
labels show the view name directly (e.g., "axial_1", "sagittal").
}
\details{
Pass the same `position` you gave [geom_brain()] and the labels line up with
the views automatically.
}
\examples{
library(ggplot2)

pos <- position_brain(hemi ~ view)
ggplot() +
  geom_brain(atlas = dk(), position = pos, show.legend = FALSE) +
  annotate_brain(atlas = dk(), position = pos)

ggplot() +
  geom_brain(atlas = dk(), show.legend = FALSE) +
  annotate_brain(atlas = dk())
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coord-funcs.R
\name{coord_brain}
\alias{coord_brain}
\title{Keep brain shapes undistorted}
\usage{
coord_brain(ratio = 1, clip = "off", ...)
}
\arguments{
\item{ratio}{Aspect ratio, expressed as `y / x`. Defaults to `1`, which
keeps brain polygons undistorted.}

\item{clip}{Should drawing be clipped to the panel extent (`"on"`) or
allowed to overflow (`"off"`)? Defaults to `"off"` so region outlines at
the panel edge are not cut.}

\item{...}{Additional arguments passed to [ggplot2::coord_fixed()].}
}
\value{
A ggplot2 coordinate system that registers as a default.
}
\description{
Fixes the aspect ratio so brains aren't stretched by the shape of the
plotting window. [geom_brain()] adds `coord_brain()` for you, so you rarely
need to call it yourself -- reach for it only to adjust the `ratio` or
`clip` behaviour. You can safely add your own coord or stack several
`geom_brain()` layers; it steps aside cleanly.
}
\examples{
library(ggplot2)
\dontrun{
poly <- ggseg.formats::as_polygon_atlas(dk())
# Equivalent to the default; shown explicitly:
ggplot() +
  geom_brain(atlas = poly) +
  coord_brain()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scale-brain.R
\name{scale_brain_manual}
\alias{scale_brain_manual}
\alias{scale_colour_brain_manual}
\alias{scale_color_brain_manual}
\alias{scale_fill_brain_manual}
\title{Manual colour and fill scales for brain plots}
\usage{
scale_brain_manual(
  palette,
  na.value = "grey",
  ...,
  aesthetics = c("fill", "colour", "color")
)

scale_colour_brain_manual(...)

scale_color_brain_manual(...)

scale_fill_brain_manual(...)
}
\arguments{
\item{palette}{Named character vector mapping region names to colours.}

\item{na.value}{Colour for `NA` entries (default: `"grey"`).}

\item{...}{Additional arguments (unused).}

\item{aesthetics}{Which aesthetic to scale: `"fill"`, `"colour"`, or
`"color"`.}
}
\value{
A ggplot2 scale object.
}
\description{
Apply a custom named colour palette to brain atlas plots. Use this
when you want to override the atlas default colours with your own
colour mapping.
}
\examples{
library(ggplot2)

pal <- c("insula" = "red", "precentral" = "blue")
ggplot() +
  geom_brain(atlas = dk(), aes(fill = region), show.legend = FALSE) +
  scale_fill_brain_manual(palette = pal)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggseg-package.R
\docType{import}
\name{reexports}
\alias{reexports}
\alias{dk}
\alias{aseg}
\alias{tracula}
\alias{suit}
\title{Objects exported from other packages}
\keyword{internal}
\description{
These objects are imported from other packages. Follow the links
below to see their documentation.

\describe{
  \item{ggseg.formats}{\code{\link[ggseg.formats]{aseg}}, \code{\link[ggseg.formats]{dk}}, \code{\link[ggseg.formats]{suit}}, \code{\link[ggseg.formats]{tracula}}}
}}
